@@ -1,8 +1,9 @@
 /*
  * about_dialog.h - declaration of class aboutDialog
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -52,8 +53,8 @@ public:
 
 
 protected:
-	void keyPressEvent( QKeyEvent * _ke );
-	void resizeEvent( QResizeEvent * _re );
+	virtual void keyPressEvent( QKeyEvent * _ke );
+	virtual void resizeEvent( QResizeEvent * _re );
 
 
 private:
