@@ -1,8 +1,9 @@
 /*
  * embed.h - misc. stuff for using embedded data (resources linked into binary)
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -42,10 +43,31 @@
 namespace embed
 {
 
+struct descriptor
+{
+	int size;
+	const unsigned char * data;
+	const char * name;
+} ;
+
+
 QPixmap getIconPixmap( const char *  _name, int _w = -1, int _h = -1 );
 QString getText( const char * _name );
 void loadTranslation( const QString & _tname );
 
 }
 
+
+#ifdef PLUGIN_NAME
+namespace PLUGIN_NAME
+{
+
+QPixmap getIconPixmap( const char *  _name, int _w = -1, int _h = -1 );
+QString getText( const char * _name );
+void loadTranslation( const QString & _tname );
+
+}
+#endif
+
+
 #endif
