@@ -1,8 +1,9 @@
 /*
  * export.h - header which is needed for song-export
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -26,10 +27,11 @@
 
 #include "types.h"
 
-class audioDevice;
+
+class audioFileDevice;
 
 
-typedef audioDevice * ( * getDeviceInst)( Uint32 _sample_rate,
+typedef audioFileDevice * ( * getDeviceInst)( Uint32 _sample_rate,
 						Uint32 _channels,
 						bool & _success_ful,
 						const QString & _file,
@@ -47,6 +49,7 @@ enum fileTypes
 } ;
 
 
+
 struct fileEncodeDevice
 {
 	fileTypes m_fileType;
@@ -55,6 +58,7 @@ struct fileEncodeDevice
 	getDeviceInst m_getDevInst;
 } ;
 
+
 extern fileEncodeDevice fileEncodeDevices[];
 
 
