@@ -1,8 +1,9 @@
 /*
  * led_checkbox.h - class ledCheckBox, an improved QCheckBox
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -29,11 +30,11 @@
 
 #ifdef QT4
 
-#include <QCheckBox>
+#include <QWidget>
 
 #else
 
-#include <qcheckbox.h>
+#include <qwidget.h>
 
 #endif
 
@@ -41,8 +42,9 @@
 class QPixmap;
 
 
-class ledCheckBox : public QCheckBox
+class ledCheckBox : public QWidget
 {
+	Q_OBJECT
 public:
 	enum ledColors
 	{
@@ -54,13 +56,36 @@ public:
 	virtual ~ledCheckBox();
 
 
+	inline bool isChecked( void ) const
+	{
+		return( m_checked );
+	}
+
+	inline const QString & text( void )
+	{
+		return( m_text );
+	}
+
+
+public slots:
+	void toggle( void );
+	void setChecked( bool _on );
+
+
 protected:
 	virtual void paintEvent( QPaintEvent * _pe );
+	virtual void mousePressEvent( QMouseEvent * _me );
 
 
 private:
 	QPixmap * m_ledOnPixmap;
 	QPixmap * m_ledOffPixmap;
+	
+	bool m_checked;
+	QString m_text;
+	
+signals:
+	void toggled( bool );
 
 } ;
 
