@@ -1,8 +1,9 @@
 /*
  * text_float.h - class textFloat, a floating text-label
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -30,10 +31,12 @@
 #ifdef QT4
 
 #include <QWidget>
+#include <QPixmap>
 
 #else
 
 #include <qwidget.h>
+#include <qpixmap.h>
 
 #endif
 
@@ -46,19 +49,38 @@ public:
 	{
 	}
 
+	void setTitle( const QString & _title );
 	void setText( const QString & _text );
+	void setPixmap( const QPixmap & _pixmap );
 
 	void reparent( QWidget * _new_parent );
 
 	void setVisibilityTimeOut( int _msecs );
 
 
+	static textFloat * displayMessage( const QString & _msg,
+						int _timeout = 2000,
+						QWidget * _parent = NULL,
+						int _add_y_margin = 0 );
+	static textFloat * displayMessage( const QString & _title,
+						const QString & _msg,
+						const QPixmap & _pixmap =
+								QPixmap(),
+						int _timeout = 2000,
+						QWidget * _parent = NULL );
+
+
 protected:
 	virtual void paintEvent( QPaintEvent * _me );
+	virtual void mousePressEvent( QMouseEvent * _me );
 
 
 private:
+	void updateSize( void );
+
+	QString m_title;
 	QString m_text;
+	QPixmap m_pixmap;
 
 } ;
 
