@@ -2,8 +2,9 @@
  * arp_and_chords_tab_widget.cpp - widget for use in arp/chord-tab of 
  *                                 channel-window
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -38,12 +39,18 @@
 #include <qdom.h>
 #include <qlabel.h>
 #include <qcombobox.h>
+#include <qwhatsthis.h>
 
 #define setChecked setOn
 
 #endif
 
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
+
 #include "arp_and_chords_tab_widget.h"
 #include "embed.h"
 #include "note_play_handle.h"
@@ -52,11 +59,9 @@
 #include "pixmap_button.h"
 #include "knob.h"
 #include "tooltip.h"
-
-
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
+#include "gui_templates.h"
+#include "tempo_sync_knob.h"
+#include "channel_track.h"
 
 
 
@@ -182,23 +187,21 @@ arpAndChordsTabWidget::chord arpAndChordsTabWidget::s_chords[] =
 } ;
 
 
-const int CHORDS_GROUPBOX_X = 5;
+const int CHORDS_GROUPBOX_X = 4;
 const int CHORDS_GROUPBOX_Y = 5;
-const int CHORDS_GROUPBOX_WIDTH = 240;
+const int CHORDS_GROUPBOX_WIDTH = 238;
 const int CHORDS_GROUPBOX_HEIGHT = 65;
 const int ARP_GROUPBOX_X = CHORDS_GROUPBOX_X;
 const int ARP_GROUPBOX_Y = 10 + CHORDS_GROUPBOX_Y + CHORDS_GROUPBOX_HEIGHT;
 const int ARP_GROUPBOX_WIDTH = CHORDS_GROUPBOX_WIDTH;
-const int ARP_GROUPBOX_HEIGHT = 180 - ARP_GROUPBOX_Y;
+const int ARP_GROUPBOX_HEIGHT = 200 - ARP_GROUPBOX_Y;
 
 
 
-arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
-							QWidget * _parent ) :
-	QWidget( _parent ),
+arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track ) :
+	QWidget( _channel_track->tabWidgetParent() ),
 	settings(),
-	m_arpDirection( UP ),
-	m_channelTrack( _channel_track )
+	m_arpDirection( UP )
 {
 	m_chordsGroupBox = new groupBox( tr( "CHORDS" ), this );
 	m_chordsGroupBox->setGeometry( CHORDS_GROUPBOX_X, CHORDS_GROUPBOX_Y,
@@ -207,7 +210,7 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 
 	m_chordsComboBox = new QComboBox( m_chordsGroupBox );
 	m_chordsComboBox->setFont( pointSize<9>( m_chordsComboBox->font() ) );
-	m_chordsComboBox->setGeometry( 10, 25, 100, 22 );
+	m_chordsComboBox->setGeometry( 10, 25, 140, 22 );
 
 	for( int i = 0; s_chords[i].interval[0] != -1; ++i )
 	{
@@ -223,7 +226,7 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 	m_chordRangeKnob->setLabel( tr( "RANGE" ) );
 	m_chordRangeKnob->setRange( 1.0, 9.0, 1.0 );
 	m_chordRangeKnob->setValue( 1.0, TRUE );
-	m_chordRangeKnob->move( 123, 23 );
+	m_chordRangeKnob->move( 164, 24 );
 	m_chordRangeKnob->setHintText( tr( "Chord range:" ) + " ", " " +
 							tr( "octave(s)" ) );
 #ifdef QT4
@@ -258,7 +261,7 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 			"possible chords, you can select." ) );
 	m_arpComboBox = new QComboBox( m_arpGroupBox );
 	m_arpComboBox->setFont( pointSize<9>( m_arpComboBox->font() ) );
-	m_arpComboBox->setGeometry( 10, 25, 100, 22 );
+	m_arpComboBox->setGeometry( 10, 25, 140, 22 );
 
 	for( int i = 0; s_chords[i].interval[0] != -1; ++i )
 	{
@@ -273,7 +276,7 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 	m_arpRangeKnob->setLabel( tr( "RANGE" ) );
 	m_arpRangeKnob->setRange( 1.0, 9.0, 1.0 );
 	m_arpRangeKnob->setValue( 1.0, TRUE );
-	m_arpRangeKnob->move( 124, 24 );
+	m_arpRangeKnob->move( 164, 24 );
 	m_arpRangeKnob->setHintText( tr( "Arpeggio range:" ) + " ", " " +
 							tr( "octave(s)" ) );
 #ifdef QT4
@@ -285,12 +288,12 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 			"The selected arpeggio will be played within specified "
 			"amount of octaves." ) );
 
-	m_arpTimeKnob = new knob( knobBright_26, m_arpGroupBox,
+	m_arpTimeKnob = new tempoSyncKnob( knobBright_26, m_arpGroupBox,
 							tr( "Arpeggio time" ) );
 	m_arpTimeKnob->setLabel( tr( "TIME" ) );
 	m_arpTimeKnob->setRange( 10.0, 1000.0, 1.0 );
 	m_arpTimeKnob->setValue( 100.0, TRUE );
-	m_arpTimeKnob->move( 164, 24 );
+	m_arpTimeKnob->move( 164, 70 );
 	m_arpTimeKnob->setHintText( tr( "Arpeggio time:" ) + " ", " " +
 								tr( "ms" ) );
 #ifdef QT4
@@ -307,7 +310,7 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 	m_arpGateKnob->setLabel( tr( "GATE" ) );
 	m_arpGateKnob->setRange( 1.0, 200.0, 1.0 );
 	m_arpGateKnob->setValue( 100.0, TRUE );
-	m_arpGateKnob->move( 204, 24 );
+	m_arpGateKnob->move( 204, 70 );
 	m_arpGateKnob->setHintText( tr( "Arpeggio gate:" ) + " ", tr( "%" ) );
 #ifdef QT4
 	m_arpGateKnob->setWhatsThis(
@@ -324,31 +327,9 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 	m_arpDirectionLbl->setFont( pointSize<6>( m_arpDirectionLbl->font() ) );
 
 
-/*	m_arpOffBtn = new pixmapButton( m_arpGroupBox );
-	m_arpOffBtn->move( 10, 87 );
-	m_arpOffBtn->setActiveGraphic( embed::getIconPixmap( "arp_off_on" ) );
-	m_arpOffBtn->setInactiveGraphic( embed::getIconPixmap(
-							"arp_off_off" ) );
-#ifdef QT4
-	m_arpOffBtn->setChecked( TRUE );
-#else
-	m_arpOffBtn->setOn( TRUE );
-#endif
-#ifndef QT4
-	m_arpOffBtn->setBackgroundMode( Qt::PaletteBackground );
-#endif
-#ifdef QT4
-	m_arpOffBtn->setToolTip(
-#else
-	toolTip::add( m_arpOffBtn,
-#endif
-		tr( "arpeggio direction = none = no arpeggio" ) );
-	connect( m_arpOffBtn, SIGNAL( toggled( bool ) ), this,
-						SLOT( arpOffToggled( bool ) ) );*/
-
 
 	m_arpUpBtn = new pixmapButton( m_arpGroupBox );
-	m_arpUpBtn->move( 80, 70 );
+	m_arpUpBtn->move( 70, 70 );
 	m_arpUpBtn->setActiveGraphic( embed::getIconPixmap( "arp_up_on" ) );
 	m_arpUpBtn->setInactiveGraphic( embed::getIconPixmap( "arp_up_off" ) );
 #ifdef QT4
@@ -364,7 +345,7 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 						SLOT( arpUpToggled( bool ) ) );
 
 	m_arpDownBtn = new pixmapButton( m_arpGroupBox );
-	m_arpDownBtn->move( 100, 70 );
+	m_arpDownBtn->move( 90, 70 );
 	m_arpDownBtn->setActiveGraphic( embed::getIconPixmap( "arp_down_on" ) );
 	m_arpDownBtn->setInactiveGraphic( embed::getIconPixmap(
 							"arp_down_off" ) );
@@ -376,7 +357,7 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 					SLOT( arpDownToggled( bool ) ) );
 
 	m_arpUpAndDownBtn = new pixmapButton( m_arpGroupBox );
-	m_arpUpAndDownBtn->move( 120, 70 );
+	m_arpUpAndDownBtn->move( 110, 70 );
 	m_arpUpAndDownBtn->setActiveGraphic( embed::getIconPixmap(
 						"arp_up_and_down_on" ) );
 	m_arpUpAndDownBtn->setInactiveGraphic( embed::getIconPixmap(
@@ -390,7 +371,7 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 					SLOT( arpUpAndDownToggled( bool ) ) );
 
 	m_arpRandomBtn = new pixmapButton( m_arpGroupBox );
-	m_arpRandomBtn->move( 140, 70 );
+	m_arpRandomBtn->move( 130, 70 );
 	m_arpRandomBtn->setActiveGraphic( embed::getIconPixmap(
 							"arp_random_on" ) );
 	m_arpRandomBtn->setInactiveGraphic( embed::getIconPixmap(
@@ -403,7 +384,6 @@ arpAndChordsTabWidget::arpAndChordsTabWidget( channelTrack * _channel_track,
 					SLOT( arpRandomToggled( bool ) ) );
 
 	QButtonGroup * m_arpDirections_group = new QButtonGroup( this );
-//	m_arpDirections_group->addButton( m_arpOffBtn );
 	m_arpDirections_group->addButton( m_arpUpBtn );
 	m_arpDirections_group->addButton( m_arpDownBtn );
 	m_arpDirections_group->addButton( m_arpUpAndDownBtn );
@@ -626,38 +606,41 @@ void arpAndChordsTabWidget::processNote( notePlayHandle * _n )
 void arpAndChordsTabWidget::saveSettings( QDomDocument & _doc,
 							QDomElement & _parent )
 {
-	QDomElement elw_de = _doc.createElement( nodeName() );
-	elw_de.setAttribute( "chorddisabled", QString::number(
+	QDomElement act_de = _doc.createElement( nodeName() );
+	act_de.setAttribute( "chorddisabled", QString::number(
 					!m_chordsGroupBox->isActive() ) );
 #ifdef QT4
-	elw_de.setAttribute( "chord", QString::number(
+	act_de.setAttribute( "chord", QString::number(
 					m_chordsComboBox->currentIndex() ) );
 #else
-	elw_de.setAttribute( "chord", QString::number(
+	act_de.setAttribute( "chord", QString::number(
 					m_chordsComboBox->currentItem() ) );
 #endif
-	elw_de.setAttribute( "chordrange", QString::number(
+	act_de.setAttribute( "chordrange", QString::number(
 					m_chordRangeKnob->value() ) );
 
-	elw_de.setAttribute( "arpdisabled", QString::number(
+	act_de.setAttribute( "arpdisabled", QString::number(
 						!m_arpGroupBox->isActive() ) );
 #ifdef QT4
-	elw_de.setAttribute( "arp", QString::number(
+	act_de.setAttribute( "arp", QString::number(
 					m_arpComboBox->currentIndex() ) );
 #else
-	elw_de.setAttribute( "arp", QString::number(
+	act_de.setAttribute( "arp", QString::number(
 					m_arpComboBox->currentItem() ) );
 #endif
-	elw_de.setAttribute( "arprange", QString::number(
+	act_de.setAttribute( "arprange", QString::number(
 					m_arpRangeKnob->value() ) );
-	elw_de.setAttribute( "arptime", QString::number(
+	act_de.setAttribute( "arptime", QString::number(
 					m_arpTimeKnob->value() ) );
-	elw_de.setAttribute( "arpgate", QString::number(
+	act_de.setAttribute( "arpgate", QString::number(
 					m_arpGateKnob->value() ) );
-	elw_de.setAttribute( "arpdir", QString::number(
+	act_de.setAttribute( "arpdir", QString::number(
 					m_arpDirection ) );
+	act_de.setAttribute( "arpsyncmode", QString::number(
+				( int ) m_arpTimeKnob->getSyncMode() ) );
 
-	_parent.appendChild( elw_de );
+
+	_parent.appendChild( act_de );
 }
 
 
@@ -683,6 +666,9 @@ void arpAndChordsTabWidget::loadSettings( const QDomElement & _this )
 	m_arpGateKnob->setValue( _this.attribute( "arpgate" ).toFloat() );
 	m_arpDirection = static_cast<arpDirections>(
 					_this.attribute( "arpdir" ).toInt() );
+	m_arpTimeKnob->setSyncMode( 
+		( tempoSyncKnob::tempoSyncMode ) _this.attribute(
+						 "arpsyncmode" ).toInt() );
 
 	m_arpGroupBox->setState( m_arpDirection != OFF &&
 				!_this.attribute( "arpdisabled" ).toInt() );
