@@ -2,8 +2,9 @@
  * envelope_tab_widget.cpp - widget for use in envelope/lfo/filter-tab of
  *                           channel-window
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -51,12 +52,14 @@
 #include "group_box.h"
 #include "tab_widget.h"
 #include "embed.h"
-#include "templates.h"
+#include "gui_templates.h"
+#include "channel_track.h"
+
 
 
 const int TARGETS_TABWIDGET_X = 4;
 const int TARGETS_TABWIDGET_Y = 5;
-const int TARGETS_TABWIDGET_WIDTH = 240;
+const int TARGETS_TABWIDGET_WIDTH = 238;
 const int TARGETS_TABWIDGET_HEIGTH = 175;
 
 const int FILTER_GROUPBOX_X = TARGETS_TABWIDGET_X;
@@ -71,7 +74,7 @@ const float RES_PRECISION = 1000.0f;
 
 // names for env- and lfo-targets - first is name being displayed to user
 // and second one is used internally, e.g. for saving/restoring settings
-const QString targetNames[envelopeTabWidget::TARGET_COUNT][2] =
+static const QString targetNames[envelopeTabWidget::TARGET_COUNT][2] =
 {
 	{ envelopeTabWidget::tr( "VOLUME" ), "vol" },
 /*	envelopeTabWidget::tr( "Pan" ),
@@ -82,11 +85,9 @@ const QString targetNames[envelopeTabWidget::TARGET_COUNT][2] =
  
 
 
-envelopeTabWidget::envelopeTabWidget( channelTrack * _channel_track,
-							QWidget * _parent ) :
-	QWidget( _parent ),
-	settings(),
-	m_channelTrack( _channel_track )
+envelopeTabWidget::envelopeTabWidget( channelTrack * _channel_track ) :
+	QWidget( _channel_track->tabWidgetParent() ),
+	settings()
 {
 
 	m_targetsTabWidget = new tabWidget( tr( "TARGET" ), this );
@@ -154,7 +155,6 @@ envelopeTabWidget::envelopeTabWidget( channelTrack * _channel_track,
 	m_filterComboBox->addItem( tr( "Allpass" ) );
 	m_filterComboBox->addItem( tr( "Moog" ) );
 	m_filterComboBox->addItem( tr( "2x LowPass" ) );
-	m_filterComboBox->addItem( tr( "2x Moog" ) );
 
 #ifdef QT4
 	m_filterComboBox->setWhatsThis(
@@ -269,7 +269,7 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 		int old_filter_cut = 0;
 		int old_filter_res = 0;
 
-		basicFilters<>::filterTypes filter = static_cast<basicFilters<>::filterTypes>( m_filterComboBox->
+		basicFilters<>::filterTypes filter = basicFilters<>::getFilterType( m_filterComboBox->
 #ifdef QT4
 				currentIndex()
 #else
@@ -512,7 +512,8 @@ void envelopeTabWidget::saveSettings( QDomDocument & _doc,
 	{
 		QDomElement target_de = _doc.createElement(
 						m_envLFOWidgets[i]->nodeName() +
-						targetNames[i][1].toLower() );
+					QString(
+						targetNames[i][1] ).toLower() );
 		m_envLFOWidgets[i]->saveSettings( _doc, target_de );
 		etw_de.appendChild( target_de );
 	}
@@ -542,7 +543,7 @@ void envelopeTabWidget::loadSettings( const QDomElement & _this )
 			{
 				if( node.nodeName() ==
 						m_envLFOWidgets[i]->nodeName() +
-						targetNames[i][1].toLower() )
+					QString( targetNames[i][1] ).toLower() )
 				{
 					m_envLFOWidgets[i]->loadSettings(
 							node.toElement() );
