@@ -1,8 +1,9 @@
 /*
  * lmms.cpp - LMMS-base-system, contains main() and main-window
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -27,32 +28,22 @@
 #ifdef QT4
 
 #include <QApplication>
-#include <QLocale>
 #include <QFileDialog>
 #include <QCloseEvent>
 #include <QSplitter>
-#include <QStatusBar>
-#include <QMenuBar>
-#include <QToolButton>
 #include <QSplashScreen>
-#include <QStatusBar>
-#include <QToolBar>
-#include <QAction>
 #include <QMessageBox>
+#include <QMenuBar>
 
 #else
 
 #include <qsplitter.h>
 #include <qapplication.h>
-#include <qtextcodec.h>
 #include <qdir.h>
 #include <qfiledialog.h>
 #include <qpopupmenu.h>
-#include <qmenubar.h>
-#include <qtoolbutton.h>
-#include <qstatusbar.h>
-#include <qaction.h>
 #include <qmessagebox.h>
+#include <qmenubar.h>
 
 #if QT_VERSION >= 0x030200
 #include <qsplashscreen.h>
@@ -71,7 +62,8 @@
 #include "piano_roll.h"
 #include "embed.h"
 #include "about_dialog.h"
-#include "browser.h"
+#include "file_browser.h"
+#include "plugin_browser.h"
 #include "side_bar.h"
 #include "config_mgr.h"
 #include "mixer.h"
@@ -80,6 +72,7 @@
 #include "setup_dialog.h"
 #include "preset_preview_play_handle.h"
 #include "audio_dummy.h"
+#include "tool_button.h"
 
 
 #if QT_VERSION >= 0x030100
@@ -97,6 +90,7 @@ lmmsMainWin::lmmsMainWin() :
 			0 , NULL, WDestructiveClose
 #endif
 		),
+	m_workspace( NULL ),
 	m_templatesMenu( NULL )
 {
 #ifdef QT4
@@ -106,12 +100,20 @@ lmmsMainWin::lmmsMainWin() :
 	// small workaround, because inst() is called by objects which
 	// are created within this function
 	s_instanceOfMe = this;
+	bool no_mdi = configManager::inst()->value( "app", "gimplikewindows"
+								).toInt();
 
-	QStatusBar * status_bar = new QStatusBar( this );
-	status_bar->setSizeGripEnabled( TRUE );
+	QWidget * main_widget = new QWidget( this );
+	QVBoxLayout * vbox = new QVBoxLayout( main_widget );
+	vbox->setSpacing( 0 );
+	vbox->setMargin( 0 );
 
-	QWidget * w = new QWidget( this );
+
+	QWidget * w = new QWidget( main_widget );
 	QHBoxLayout * hbox = new QHBoxLayout( w );
+	hbox->setSpacing( 0 );
+	hbox->setMargin( 0 );
+
 	sideBar * side_bar = new sideBar( Qt::Vertical, w );
 	side_bar->setStyle( sideBar::VSNET/*KDEV3ICON*/ );
 	side_bar->setPosition( sideBar::DockLeft );
@@ -123,42 +125,71 @@ lmmsMainWin::lmmsMainWin() :
 
 	int id = 0;
 	QString wdir = configManager::inst()->lmmsWorkingDir();
-	side_bar->appendTab( new browser( wdir+"projects", "*.xml",
+	side_bar->appendTab( new pluginBrowser( splitter ), ++id );
+	side_bar->appendTab( new fileBrowser( wdir+"projects",
+							"*.mmp *.xml *.mid",
 							tr( "My projects" ),
 					embed::getIconPixmap( "project_file" ),
 							splitter ), ++id );
-	side_bar->appendTab( new browser( wdir+"samples", "*.wav *.ogg *.voc "
-						"*.aif *.aiff *.au *.raw",
+	side_bar->appendTab( new fileBrowser( wdir+"samples", "*.wav *.ogg *.au"
+					"*.voc *.aif *.aiff *.flac *.raw",
 							tr( "My samples" ),
 					embed::getIconPixmap( "sound_file" ),
 							splitter ), ++id );
-	side_bar->appendTab( new browser( wdir+"presets", "*.cs.xml",
+	side_bar->appendTab( new fileBrowser( wdir+"presets", "*.cs.xml",
 							tr( "My presets" ),
 					embed::getIconPixmap( "preset_file" ),
 							splitter ), ++id );
-	side_bar->appendTab( new browser( QDir::homePath(), "*",
+	side_bar->appendTab( new fileBrowser( QDir::homePath(), "*",
 							tr( "My home" ),
 					embed::getIconPixmap( "home" ),
 							splitter ), ++id );
-	side_bar->appendTab( new browser( QDir::rootPath(), "*",
+	side_bar->appendTab( new fileBrowser( QDir::rootPath(), "*",
 							tr( "Root directory" ),
 					embed::getIconPixmap( "root" ),
 							splitter ), ++id );
 
-	m_workspace = new lmmsWorkspace( splitter );
-	m_workspace->setScrollBarsEnabled( TRUE );
+	if( no_mdi == FALSE )
+	{
+		m_workspace = new QWorkspace( splitter );
+		m_workspace->setScrollBarsEnabled( TRUE );
 
 #ifdef QT4
-	m_workspace->setBackground( embed::getIconPixmap(
-						"background_artwork" ) );
+/*		m_workspace->setBackground( embed::getIconPixmap(
+						"background_artwork" ) );*/
 #else
-	m_workspace->setPaletteBackgroundPixmap( embed::getIconPixmap(
+		m_workspace->setPaletteBackgroundPixmap( embed::getIconPixmap(
 						"background_artwork" ) );
 #endif
+	}
 
 	hbox->addWidget( side_bar );
 	hbox->addWidget( splitter );
-	setCentralWidget( w );
+
+
+	// create global-toolbar at the top of our window
+	m_toolBar = new QWidget( main_widget );
+	m_toolBar->setFixedHeight( 64 );
+	m_toolBar->move( 0, 0 );
+#ifdef QT4
+	QPalette pal;
+	pal.setBrush( m_toolBar->backgroundRole(), QBrush(
+				embed::getIconPixmap( "main_toolbar_bg" ) ) );
+	m_toolBar->setPalette( pal );
+#else
+	m_toolBar->setPaletteBackgroundPixmap(
+				embed::getIconPixmap( "main_toolbar_bg" ) );
+#endif
+
+	// add layout for organizing quite complex toolbar-layouting
+	m_toolBarLayout = new QGridLayout( m_toolBar/*, 2, 1*/ );
+	m_toolBarLayout->setMargin( 0 );
+	m_toolBarLayout->setSpacing( 0 );
+
+	vbox->addWidget( m_toolBar );
+	vbox->addWidget( w );
+	setCentralWidget( main_widget );
+
 
 
 #if QT_VERSION >= 0x030200
@@ -186,136 +217,84 @@ lmmsMainWin::lmmsMainWin() :
 	resetWindowTitle( "" );
 	setWindowIcon( embed::getIconPixmap( "icon" ) );
 
-	// TODO: when dropping qt3-support clean up the whole menu- and toolbar
-	// code and create QAction's to be inserted in menu- and toolbar - the
-	// current situation is horrible.....!!!!!!!
 
+	// create tool-buttons
+	toolButton * project_new = new toolButton(
+					embed::getIconPixmap( "project_new" ),
+					tr( "Create new project" ),
+					this, SLOT( createNewProject() ),
+							m_toolBar );
 
-	m_projectTools = new QToolBar( this );
-#ifdef QT4
-	addToolBar( Qt::TopToolBarArea, m_projectTools );
-#else
-	addDockWindow( m_projectTools, tr( "Song management" ), DockTop,
-									TRUE );
-#endif
-
-	QAction * a;
-
-#ifdef QT4
-	a = new QAction( embed::getIconPixmap( "project_new" ),
-						tr( "Create new project" ),
-							m_projectTools );
-	a->setShortcut( tr( "Ctrl+N" ) );
-	connect( a, SIGNAL( activated() ), this, SLOT( createNewProject() ) );
-
-	a = new QAction( embed::getIconPixmap( "project_open" ),
-						tr( "Open existing project" ),
-							m_projectTools );
-	a->setShortcut( tr( "Ctrl+O" ) );
-	connect( a, SIGNAL( activated() ), this, SLOT( openProject() ) );
-
-	a = new QAction( embed::getIconPixmap( "project_save" ),
-						tr( "Save current project" ),
-							m_projectTools );
-	a->setShortcut( tr( "Ctrl+S" ) );
-	connect( a, SIGNAL( activated() ), this, SLOT( saveProject() ) );
-
-	a = new QAction( embed::getIconPixmap( "project_export" ),
-						tr( "Export current project" ),
-							m_projectTools );
-	a->setShortcut( tr( "Ctrl+E" ) );
-	connect( a, SIGNAL( activated() ), songEditor::inst(),
-						SLOT( exportProject() ) );
-
-#else
-/*	a = new QAction( embed::getIconPixmap( "project_new" ),
-							tr( "Create new project" ),
-							CTRL + Key_N, this );
-	connect( a, SIGNAL( activated() ), this, SLOT( createNewProject() ) );
-	a->addTo( m_projectTools );*/
-	QToolButton * t = new QToolButton( embed::getIconPixmap(
-							"project_new" ),
-						tr( "Create new project" ),
-						QString::null, this,
-						SLOT( createNewProject() ),
-							m_projectTools );
-	t->setAccel( Qt::Key_F6 );
 	QDir d( configManager::inst()->projectsDir() + "templates" );
-	QStringList templates = d.entryList( "*.xml",
+	QStringList templates = d.entryList(
+#ifdef QT4
+						QStringList( "*.mpt" ),
+#else
+						"*.mpt",
+#endif
 						QDir::Files | QDir::Readable );
 	if( !templates.isEmpty() )
 	{
-		m_templatesMenu = new QMenu( t );
+		m_templatesMenu = new QMenu( project_new );
 		for( QStringList::iterator it = templates.begin();
 						it != templates.end(); ++it )
 		{
-			m_templatesMenu->insertItem(
+			m_templatesMenu->addAction(
 					embed::getIconPixmap( "project_file" ),
 					( *it ).left( ( *it ).length() - 4 ) );
 		}
 		connect( m_templatesMenu, SIGNAL( activated( int ) ),
 			this, SLOT( createNewProjectFromTemplate( int ) ) );
-		t->setMenu( m_templatesMenu );
-		t->setPopupDelay( 0 );
+		project_new->setMenu( m_templatesMenu );
+#ifdef QT4
+		project_new->setPopupMode( toolButton::MenuButtonPopup );
+#else
+		project_new->setPopupDelay( 0 );
+#endif
 	}
 
-	a = new QAction( 
-#if QT_VERSION < 0x030200
-			"",
-#endif		    
-			embed::getIconPixmap( "project_open" ),
-						tr( "Open existing project" ),
-							CTRL + Key_O, this );
-	connect( a, SIGNAL( activated() ), this, SLOT( openProject() ) );
-	a->addTo( m_projectTools );
 
-	a = new QAction( 
-#if QT_VERSION < 0x030200
-			"",
-#endif		    
-			embed::getIconPixmap( "project_save" ),
-						tr( "Save current project" ),
-							CTRL + Key_S, this );
-	connect( a, SIGNAL( activated() ), this, SLOT( saveProject() ) );
-	a->addTo( m_projectTools );
-
-	a = new QAction( 
-#if QT_VERSION < 0x030200
-			"",
-#endif		    
-			embed::getIconPixmap( "project_export" ),
-						tr( "Export current project" ),
-							CTRL + Key_E, this );
-	connect( a, SIGNAL( activated() ), songEditor::inst(),
-						SLOT( exportProject() ) );
-	a->addTo( m_projectTools );
-
-#endif
+	toolButton * project_open = new toolButton( 
+					embed::getIconPixmap( "project_open" ),
+					tr( "Open existing project" ),
+					this, SLOT( openProject() ),
+								m_toolBar );
 
 
+	toolButton * project_save = new toolButton( 
+					embed::getIconPixmap( "project_save" ),
+					tr( "Save current project" ),
+					this, SLOT( saveProject() ),
+								m_toolBar );
 
-	m_windowTools = new QToolBar( tr( "Windows" ), this );
-#ifdef QT4
-	addToolBar( Qt::TopToolBarArea, m_windowTools );
-#else
-	addDockWindow( m_windowTools, tr( "Windows" ), Qt::DockTop, FALSE );
-#endif
+
+	toolButton * project_export = new toolButton( 
+				embed::getIconPixmap( "project_export" ),
+					tr( "Export current project" ),
+					songEditor::inst(),
+							SLOT( exportProject() ),
+								m_toolBar );
+
+
+	m_toolBarLayout->addWidget( project_new, 0, 0 );
+	m_toolBarLayout->addWidget( project_open, 0, 1 );
+	m_toolBarLayout->addWidget( project_save, 0, 2 );
+	m_toolBarLayout->addWidget( project_export, 0, 3 );
 
 
-#ifndef QT4
 
 	// window-toolbar
-	m_bbEditorWindow = new QToolButton( embed::getIconPixmap( "bb_track" ),
-						tr( "Show/Hide Beat+Bassline "
-							"Editor" ) + " (F6)",
-						QString::null,
+	toolButton * bb_editor_window = new toolButton(
+					embed::getIconPixmap( "bb_track" ),
+					tr( "Show/hide Beat+Bassline Editor" ) +
+									" (F6)",
 					this, SLOT( toggleBBEditorWin() ),
-								m_windowTools );
-	m_bbEditorWindow->setAccel( Qt::Key_F6 );
+								m_toolBar );
+	bb_editor_window->setShortcut( Qt::Key_F6 );
 #ifdef QT4
-	m_bbEditorWindow->setWhatsThis( 
+	bb_editor_window->setWhatsThis(
 #else
-	QWhatsThis::add( m_bbEditorWindow, 
+	QWhatsThis::add( bb_editor_window, 
 #endif
 		tr( "By pressing this button, you can show or hide the "
 			"Beat+Bassline Editor. The Beat+Bassline Editor is "
@@ -325,183 +304,178 @@ lmmsMainWin::lmmsMainWin() :
 			"that." ) );
 
 
-	m_pianoRollWindow = new QToolButton( embed::getIconPixmap( "piano" ),
-						tr( "Show/Hide Piano-Roll" ) +
+	toolButton * piano_roll_window = new toolButton(
+						embed::getIconPixmap( "piano" ),
+						tr( "Show/hide Piano-Roll" ) +
 									" (F7)",
-								QString::null,
 					this, SLOT( togglePianoRollWin() ),
-								m_windowTools );
-	m_pianoRollWindow->setAccel( Key_F7 );
-	QWhatsThis::add( m_pianoRollWindow, tr( "By pressing this button, you "
-						"can show or hide the "
-						"Piano-Roll. With the help of "
-						"the Piano-Roll you can edit "
-						"melody-patterns in an easy "
-						"way." ) );
-
-	m_projectNotesWindow = new QToolButton( embed::getIconPixmap(
-							"project_notes" ),
-						tr( "Show/Hide Project notes" )+
-								" (F10)",
-								QString::null,
-					this, SLOT( toggleProjectNotesWin() ),
-								m_windowTools );
-	m_projectNotesWindow->setAccel( Key_F10 );
-	QWhatsThis::add( m_projectNotesWindow, tr( "By pressing this button, "
-							"you can show or hide "
-							"the project notes "
-							"window. "
-							"In this window you "
-							"can put down your "
-							"project notes.") );
-
-	m_songEditorWindow = new QToolButton( embed::getIconPixmap(
-								"songeditor" ),
-						tr( "Show/Hide Song-Editor" ) +
-									" (F8)",
-								QString::null,
-					this, SLOT( toggleSongEditorWin() ),
-								m_windowTools );
-	m_songEditorWindow->setAccel( Qt::Key_F8 );
+								m_toolBar );
+	piano_roll_window->setShortcut( Qt::Key_F7 );
 #ifdef QT4
-	m_songEditorWindow->setWhatsThis(
+	piano_roll_window->setWhatsThis(
 #else
-	QWhatsThis::add( m_songEditorWindow,
+	QWhatsThis::add( piano_roll_window,
+#endif
+			tr( "By pressing this button, you can show or hide the "
+				"Piano-Roll. With the help of the Piano-Roll "
+				"you can edit melody-patterns in an easy way."
+				) );
+
+	toolButton * song_editor_window = new toolButton(
+					embed::getIconPixmap( "songeditor" ),
+					tr( "Show/hide Song-Editor" ) + " (F8)",
+					this, SLOT( toggleSongEditorWin() ),
+								m_toolBar );
+	song_editor_window->setShortcut( Qt::Key_F8 );
+#ifdef QT4
+	song_editor_window->setWhatsThis(
+#else
+	QWhatsThis::add( song_editor_window,
 #endif
 		tr( "By pressing this button, you can show or hide the "
-			"Song-Editor. With the Song-Editor you can edit the "
-			"song and set when and which track should be played. "
+			"Song-Editor. With the help of the Song-Editor you can "
+			"edit song-playlist and specify when which track "
+			"should be played. "
 			"You can also insert and move samples (e.g. "
-			"rap-samples) directly in the playlist." ) );
+			"rap-samples) directly into the playlist." ) );
 
 
-	m_effectBoardWindow = new QToolButton( embed::getIconPixmap(
-							"effect_board" ),
-						tr( "Show/Hide EffectBoard" ) +
-									" (F9)",
-						QString::null,
-						this, SLOT( emptySlot() ),
-								m_windowTools );
-	m_effectBoardWindow->setAccel( Qt::Key_F9 );
+	toolButton * effect_board_window = new toolButton(
+					embed::getIconPixmap( "effect_board" ),
+					tr( "Show/hide EffectBoard" ) + " (F9)",
+					this, SLOT( emptySlot() ), m_toolBar );
+	effect_board_window->setShortcut( Qt::Key_F9 );
 #ifdef QT4
-	m_effectBoardWindow->setWhatsThis( 
+	effect_board_window->setWhatsThis( 
 #else
-	QWhatsThis::add( m_effectBoardWindow,
+	QWhatsThis::add( effect_board_window,
 #endif
 		tr( "By pressing this button, you can show or hide the "
 			"EffectBoard. The EffectBoard is a very powerful tool "
 			"for managing effects for your song. You can insert "
 			"effects into different effect-channels." ) );
 
+	toolButton * project_notes_window = new toolButton(
+					embed::getIconPixmap( "project_notes" ),
+					tr( "Show/hide project notes" ) +
+								" (F10)",
+					this, SLOT( toggleProjectNotesWin() ),
+								m_toolBar );
+	project_notes_window->setShortcut( Qt::Key_F10 );
+#ifdef QT4
+	project_notes_window->setWhatsThis(
+#else
+	QWhatsThis::add( project_notes_window,
 #endif
+		tr( "By pressing this button, you can show or hide the "
+			"project notes window. In this window you can put "
+			"down your project notes.") );
+
+	m_toolBarLayout->addWidget( bb_editor_window, 1, 0 );
+	m_toolBarLayout->addWidget( piano_roll_window, 1, 1 );
+	m_toolBarLayout->addWidget( song_editor_window, 1, 2 );
+	m_toolBarLayout->addWidget( effect_board_window, 1, 3 );
+	m_toolBarLayout->addWidget( project_notes_window, 1, 4 );
+
+	m_toolBarLayout->setColumnStretch( 100, 1 );
 
-//#ifndef QT4
-	// create a "What's this"-Button
-//	(void) QWhatsThis::whatsThisButton( m_windowTools );
 
 
 	// project-popup-menu
-	m_projectMenu = new QMenu( this );
+	QMenu * project_menu = new QMenu( this );
 #ifdef QT4
-	menuBar()->addMenu( m_projectMenu )->setText( tr( "&Project" ) );
+	menuBar()->addMenu( project_menu )->setText( tr( "&Project" ) );
 #else
-	menuBar()->insertItem( tr( "&Project" ), m_projectMenu );
+	menuBar()->insertItem( tr( "&Project" ), project_menu );
 #endif
-	m_projectMenu->addAction( embed::getIconPixmap( "project_new" ),
+	project_menu->addAction( embed::getIconPixmap( "project_new" ),
 					tr( "&New" ),
 					this, SLOT( createNewProject() ),
 					Qt::CTRL + Qt::Key_N );
 
-	m_projectMenu->addAction( embed::getIconPixmap( "project_open" ),
+	project_menu->addAction( embed::getIconPixmap( "project_open" ),
 					tr( "&Open..." ),
 					this, SLOT( openProject() ),
 					Qt::CTRL + Qt::Key_O );	
 
-	m_projectMenu->addAction( embed::getIconPixmap( "project_save" ),
+	project_menu->addAction( embed::getIconPixmap( "project_save" ),
 					tr( "&Save" ),
 					this, SLOT( saveProject() ),
 					Qt::CTRL + Qt::Key_S );
 
-	m_projectMenu->addAction( embed::getIconPixmap( "project_saveas" ),
+	project_menu->addAction( embed::getIconPixmap( "project_saveas" ),
 					tr( "Save &As..." ),
 					this, SLOT( saveProjectAs() ),
 					Qt::CTRL + Qt::SHIFT + Qt::Key_S );
 #ifdef QT4
-	m_projectMenu->addSeparator();
+	project_menu->addSeparator();
 #else
-	m_projectMenu->insertSeparator();
+	project_menu->insertSeparator();
 #endif
-	m_projectMenu->addAction( /*embed::getIconPixmap( "project_import" ),*/
+	project_menu->addAction( /*embed::getIconPixmap( "project_import" ),*/
 					tr( "Import file" ),
 					songEditor::inst(),
 					SLOT( importProject() ) );
 #ifdef QT4
-	m_projectMenu->addSeparator();
+	project_menu->addSeparator();
 #else
-	m_projectMenu->insertSeparator();
+	project_menu->insertSeparator();
 #endif
-	m_projectMenu->addAction( embed::getIconPixmap( "project_export" ),
+	project_menu->addAction( embed::getIconPixmap( "project_export" ),
 					tr( "E&xport" ),
 					songEditor::inst(),
 					SLOT( exportProject() ),
 					Qt::CTRL + Qt::Key_E );
 #ifdef QT4
-	m_projectMenu->addSeparator();
+	project_menu->addSeparator();
 #else
-	m_projectMenu->insertSeparator();
+	project_menu->insertSeparator();
 #endif
-	m_projectMenu->addAction( embed::getIconPixmap( "exit" ), tr( "&Quit" ),
+	project_menu->addAction( embed::getIconPixmap( "exit" ), tr( "&Quit" ),
 					qApp, SLOT( closeAllWindows() ),
 					Qt::CTRL+Qt::Key_Q );
 
 
-	QMenu * m_settingsMenu = new QMenu( this );
+	QMenu * settings_menu = new QMenu( this );
 #ifdef QT4
-	menuBar()->addMenu( m_settingsMenu )->setText( tr( "&Settings" ) );
+	menuBar()->addMenu( settings_menu )->setText( tr( "&Settings" ) );
 #else
-	menuBar()->insertItem( tr( "&Settings" ), m_settingsMenu );
+	menuBar()->insertItem( tr( "&Settings" ), settings_menu );
 #endif
-	m_settingsMenu->addAction( embed::getIconPixmap( "setup_general" ),
+	settings_menu->addAction( embed::getIconPixmap( "setup_general" ),
 					tr( "Show settings dialog" ),
 					this, SLOT( showSettingsDialog() ) );
-	m_settingsMenu->addAction( embed::getIconPixmap( "wizard" ),
+	settings_menu->addAction( embed::getIconPixmap( "wizard" ),
 					tr( "Show setup wizard" ),
 					configManager::inst(), SLOT( exec() ) );
 
 	
 	// help-popup-menu
-	m_helpMenu = new QMenu( this );
+	QMenu * help_menu = new QMenu( this );
 #ifdef QT4
-	menuBar()->addMenu( m_helpMenu )->setText( tr( "&Help" ) );
+	menuBar()->addMenu( help_menu )->setText( tr( "&Help" ) );
 #else
-	menuBar()->insertItem( tr( "&Help" ), m_helpMenu );
+	menuBar()->insertItem( tr( "&Help" ), help_menu );
 #endif
-	m_helpMenu->addAction( embed::getIconPixmap( "help" ), tr( "Help" ),
+	help_menu->addAction( embed::getIconPixmap( "help" ), tr( "Help" ),
 							this, SLOT( help() ) );
-	m_helpMenu->addAction( embed::getIconPixmap( "whatsthis" ),
+	help_menu->addAction( embed::getIconPixmap( "whatsthis" ),
 					tr( "What's this?" ),
 					this, SLOT( enterWhatsThisMode() ) );
 #ifdef QT4
-	m_helpMenu->addSeparator();
+	help_menu->addSeparator();
 #else
-	m_helpMenu->insertSeparator();
+	help_menu->insertSeparator();
 #endif
-	m_helpMenu->addAction( embed::getIconPixmap( "icon" ), tr( "About" ),
+	help_menu->addAction( embed::getIconPixmap( "icon" ), tr( "About" ),
 						this, SLOT( aboutLMMS() ) );
 
-	// now start mixer-thread
-	mixer::inst()->start(
-#if QT_VERSION >= 0x030200
-			QThread::HighestPriority
-#endif
-				);
-
 	// setup-dialog opened before?
 	if( !configManager::inst()->value( "app", "configured" ).toInt() )
 	{
 		// no, so show it that user can setup everything
-		setupDialog().exec();
-		configManager::inst()->setValue( "app", "configured", "1" );
+/*		setupDialog().exec();
+		configManager::inst()->setValue( "app", "configured", "1" );*/
 	}
 	// look whether mixer could use a audio-interface beside audioDummy
 	else if( mixer::inst()->audioDevName() == audioDummy::name() )
@@ -517,8 +491,8 @@ lmmsMainWin::lmmsMainWin() :
 
 lmmsMainWin::~lmmsMainWin()
 {
-	// first make sure mixer-thread is in our way during clean-up...
-	mixer::inst()->quitThread();
+	// first make sure, there're no mixing/audio-device-threads any more
+	mixer::inst()->stopProcessing();
 
 	// destroy editors with all their children
 	delete songEditor::inst();
@@ -542,6 +516,36 @@ lmmsMainWin::~lmmsMainWin()
 
 
 
+int lmmsMainWin::addWidgetToToolBar( QWidget * _w, int _row, int _col )
+{
+	int col = ( _col == -1 ) ? m_toolBarLayout->columnCount() + 6 : _col;
+	if( _w->height() > 32 || _row == -1 )
+	{
+#ifdef QT4
+		m_toolBarLayout->addWidget( _w, 0, col, 2, 1 );
+#else
+		m_toolBarLayout->addMultiCellWidget( _w, 0, 1, col, col );
+#endif
+	}
+	else
+	{
+		m_toolBarLayout->addWidget( _w, _row, col );
+	}
+	return( col );
+}
+
+
+
+
+void lmmsMainWin::addSpacingToToolBar( int _size )
+{
+	m_toolBarLayout->setColumnMinimumWidth( m_toolBarLayout->columnCount() +
+								6, _size );
+}
+
+
+
+
 void lmmsMainWin::resetWindowTitle( const QString & _add )
 {
 	QString title = _add;
@@ -564,6 +568,36 @@ void lmmsMainWin::resetWindowTitle( const QString & _add )
 
 
 
+void lmmsMainWin::keyPressEvent( QKeyEvent * _ke )
+{
+	switch( _ke->key() )
+	{
+		case Qt::Key_Control: m_keyMods.m_ctrl = TRUE; break;
+		case Qt::Key_Shift: m_keyMods.m_shift = TRUE; break;
+		case Qt::Key_Alt: m_keyMods.m_alt = TRUE; break;
+		default:
+			QMainWindow::keyPressEvent( _ke );
+	}
+}
+
+
+
+
+void lmmsMainWin::keyReleaseEvent( QKeyEvent * _ke )
+{
+	switch( _ke->key() )
+	{
+		case Qt::Key_Control: m_keyMods.m_ctrl = FALSE; break;
+		case Qt::Key_Shift: m_keyMods.m_shift = FALSE; break;
+		case Qt::Key_Alt: m_keyMods.m_alt = FALSE; break;
+		default:
+			QMainWindow::keyReleaseEvent( _ke );
+	}
+}
+
+
+
+
 void lmmsMainWin::closeEvent( QCloseEvent * _ce )
 {
 	if( songEditor::inst()->mayChangeProject() == TRUE )
@@ -600,7 +634,7 @@ void lmmsMainWin::createNewProjectFromTemplate( int _idx )
 	{
 		songEditor::inst()->createNewProjectFromTemplate(
 			configManager::inst()->projectsDir() + "templates/" +
-				m_templatesMenu->text( _idx ) + ".xml" );
+				m_templatesMenu->text( _idx ) + ".mpt" );
 	}
 #endif
 }
@@ -614,10 +648,10 @@ void lmmsMainWin::openProject( void )
 	{
 #ifdef QT4
 		QFileDialog ofd( this, tr( "Open project" ), "",
-					tr( "MultiMedia Project (*.xml)" ) );
+				tr( "MultiMedia Project (*.mmp *.xml)" ) );
 #else
 		QFileDialog ofd( QString::null,
-					tr( "MultiMedia Project (*.xml)" ),
+				tr( "MultiMedia Project (*.mmp *.xml)" ),
 							this, "", TRUE );
 		ofd.setWindowTitle( tr( "Open project" ) );
 #endif
@@ -655,9 +689,12 @@ bool lmmsMainWin::saveProjectAs( void )
 {
 #ifdef QT4
 	QFileDialog sfd( this, tr( "Save project" ), "",
-					tr( "MultiMedia Project (*.xml)" ) );
+				tr( "MultiMedia Project (*.mmp);;"
+				"MultiMedia Project Template (*.mpt)" ) );
 #else
-	QFileDialog sfd( QString::null, tr( "MultiMedia Project (*.xml)" ),
+	QFileDialog sfd( QString::null,
+				tr( "MultiMedia Project (*.mmp);;"
+					"MultiMedia Project Template (*.mpt)" ),
 							this, "", TRUE );
 	sfd.setWindowTitle( tr( "Save project" ) );
 #endif
@@ -701,7 +738,8 @@ bool lmmsMainWin::saveProjectAs( void )
 void lmmsMainWin::toggleBBEditorWin( void )
 {
 	if( bbEditor::inst()->isHidden() == TRUE ||
-			m_workspace->activeWindow() != bbEditor::inst() )
+		( m_workspace != NULL &&
+		  	m_workspace->activeWindow() != bbEditor::inst() ) )
 	{
 		bbEditor::inst()->show();
 		bbEditor::inst()->setFocus();
@@ -718,7 +756,8 @@ void lmmsMainWin::toggleBBEditorWin( void )
 void lmmsMainWin::toggleSongEditorWin( void )
 {
 	if( songEditor::inst()->isHidden() == TRUE ||
-			m_workspace->activeWindow() != songEditor::inst() )
+		( m_workspace != NULL &&
+			m_workspace->activeWindow() != songEditor::inst() ) )
 	{
 		songEditor::inst()->show();
 		songEditor::inst()->setFocus();
@@ -735,8 +774,8 @@ void lmmsMainWin::toggleSongEditorWin( void )
 void lmmsMainWin::toggleProjectNotesWin( void )
 {
 	if( songEditor::inst()->getProjectNotesWindow()->isHidden() == TRUE ||
-		m_workspace->activeWindow() !=
-				songEditor::inst()->getProjectNotesWindow() )
+		( m_workspace != NULL && m_workspace->activeWindow() !=
+				songEditor::inst()->getProjectNotesWindow() ) )
 	{
 		songEditor::inst()->getProjectNotesWindow()->show();
 		songEditor::inst()->getProjectNotesWindow()->setFocus();
@@ -753,7 +792,8 @@ void lmmsMainWin::toggleProjectNotesWin( void )
 void lmmsMainWin::togglePianoRollWin( void )
 {
 	if( pianoRoll::inst()->isHidden() == TRUE ||
-			m_workspace->activeWindow() != pianoRoll::inst() )
+		( m_workspace != NULL &&
+			m_workspace->activeWindow() != pianoRoll::inst() ) )
 	{
 		pianoRoll::inst()->show();
 		pianoRoll::inst()->setFocus();
@@ -789,11 +829,8 @@ void lmmsMainWin::help( void )
 					tr( "Currently there's no help "
 						"available in LMMS.\n"
 						"Please visit "
-						"http://lmms.sourceforge.net "
-						"and click on "
-						"\"Documentation\".\nThere "
-						"hopefully you'll find the "
-						"stuff you want to know..." ),
+						"http://wiki.mindrules.net "
+						"for documentation on LMMS." ),
 					QMessageBox::Ok );
 }
 
