@@ -1,8 +1,9 @@
 /*
  * mixer.cpp - audio-device-independent mixer for LMMS
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -22,6 +23,12 @@
  */
 
 
+#include <config.h>
+
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
 #include "mixer.h"
 #include "play_handle.h"
 #include "song_editor.h"
@@ -30,9 +37,10 @@
 #include "buffer_allocator.h"
 #include "debug.h"
 #include "config_mgr.h"
+#include "audio_port.h"
 
 #include "audio_device.h"
-#include "midi_device.h"
+#include "midi_client.h"
 
 // platform-specific audio-interface-classes
 #include "audio_alsa.h"
@@ -43,6 +51,7 @@
 
 // platform-specific midi-interface-classes
 #include "midi_alsa_raw.h"
+#include "midi_alsa_seq.h"
 #include "midi_oss.h"
 #include "midi_dummy.h"
 
@@ -50,27 +59,50 @@
 
 Uint32 SAMPLE_RATES[QUALITY_LEVELS] = { 44100, 88200 } ;
 
+
+class microTimer
+{
+public:
+	microTimer( void )
+	{
+		reset();
+	}
+
+	~microTimer()
+	{
+	}
+
+	void reset( void )
+	{
+		gettimeofday( &begin, NULL );
+	}
+
+	Uint32 elapsed( void ) const
+	{
+		struct timeval now;
+		gettimeofday( &now, NULL );
+		return( ( now.tv_sec - begin.tv_sec ) * 1000 * 1000 +
+					( now.tv_usec - begin.tv_usec ) );
+	}
+
+private:
+	struct timeval begin;
+
+} ;
+
+
+
 mixer * mixer::s_instanceOfMe = NULL;
 
 
 mixer::mixer() :
-#ifndef QT4
 	QObject(),
-#endif
-	QThread(),
-	m_silence(),
-#ifndef DISABLE_SURROUND
-	m_surroundSilence(),
-#endif
 	m_framesPerAudioBuffer( DEFAULT_BUFFER_SIZE ),
-	m_buffer1( NULL ),
-	m_buffer2( NULL ),
 	m_curBuf( NULL ),
 	m_nextBuf( NULL ),
-	m_discardCurBuf( FALSE ),
+	m_cpuLoad( 0 ),
 	m_qualityLevel( DEFAULT_QUALITY_LEVEL ),
-	m_masterOutput( 1.0f ),
-	m_quit( FALSE ),
+	m_masterGain( 1.0f ),
 	m_audioDev( NULL ),
 	m_oldAudioDev( NULL )
 {
@@ -90,47 +122,21 @@ mixer::mixer() :
 				QString::number( m_framesPerAudioBuffer ) );
 	}
 
-	m_buffer1 = bufferAllocator::alloc<surroundSampleFrame>(
+	m_curBuf = bufferAllocator::alloc<surroundSampleFrame>(
 						m_framesPerAudioBuffer );
-	m_buffer2 = bufferAllocator::alloc<surroundSampleFrame>(
+	m_nextBuf = bufferAllocator::alloc<surroundSampleFrame>(
 						m_framesPerAudioBuffer );
 
-	m_curBuf = m_buffer1;
-	m_nextBuf = m_buffer2;
-
 
 	m_audioDev = tryAudioDevices();
-	m_midiDev = tryMIDIDevices();
+	m_midiClient = tryMIDIClients();
 
 
-	for( int i = 0; i < MAX_SAMPLE_PACKETS; ++i )
-	{
-		m_samplePackets[i].m_buffer = NULL;
-		m_samplePackets[i].m_state = samplePacket::UNUSED;
-	}
-
-	m_silence = bufferAllocator::alloc<sampleFrame>(
-						m_framesPerAudioBuffer );
-	m_surroundSilence = bufferAllocator::alloc<surroundSampleFrame>(
-						m_framesPerAudioBuffer );
-
-	for( Uint32 frame = 0; frame < m_framesPerAudioBuffer; ++frame )
-	{
-		for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
-		{
-			m_silence[frame][chnl] = 0.0f;
-		}
-#ifndef DISABLE_SURROUND
-		for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
-		{
-			m_surroundSilence[frame][chnl] = 0.0f;
-		}
-#endif
-	}
-
 	// now clear our two output-buffers before using them...
-	clearAudioBuffer( m_buffer1, m_framesPerAudioBuffer );
-	clearAudioBuffer( m_buffer2, m_framesPerAudioBuffer );
+	clearAudioBuffer( m_curBuf, m_framesPerAudioBuffer );
+	clearAudioBuffer( m_nextBuf, m_framesPerAudioBuffer );
+
+	m_audioDev->startProcessing();
 
 }
 
@@ -140,175 +146,108 @@ mixer::mixer() :
 mixer::~mixer()
 {
 	delete m_audioDev;
+	delete m_midiClient;
 
-	bufferAllocator::free( m_buffer1 );
-	bufferAllocator::free( m_buffer2 );
-
-	for( int i = 0; i < MAX_SAMPLE_PACKETS; ++i )
-	{
-		if( m_samplePackets[i].m_state != samplePacket::UNUSED )
-		{
-			bufferAllocator::free( m_samplePackets[i].m_buffer );
-		}
-	}
+	bufferAllocator::free( m_curBuf );
+	bufferAllocator::free( m_nextBuf );
 }
 
 
 
 
-void mixer::quitThread( void )
+void mixer::stopProcessing( void )
 {
-	// make sure there're no mutexes locked anymore...
-	m_safetySyncMutex.unlock();
-	m_devMutex.unlock();
-
-	// now tell mixer-thread to quit
-	m_quit = TRUE;
-
-	wait( 1000 );
-	terminate();
+	m_audioDev->stopProcessing();
 }
 
 
 
 
-void mixer::run( void )
+const surroundSampleFrame * mixer::renderNextBuffer( void )
 {
+	microTimer timer;
 
-	while( m_quit == FALSE )
+	// now we have to make sure no other thread does anything bad
+	// while we're acting...
+	m_mixMutex.lock();
+
+	// remove all play-handles that have to be deleted and delete
+	// them if they still exist...
+	// maybe this algorithm could be optimized...
+	while( !m_playHandlesToRemove.empty() )
 	{
-
-		// remove all play-handles that have to be deleted and delete
-		// them if they still exist...
-		// maybe this algorithm could be optimized...
-		while( !m_playHandlesToRemove.isEmpty() )
-		{
-			playHandleVector::iterator it = m_playHandles.begin();
-
-			while( it != m_playHandles.end() )
-			{
-				if( *it == m_playHandlesToRemove.first() )
-				{
-					m_playHandles.erase( it );
-					delete m_playHandlesToRemove.first();
-					break;
-				}
-				++it;
-			}
-
-			m_playHandlesToRemove.erase(
-						m_playHandlesToRemove.begin() );
-
-		}
-
-		// now we have to make sure no other thread does anything bad
-		// while we're acting...
-		m_safetySyncMutex.lock();
-
-/*		following code is faster but unstable since using iterators
-		while deleting from vector is dangerous and often leads to
-		undefined results...
-
 		playHandleVector::iterator it = m_playHandles.begin();
+
 		while( it != m_playHandles.end() )
 		{
-			if( ( *it )->done() )
+			if( *it == m_playHandlesToRemove.front() )
 			{
-				// delete all play-handles which have
-				// played completely now
-				delete *it;
 				m_playHandles.erase( it );
+				delete m_playHandlesToRemove.front();
+				break;
 			}
-			else
-			{
-				// play all uncompletely-played play-handles...
-				( *it )->play();
-				++it;
-			}
-		}*/
-		csize idx = 0;
-		while( idx < m_playHandles.size() )
-		{
-			register playHandle * n = m_playHandles[idx];
-			if( n->done() )
-			{
-				// delete all play-handles which have
-				// played completely now
-				delete n;
-				m_playHandles.erase( m_playHandles.begin() +
-									idx );
-			}
-			else
-			{
-				// play all uncompletely-played play-handles...
-				n->play();
-				++idx;
-			}
+			++it;
 		}
 
-		songEditor::inst()->processNextBuffer();
+		m_playHandlesToRemove.erase(
+					m_playHandlesToRemove.begin() );
+	}
 
-		// check for samples-packets that have to be mixed in
-		// the current audio-buffer
-		for( int i = 0; i < MAX_SAMPLE_PACKETS; ++i )
+	// now swap the buffers... current buffer becomes next (last)
+	// buffer and the next buffer becomes current (first) buffer
+	qSwap( m_curBuf, m_nextBuf );
+
+	// clear last audio-buffer
+	clearAudioBuffer( m_curBuf, m_framesPerAudioBuffer );
+
+
+	csize idx = 0;
+	while( idx < m_playHandles.size() )
+	{
+		register playHandle * n = m_playHandles[idx];
+		if( n->done() )
 		{
-			if( m_samplePackets[i].m_state == samplePacket::READY )
-			{
-				if( m_samplePackets[i].m_framesAhead <=
-							m_framesPerAudioBuffer )
-				{
-					// found one! mix it...
-					mixSamplePacket( &m_samplePackets[i] );
-					// now this audio-sample can be used
-					// again
-					bufferAllocator::free(
-						m_samplePackets[i].m_buffer );
-					m_samplePackets[i].m_state =
-							samplePacket::UNUSED;
-				}
-				else
-				{
-					m_samplePackets[i].m_framesAhead -=
-							m_framesPerAudioBuffer;
-				}
-
-			}
-
-		}
-
-		if( !m_discardCurBuf )
-		{
-			m_devMutex.lock();
-			// write actual data to our current output-device
-			// (blocking!)
-			m_audioDev->writeBuffer( m_curBuf,
-							m_framesPerAudioBuffer,
-						SAMPLE_RATES[m_qualityLevel],
-							m_masterOutput );
-			m_devMutex.unlock();
+			// delete all play-handles which have
+			// played completely now
+			delete n;
+			m_playHandles.erase( m_playHandles.begin() +
+								idx );
 		}
 		else
 		{
-			m_discardCurBuf = FALSE;
+			// play all uncompletely-played play-handles...
+			n->play();
+			++idx;
 		}
-
-		emit nextAudioBuffer( m_curBuf, m_framesPerAudioBuffer );
-
-
-		m_safetySyncMutex.unlock();
-
-
-		// clear last audio-buffer
-		clearAudioBuffer( m_curBuf, m_framesPerAudioBuffer );
-
-		// now swap the buffers... current buffer becomes next (last)
-		// buffer and the next buffer becomes current (first) buffer
-		qSwap( m_curBuf, m_nextBuf );
-
-		// and trigger LFOs
-		envelopeAndLFOWidget::triggerLFO();
 	}
 
+	songEditor::inst()->processNextBuffer();
+
+	for( vvector<audioPort *>::iterator it = m_audioPorts.begin();
+					it != m_audioPorts.end(); ++it )
+	{
+		if( ( *it )->m_bufferUsage != audioPort::NONE )
+		{
+			processBuffer( ( *it )->firstBuffer(),
+					( *it )->nextFxChannel() );
+			( *it )->nextPeriod();
+		}
+	}
+
+
+	emit nextAudioBuffer( m_curBuf, m_framesPerAudioBuffer );
+
+	m_mixMutex.unlock();
+
+
+	// and trigger LFOs
+	envelopeAndLFOWidget::triggerLFO();
+
+	const float new_cpu_load = timer.elapsed() / 10000.0f * sampleRate() /
+							m_framesPerAudioBuffer;
+	m_cpuLoad = tLimit( (int) ( new_cpu_load + m_cpuLoad ) / 2, 0, 100 );
+
+	return( m_curBuf );
 }
 
 
@@ -318,7 +257,7 @@ void mixer::run( void )
 // all remaining notes etc. would be played until their end
 void mixer::clear( void )
 {
-	m_midiDev->noteOffAll();
+	// TODO: m_midiClient->noteOffAll();
 	for( playHandleVector::iterator it = m_playHandles.begin();
 					it != m_playHandles.end(); ++it )
 	{
@@ -331,21 +270,7 @@ void mixer::clear( void )
 
 void FASTCALL mixer::clearAudioBuffer( sampleFrame * _ab, Uint32 _frames )
 {
-	if( _frames == m_framesPerAudioBuffer )
-	{
-		memcpy( _ab, m_silence, m_framesPerAudioBuffer *
-							BYTES_PER_FRAME );
-	}
-	else
-	{
-		for( Uint32 frame = 0; frame < _frames; ++frame )
-		{
-			for( Uint8 ch = 0; ch < DEFAULT_CHANNELS; ++ch )
-			{
-				_ab[frame][ch] = 0.0f;
-			}
-		}
-	}
+	memset( _ab, 0, sizeof( *_ab ) * _frames );
 }
 
 
@@ -354,73 +279,52 @@ void FASTCALL mixer::clearAudioBuffer( sampleFrame * _ab, Uint32 _frames )
 void FASTCALL mixer::clearAudioBuffer( surroundSampleFrame * _ab,
 								Uint32 _frames )
 {
-	if( _frames == m_framesPerAudioBuffer )
-	{
-		memcpy( _ab, m_surroundSilence, m_framesPerAudioBuffer *
-						BYTES_PER_SURROUND_FRAME );
-	}
-	else
-	{
-		for( Uint32 frame = 0; frame < _frames; ++frame )
-		{
-			for( Uint8 ch = 0; ch < DEFAULT_CHANNELS; ++ch )
-			{
-				_ab[frame][ch] = 0.0f;
-			}
-		}
-	}
+	memset( _ab, 0, sizeof( *_ab ) * _frames );
 }
 #endif
 
 
 
-void FASTCALL mixer::addBuffer( sampleFrame * _buf, Uint32 _frames,
+void FASTCALL mixer::bufferToPort( sampleFrame * _buf, Uint32 _frames,
 						Uint32 _frames_ahead,
-						volumeVector & _volume_vector )
+						volumeVector & _volume_vector,
+						audioPort * _port )
 {
-#ifdef LMMS_DEBUG
-	bool success = FALSE;
-#endif
-	for ( Uint16 i = 0; i < MAX_SAMPLE_PACKETS; ++i )
+	Uint32 start_frame = _frames_ahead % m_framesPerAudioBuffer;
+	Uint32 end_frame = start_frame + _frames;
+	Uint32 loop1_frame = tMin( end_frame, m_framesPerAudioBuffer );
+	for( Uint32 frame = start_frame; frame < loop1_frame; ++frame )
 	{
-		if( m_samplePackets[i].m_state == samplePacket::UNUSED )
+		for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
 		{
-			m_samplePackets[i].m_state = samplePacket::FILLING;
-			m_samplePackets[i].m_frames = _frames;//m_framesPerAudioBuffer;
-			m_samplePackets[i].m_framesDone  = 0;
-			m_samplePackets[i].m_framesAhead  = _frames_ahead;
-
-			m_samplePackets[i].m_buffer =
-				bufferAllocator::alloc<surroundSampleFrame>(
-						m_framesPerAudioBuffer );
-			// now we have to make a surround-buffer out of a
-			// stereo-buffer (could be done more easily if there
-			// would be no volume-vector...)
-			for( Uint32 frame = 0; frame < _frames/*m_framesPerAudioBuffer*/;
-								++frame )
-			{
-				for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS;
-									++chnl )
-				{
-			m_samplePackets[i].m_buffer[frame][chnl] =
-					_buf[frame][chnl%DEFAULT_CHANNELS] *
+			_port->firstBuffer()[frame][chnl] +=
+					_buf[frame - start_frame][chnl %
+							DEFAULT_CHANNELS] *
 						_volume_vector.vol[chnl];
-				}
-			}
-
-			m_samplePackets[i].m_state = samplePacket::READY;
-#ifdef LMMS_DEBUG
-			success = TRUE;
-#endif
-			break;
 		}
 	}
-#ifdef LMMS_DEBUG
-	if( success == FALSE )
+	if( end_frame > m_framesPerAudioBuffer )
 	{
-		qWarning( "No sample-packets left in mixer::addBuffer(...)!\n" );
+		Uint32 frames_done = m_framesPerAudioBuffer - start_frame;
+		end_frame = tMin( end_frame -= m_framesPerAudioBuffer,
+						m_framesPerAudioBuffer );
+		for( Uint32 frame = 0; frame < end_frame; ++frame )
+		{
+			for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
+			{
+				_port->secondBuffer()[frame][chnl] +=
+					_buf[frames_done + frame][chnl %
+							DEFAULT_CHANNELS] *
+						_volume_vector.vol[chnl];
+			}
+		}
+		_port->m_bufferUsage = audioPort::BOTH;
 	}
-#endif
+	else if( _port->m_bufferUsage == audioPort::NONE )
+	{
+		_port->m_bufferUsage = audioPort::FIRST;
+	}
+
 }
 
 
@@ -428,8 +332,6 @@ void FASTCALL mixer::addBuffer( sampleFrame * _buf, Uint32 _frames,
 
 void mixer::setHighQuality( bool _hq_on )
 {
-	m_safetySyncMutex.lock();
-
 	// delete (= close) our audio-device
 	delete m_audioDev;
 
@@ -444,8 +346,7 @@ void mixer::setHighQuality( bool _hq_on )
 	}
 	// and re-open device
 	m_audioDev = tryAudioDevices();
-
-	m_safetySyncMutex.unlock();
+	m_audioDev->startProcessing();
 
 	emit( sampleRateChanged() );
 
@@ -456,8 +357,7 @@ void mixer::setHighQuality( bool _hq_on )
 
 void FASTCALL mixer::setAudioDevice( audioDevice * _dev, bool _hq )
 {
-
-	m_devMutex.lock();
+	m_audioDev->stopProcessing();
 
 	m_oldAudioDev = m_audioDev;
 
@@ -474,9 +374,6 @@ void FASTCALL mixer::setAudioDevice( audioDevice * _dev, bool _hq )
 
 	m_qualityLevel = _hq ? 1 : 0;
 	emit sampleRateChanged();
-
-	m_devMutex.unlock();
-
 }
 
 
@@ -484,11 +381,10 @@ void FASTCALL mixer::setAudioDevice( audioDevice * _dev, bool _hq )
 
 void mixer::restoreAudioDevice( void )
 {
-	m_devMutex.lock();
-
 	if( m_oldAudioDev != NULL )
 	{
-		delete m_audioDev;
+		delete m_audioDev;	// dtor automatically calls
+					// stopProcessing()
 		m_audioDev = m_oldAudioDev;
 		for( Uint8 qli = 0; qli < QUALITY_LEVELS; ++qli )
 		{
@@ -500,10 +396,8 @@ void mixer::restoreAudioDevice( void )
 			}
 		}
 		m_oldAudioDev = NULL;
-		m_discardCurBuf = TRUE;
+		m_audioDev->startProcessing();
 	}
-
-	m_devMutex.unlock();
 }
 
 
@@ -522,56 +416,9 @@ void mixer::checkValidityOfPlayHandles( void )
 
 
 
-void FASTCALL mixer::mixSamplePacket( samplePacket * _sp )
-{
-	Uint32 start_frame = _sp->m_framesAhead % m_framesPerAudioBuffer;
-	Uint32 end_frame = start_frame + _sp->m_frames;//m_framesPerAudioBuffer;
-
-	if( end_frame <= m_framesPerAudioBuffer )
-	{
-		for( Uint32 frame = start_frame; frame < end_frame; ++frame )
-		{
-			for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
-			{
-				m_curBuf[frame][chnl] +=
-					_sp->m_buffer[frame-start_frame][chnl];
-			}
-		}
-	}
-	else
-	{
-		for( Uint32 frame = start_frame; frame <
-					m_framesPerAudioBuffer; ++frame )
-		{
-			for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
-			{
-				m_curBuf[frame][chnl] +=
-					_sp->m_buffer[frame-start_frame][chnl];
-			}
-		}
-
-		Uint32 frames_done = m_framesPerAudioBuffer - start_frame;
-		end_frame = tMin( end_frame -= m_framesPerAudioBuffer,
-						m_framesPerAudioBuffer );
-
-		for( Uint32 frame = 0; frame < end_frame; ++frame )
-		{
-			for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
-			{
-				m_nextBuf[frame][chnl] +=
-					_sp->m_buffer[frames_done+frame][chnl];
-			}
-		}
-	}
-}
-
-
-
 
 audioDevice * mixer::tryAudioDevices( void )
 {
-	//m_discardCurBuf = TRUE;
-
 	bool success_ful = FALSE;
 	audioDevice * dev = NULL;
 	QString dev_name = configManager::inst()->value( "mixer", "audiodev" );
@@ -654,45 +501,71 @@ audioDevice * mixer::tryAudioDevices( void )
 
 
 
-midiDevice * mixer::tryMIDIDevices( void )
+midiClient * mixer::tryMIDIClients( void )
 {
-	QString dev_name = configManager::inst()->value( "mixer", "mididev" );
+	QString client_name = configManager::inst()->value( "mixer",
+								"mididev" );
 
 #ifdef ALSA_SUPPORT
-	if( dev_name == midiALSARaw::name() || dev_name == "" )
+	if( client_name == midiALSASeq::name() || client_name == "" )
 	{
-		midiALSARaw * malsa = new midiALSARaw();
-		if( malsa->isRunning() )
+		midiALSASeq * malsas = new midiALSASeq();
+		if( malsas->isRunning() )
 		{
-			m_midiDevName = midiALSARaw::name();
-			return( malsa );
+			m_midiClientName = midiALSASeq::name();
+			return( malsas );
 		}
-		delete malsa;
+		delete malsas;
+	}
+
+	if( client_name == midiALSARaw::name() || client_name == "" )
+	{
+		midiALSARaw * malsar = new midiALSARaw();
+		if( malsar->isRunning() )
+		{
+			m_midiClientName = midiALSARaw::name();
+			return( malsar );
+		}
+		delete malsar;
 	}
 #endif
 
 #ifdef OSS_SUPPORT
-	if( dev_name == midiOSS::name() || dev_name == "" )
+	if( client_name == midiOSS::name() || client_name == "" )
 	{
 		midiOSS * moss = new midiOSS();
 		if( moss->isRunning() )
 		{
-			m_midiDevName = midiOSS::name();
+			m_midiClientName = midiOSS::name();
 			return( moss );
 		}
 		delete moss;
 	}
 #endif
 
-	printf( "Couldn't open a MIDI-device, neither with ALSA nor with "
-		"OSS. Will use dummy-MIDI-device.\n" );
+	printf( "Couldn't create MIDI-client, neither with ALSA nor with "
+		"OSS. Will use dummy-MIDI-client.\n" );
 
-	m_midiDevName = midiDummy::name();
+	m_midiClientName = midiDummy::name();
 
 	return( new midiDummy() );
 }
 
 
 
+
+void mixer::processBuffer( surroundSampleFrame * _buf, fxChnl/* _fx_chnl */ )
+{
+	// TODO: effect-implementation
+	for( Uint32 frame = 0; frame < m_framesPerAudioBuffer; ++frame )
+	{
+		for( Uint8 chnl = 0; chnl < SURROUND_CHANNELS; ++chnl )
+		{
+			m_curBuf[frame][chnl] += _buf[frame][chnl];
+		}
+	}
+}
+
+
 #include "mixer.moc"
 
