@@ -2,8 +2,9 @@
  * piano_roll.cpp - implementation of piano-roll which is used for actual
  *                  writing of melodies
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -32,29 +33,44 @@
 #include <QPainter>
 #include <QKeyEvent>
 #include <QWheelEvent>
+#include <QComboBox>
+#include <QLayout>
+#include <QLabel>
 
 #else
 
 #include <qapplication.h>
 #include <qbuttongroup.h>
 #include <qpainter.h>
+#include <qcombobox.h>
+#include <qlayout.h>
+#include <qlabel.h>
 
 #define setChecked setOn
 
 #endif
 
 
+#ifndef __USE_XOPEN
+#define __USE_XOPEN
+#endif
+
+#include <math.h>
+
+
 #include "piano_roll.h"
 #include "song_editor.h"
+#include "lmms_main_win.h"
 #include "pattern.h"
 #include "embed.h"
-#include "crystal_button.h"
 #include "pixmap_button.h"
-#include "note_play_handle.h"
 #include "templates.h"
+#include "gui_templates.h"
 #include "timeline.h"
 #include "channel_track.h"
 #include "tooltip.h"
+#include "midi.h"
+#include "tool_button.h"
 
 
 extern tones whiteKeys[];	// defined in piano_widget.cpp
@@ -77,7 +93,7 @@ const int KEY_LINE_HEIGHT = 12;
 const int OCTAVE_HEIGHT = KEY_LINE_HEIGHT * NOTES_PER_OCTAVE;	// = 12 * 12;
 
 const int PR_BOTTOM_MARGIN = SCROLLBAR_SIZE;
-const int PR_TOP_MARGIN = 66;
+const int PR_TOP_MARGIN = 48;
 
 // width of area used for resizing (the grip at the end of a note)
 const int RESIZE_AREA_WIDTH = 3;
@@ -95,8 +111,6 @@ pianoRoll * pianoRoll::s_instanceOfMe = NULL;
 QPixmap * pianoRoll::s_whiteKeySmallPm = NULL;
 QPixmap * pianoRoll::s_whiteKeyBigPm = NULL;
 QPixmap * pianoRoll::s_blackKeyPm = NULL;
-QPixmap * pianoRoll::s_artwork1 = NULL;
-QPixmap * pianoRoll::s_artwork2 = NULL;
 QPixmap * pianoRoll::s_toolDraw = NULL;
 QPixmap * pianoRoll::s_toolErase = NULL;
 QPixmap * pianoRoll::s_toolSelect = NULL;
@@ -111,6 +125,8 @@ pianoRoll::pianoRollKeyTypes pianoRoll::prKeyOrder[] =
 } ;
 
 
+const int DEFAULT_PR_PPT = KEY_LINE_HEIGHT * DEFAULT_STEPS_PER_TACT;
+
 
 pianoRoll::pianoRoll( void ) :
 	QWidget( lmmsMainWin::inst()->workspace() ),
@@ -122,11 +138,10 @@ pianoRoll::pianoRoll( void ) :
 	m_moveStartKey( 0 ),
 	m_moveStartTact64th( 0 ),
 	m_notesEditHeight( 100 ),
-	m_ppt( KEY_LINE_HEIGHT * MAX_BEATS_PER_TACT ),
+	m_ppt( DEFAULT_PR_PPT ),
 	m_lenOfNewNotes( midiTime( 0, 16 ) ),
-	m_shiftPressed( FALSE ),
-	m_controlPressed( FALSE ),
 	m_startKey( INITIAL_START_KEY ),
+	m_keyMouseOver( INITIAL_START_KEY ),
 	m_lastKey( 0 ),
 	m_editMode( DRAW ),
 	m_scrollBack( FALSE )
@@ -147,16 +162,6 @@ pianoRoll::pianoRoll( void ) :
 		s_blackKeyPm = new QPixmap( embed::getIconPixmap(
 							"pr_black_key" ) );
 	}
-	if( s_artwork1 == NULL )
-	{
-		s_artwork1 = new QPixmap( embed::getIconPixmap(
-							"pr_artwork1" ) );
-	}
-	if( s_artwork2 == NULL )
-	{
-		s_artwork2 = new QPixmap( embed::getIconPixmap(
-							"pr_artwork2" ) );
-	}
 	if( s_toolDraw == NULL )
 	{
 		s_toolDraw = new QPixmap( embed::getIconPixmap(
@@ -183,39 +188,48 @@ pianoRoll::pianoRoll( void ) :
 	lmmsMainWin::inst()->workspace()->addWindow( this );
 #endif
 
+	// add time-line
+	m_timeLine = new timeLine( WHITE_KEY_WIDTH, 32, m_ppt,
+					songEditor::inst()->getPlayPos(
+						songEditor::PLAY_PATTERN ),
+						m_currentPosition, this );
+	connect( this, SIGNAL( positionChanged( const midiTime & ) ),
+		m_timeLine, SLOT( updatePosition( const midiTime & ) ) );
+	connect( m_timeLine, SIGNAL( positionChanged( const midiTime & ) ),
+			this, SLOT( updatePosition( const midiTime & ) ) );
+
+
+	m_toolBar = new QWidget( this );
+	m_toolBar->setFixedHeight( 32 );
+	m_toolBar->move( 0, 0 );
+#ifdef QT4
+	QPalette pal;
+	pal.setBrush( m_toolBar->backgroundRole(), QBrush(
+				embed::getIconPixmap( "toolbar_bg" ) ) );
+	m_toolBar->setPalette( pal );
+#else
+	m_toolBar->setPaletteBackgroundPixmap( embed::getIconPixmap(
+							"toolbar_bg" ) );
+#endif
+
+	QHBoxLayout * tb_layout = new QHBoxLayout( m_toolBar );
+	tb_layout->setMargin( 0 );
+	tb_layout->setSpacing( 0 );
+
+
 	// init control-buttons at the top
-	m_playButton = new pixmapButton( this );
-	m_playButton->move( 8, 7 );
-	m_playButton->setCheckable( FALSE );
-	m_playButton->setActiveGraphic( embed::getIconPixmap( "play" ) );
-	m_playButton->setInactiveGraphic( embed::getIconPixmap( "play" ) );
-	m_playButton->setBgGraphic( embed::getIconPixmap( "pr_play_ctrl_bg" ) );
-	connect( m_playButton, SIGNAL( clicked() ), this, SLOT( play() ) );
 
-	m_recordButton = new pixmapButton( this );
-	m_recordButton->move( 50, 7 );
-	m_recordButton->setCheckable( FALSE );
-	m_recordButton->setActiveGraphic( embed::getIconPixmap( "record" ) );
-	m_recordButton->setInactiveGraphic( embed::getIconPixmap( "record" ) );
-	m_recordButton->setBgGraphic(
-				embed::getIconPixmap( "pr_play_ctrl_bg" ) );
-	connect( m_recordButton, SIGNAL( clicked() ), this, SLOT( record() ) );
+	m_playButton = new toolButton( embed::getIconPixmap( "play" ),
+				tr( "Play/pause current pattern (Space)" ),
+					this, SLOT( play() ), m_toolBar );
 
-	m_stopButton = new pixmapButton( this );
-	m_stopButton->move( 92, 7 );
-	m_stopButton->setCheckable( FALSE );
-	m_stopButton->setActiveGraphic( embed::getIconPixmap( "stop" ) );
-	m_stopButton->setInactiveGraphic( embed::getIconPixmap( "stop" ) );
-	m_stopButton->setBgGraphic( embed::getIconPixmap( "pr_play_ctrl_bg" ) );
-	connect( m_stopButton, SIGNAL( clicked() ), this, SLOT( stop() ) );
+	m_recordButton = new toolButton( embed::getIconPixmap( "record" ),
+			tr( "Record notes from MIDI-device/channel-piano" ),
+					this, SLOT( record() ), m_toolBar );
 
-	toolTip::add( m_playButton,
-			tr( "Play/pause current pattern (Space)" ) );
-	toolTip::add( m_recordButton,
-			tr( "Record notes from MIDI-device to current "
-								"pattern" ) );
-	toolTip::add( m_stopButton,
-			tr( "Stop playing of current pattern (Space)" ) );
+	m_stopButton = new toolButton( embed::getIconPixmap( "stop" ),
+				tr( "Stop playing of current pattern (Space)" ),
+					this, SLOT( stop() ), m_toolBar );
 
 #ifdef QT4
 	m_playButton->setWhatsThis(
@@ -233,8 +247,8 @@ pianoRoll::pianoRoll( void ) :
 		tr( "Click here, if you want to record notes from a MIDI-"
 			"device or the virtual test-piano of the according "
 			"channel-window to the current pattern. When recording "
-			 "all notes you play will be written to this pattern "
-			"and you can edit, play etc. them afterwards." ) );
+			"all notes you play will be written to this pattern "
+			"and you can play and edit them afterwards." ) );
 #ifdef QT4
 	m_stopButton->setWhatsThis(
 #else
@@ -256,33 +270,31 @@ pianoRoll::pianoRoll( void ) :
 						SLOT( verScrolled( int ) ) );
 
 	// init edit-buttons at the top
-	m_drawButton = new crystalButton( embed::getIconPixmap( "pr_tool_bg" ),
-						embed::getIconPixmap(
-						"pr_tool_draw" ), this );
-	m_drawButton->move( 170, 1 );
-	m_drawButton->setActiveButtonBg( embed::getIconPixmap(
-						"pr_tool_bg_inset" ) );
+	m_drawButton = new toolButton( embed::getIconPixmap( "pr_tool_draw" ),
+					tr( "Draw mode (D)" ),
+					this, SLOT( drawButtonToggled() ),
+					m_toolBar );
+	m_drawButton->setCheckable( TRUE );
 	m_drawButton->setChecked( TRUE );
 
-	m_eraseButton = new crystalButton( embed::getIconPixmap( "pr_tool_bg" ),
-						embed::getIconPixmap(
-						"pr_tool_erase" ), this );
-	m_eraseButton->move( 220, 1 );
-	m_eraseButton->setActiveButtonBg( embed::getIconPixmap(
-							"pr_tool_bg_inset" ) );
-	m_selectButton = new crystalButton( embed::getIconPixmap(
-						"pr_tool_bg" ),
-						embed::getIconPixmap(
-						"pr_tool_select" ), this );
-	m_selectButton->move( 270, 1 );
-	m_selectButton->setActiveButtonBg( embed::getIconPixmap(
-							"pr_tool_bg_inset" ) );
-	m_moveButton = new crystalButton( embed::getIconPixmap( "pr_tool_bg" ),
-						embed::getIconPixmap(
-						"pr_tool_move" ), this );
-	m_moveButton->move( 320, 1 );
-	m_moveButton->setActiveButtonBg( embed::getIconPixmap(
-							"pr_tool_bg_inset" ) );
+	m_eraseButton = new toolButton( embed::getIconPixmap( "pr_tool_erase" ),
+					tr( "Erase mode (E)" ),
+					this, SLOT( eraseButtonToggled() ),
+					m_toolBar );
+	m_eraseButton->setCheckable( TRUE );
+
+	m_selectButton = new toolButton( embed::getIconPixmap(
+							"pr_tool_select" ),
+					tr( "Select mode (S)" ),
+					this, SLOT( selectButtonToggled() ),
+					m_toolBar );
+	m_selectButton->setCheckable( TRUE );
+
+	m_moveButton = new toolButton( embed::getIconPixmap( "pr_tool_move" ),
+					tr( "Move selection mode (M)" ),
+					this, SLOT( moveButtonToggled() ),
+					m_toolBar );
+	m_moveButton->setCheckable( TRUE );
 
 	QButtonGroup * tool_button_group = new QButtonGroup( this );
 	tool_button_group->addButton( m_drawButton );
@@ -294,26 +306,6 @@ pianoRoll::pianoRoll( void ) :
 	tool_button_group->hide();
 #endif
 
-	connect( m_drawButton, SIGNAL( toggled( bool ) ), this,
-					SLOT( drawButtonToggled( bool ) ) );
-	connect( m_eraseButton, SIGNAL( toggled( bool ) ), this,
-					SLOT( eraseButtonToggled( bool ) ) );
-	connect( m_selectButton, SIGNAL( toggled( bool ) ), this,
-					SLOT( selectButtonToggled( bool ) ) );
-	connect( m_moveButton, SIGNAL( toggled( bool ) ), this,
-					SLOT( moveButtonToggled( bool ) ) );
-
-	toolTip::add( m_drawButton,
-			tr( "Click if you want to draw, resize or move single "
-							"notes (= key 'D')" ) );
-	toolTip::add( m_eraseButton,
-			tr( "Click if you want to erase single notes "
-							"(= key 'E')" ) );
-	toolTip::add( m_selectButton,
-			tr( "Click if you want to select notes (= key 'S')" ) );
-	toolTip::add( m_moveButton,
-			tr( "Click if you want to move selected notes "
-							"(= key 'M')" ) );
 #ifdef QT4
 	m_drawButton->setWhatsThis(
 #else
@@ -352,42 +344,22 @@ pianoRoll::pianoRoll( void ) :
 			"mode. You can also press 'M' on your keyboard to "
 			"activate this mode." ) );
 
+	m_cutButton = new toolButton( embed::getIconPixmap( "edit_cut" ),
+					tr( "Cut selected notes (Ctrl+X)" ),
+					this, SLOT( cutSelectedNotes() ),
+					m_toolBar );
 
-	m_cutButton = new crystalButton( embed::getIconPixmap( "pr_tool_bg" ),
-						embed::getIconPixmap(
-							"pr_edit_cut" ), this );
-	m_cutButton->move( 390, 1 );
-	m_cutButton->setActiveButtonBg( embed::getIconPixmap(
-							"pr_tool_bg_inset" ) );
-	m_cutButton->setCheckable( FALSE );
-	m_copyButton = new crystalButton( embed::getIconPixmap( "pr_tool_bg" ),
-						embed::getIconPixmap(
-							"pr_edit_copy" ),
-									this );
-	m_copyButton->move( 440, 1 );
-	m_copyButton->setActiveButtonBg( embed::getIconPixmap(
-							"pr_tool_bg_inset" ) );
-	m_copyButton->setCheckable( FALSE );
-	m_pasteButton = new crystalButton( embed::getIconPixmap( "pr_tool_bg" ),
-						embed::getIconPixmap(
-							"pr_edit_paste" ),
-									this );
-	m_pasteButton->move( 490, 1 );
-	m_pasteButton->setActiveButtonBg( embed::getIconPixmap(
-							"pr_tool_bg_inset" ) );
-	m_pasteButton->setCheckable( FALSE );
+	m_copyButton = new toolButton( embed::getIconPixmap( "edit_copy" ),
+					tr( "Copy selected notes (Ctrl+C)" ),
+					this, SLOT( copySelectedNotes() ),
+					m_toolBar );
 
-	connect( m_cutButton, SIGNAL( clicked() ), this,
-						SLOT( cutSelectedNotes() ) );
-	connect( m_copyButton, SIGNAL( clicked() ), this,
-						SLOT( copySelectedNotes() ) );
-	connect( m_pasteButton, SIGNAL( clicked() ), this,
-						SLOT( pasteNotes() ) );
+	m_pasteButton = new toolButton( embed::getIconPixmap( "edit_paste" ),
+					tr( "Paste notes from clipboard "
+								"(Ctrl+V)" ),
+					this, SLOT( pasteNotes() ),
+					m_toolBar );
 
-	toolTip::add( m_cutButton, tr( "Cut selected notes (Ctrl+X)" ) );
-	toolTip::add( m_copyButton, tr( "Copy selected notes (Ctrl+C)" ) );
-	toolTip::add( m_pasteButton, tr( "Paste notes from clipboard "
-								"(Ctrl+V)" ) );
 #ifdef QT4
 	m_cutButton->setWhatsThis(
 #else
@@ -413,6 +385,51 @@ pianoRoll::pianoRoll( void ) :
 			"pasted at the first visible tact." ) );
 
 
+
+	QLabel * zoom_lbl = new QLabel( m_toolBar );
+	zoom_lbl->setPixmap( embed::getIconPixmap( "zoom" ) );
+
+	// setup zooming-stuff
+	m_zoomingComboBox = new QComboBox( m_toolBar );
+	m_zoomingComboBox->setGeometry( 580, 4, 80, 24 );
+	for( int i = 0; i < 6; ++i )
+	{
+		m_zoomingComboBox->addItem( QString::number( 25 *
+					static_cast<int>( powf( 2.0f, i ) ) ) +
+									"%" );
+	}
+#ifdef QT4
+	m_zoomingComboBox->setCurrentIndex( m_zoomingComboBox->findText(
+								"100%" ) );
+#else
+	m_zoomingComboBox->setCurrentText( "100%" );
+#endif
+	connect( m_zoomingComboBox, SIGNAL( activated( const QString & ) ),
+			this, SLOT( zoomingChanged( const QString & ) ) );
+
+
+
+	tb_layout->addSpacing( 5 );
+	tb_layout->addWidget( m_playButton );
+	tb_layout->addWidget( m_recordButton );
+	tb_layout->addWidget( m_stopButton );
+	tb_layout->addSpacing( 10 );
+	tb_layout->addWidget( m_drawButton );
+	tb_layout->addWidget( m_eraseButton );
+	tb_layout->addWidget( m_selectButton );
+	tb_layout->addWidget( m_moveButton );
+	tb_layout->addSpacing( 10 );
+	tb_layout->addWidget( m_cutButton );
+	tb_layout->addWidget( m_copyButton );
+	tb_layout->addWidget( m_pasteButton );
+	tb_layout->addSpacing( 10 );
+	m_timeLine->addToolButtons( m_toolBar );
+	tb_layout->addSpacing( 15 );
+	tb_layout->addWidget( zoom_lbl );
+	tb_layout->addSpacing( 5 );
+	tb_layout->addWidget( m_zoomingComboBox );
+	tb_layout->addStretch();
+
 	// setup our actual window
 	setWindowIcon( embed::getIconPixmap( "piano" ) );
 	resize( INITIAL_PIANOROLL_WIDTH, INITIAL_PIANOROLL_HEIGHT );
@@ -421,19 +438,10 @@ pianoRoll::pianoRoll( void ) :
 #ifndef QT4
 	setBackgroundMode( Qt::NoBackground );
 #endif
-	//setMouseTracking( TRUE );
+	setMouseTracking( TRUE );
 
 	hide();
 
-	// add time-line
-	m_timeLine = new timeLine( WHITE_KEY_WIDTH, 48, m_ppt,
-					songEditor::inst()->getPlayPos(
-						songEditor::PLAY_PATTERN ),
-						m_currentPosition, this );
-	connect( this, SIGNAL( positionChanged( const midiTime & ) ),
-		m_timeLine, SLOT( updatePosition( const midiTime & ) ) );
-	connect( m_timeLine, SIGNAL( positionChanged( const midiTime & ) ),
-			this, SLOT( updatePosition( const midiTime & ) ) );
 }
 
 
@@ -469,7 +477,7 @@ void pianoRoll::setCurrentPattern( pattern * _new_pattern )
 
 	noteVector & notes = m_pattern->notes();
 	int central_key = 0;
-	if( notes.isEmpty() == FALSE )
+	if( notes.empty() == FALSE )
 	{
 		// determine the central key so that we can scroll to it
 		int total_notes = 0;
@@ -499,10 +507,10 @@ void pianoRoll::setCurrentPattern( pattern * _new_pattern )
 	// remove all connections to other channel-tracks
 	disconnect( this, SLOT( recordNote( const note & ) ) );
 
-	// and now connect to noetFromMidiDeviceDone of channel so that
-	// we receive note-off-events from midi-keyboard for recording it
+	// and now connect to noteDone()-signal of channel so that
+	// we receive note-off-events from it's midi-port for recording it
 	connect( m_pattern->getChannelTrack(),
-			SIGNAL( noteFromMidiDeviceDone( const note & ) ),
+			SIGNAL( noteDone( const note & ) ),
 			this, SLOT( recordNote( const note & ) ) );
 
 	setWindowTitle( tr( "Piano-Roll - %1" ).arg( m_pattern->name() ) );
@@ -685,7 +693,14 @@ void pianoRoll::paintEvent( QPaintEvent * )
 		}
 		// draw key-line
 		p.drawLine( WHITE_KEY_WIDTH, key_line_y, width(), key_line_y );
-
+		if( key == m_keyMouseOver )
+		{
+			p.fillRect( WHITE_KEY_WIDTH, key_line_y -
+							KEY_LINE_HEIGHT,
+					width() - WHITE_KEY_WIDTH,
+					KEY_LINE_HEIGHT,
+					QColor( 64, 64, 64 ) );
+		}
 		++key;
 	}
 
@@ -757,7 +772,7 @@ void pianoRoll::paintEvent( QPaintEvent * )
 
 
 	// draw artwork-stuff
-	p.drawPixmap( 0, 0, *s_artwork1 );
+/*	p.drawPixmap( 0, 0, *s_artwork1 );
 
 	int artwork_x = s_artwork1->width();
 
@@ -765,19 +780,20 @@ void pianoRoll::paintEvent( QPaintEvent * )
 	{
 		p.drawPixmap( artwork_x, 0, *s_artwork2 );
 		artwork_x += s_artwork2->width();
-	}
+	}*/
 
 
-	// set clipping area, because we may not draw on keyboard...
+	// set clipping area, because we are not allowed to paint over
+	// keyboard...
 	p.setClipRect( WHITE_KEY_WIDTH, PR_TOP_MARGIN, width()-WHITE_KEY_WIDTH,
 				height()-PR_TOP_MARGIN-PR_BOTTOM_MARGIN );
 
 	// draw vertical raster
 	int tact_16th = m_currentPosition / 4;
 	int offset = ( m_currentPosition % 4 ) * m_ppt /
-							MAX_BEATS_PER_TACT / 4;
+						DEFAULT_STEPS_PER_TACT / 4;
 	for( int x = WHITE_KEY_WIDTH - offset; x < width();
-			x += m_ppt/MAX_BEATS_PER_TACT, ++tact_16th )
+			x += m_ppt / DEFAULT_STEPS_PER_TACT, ++tact_16th )
 	{
 		if( x >= WHITE_KEY_WIDTH )
 		{
@@ -999,6 +1015,7 @@ void pianoRoll::resizeEvent( QResizeEvent * )
 
 	songEditor::inst()->getPlayPos( songEditor::PLAY_PATTERN
 					).m_timeLine->setFixedWidth( width() );
+	m_toolBar->setFixedWidth( width() );
 }
 
 
@@ -1241,7 +1258,9 @@ void pianoRoll::mousePressEvent( QMouseEvent * _me )
 					songEditor::inst()->playing() == FALSE )
 		{
 			m_pattern->getChannelTrack()->processInEvent(
-					midiEvent( NOTE_ON, 0, key_num, vol ) );
+					midiEvent( NOTE_ON, 0, key_num,
+							vol * 127 / 100 ),
+								midiTime() );
 		}
 	}
 }
@@ -1257,12 +1276,13 @@ void pianoRoll::mouseReleaseEvent( QMouseEvent * _me )
 		{
 			m_pattern->getChannelTrack()->processInEvent(
 				midiEvent( NOTE_OFF, 0,
-						m_currentNote->key() ) );
+					m_currentNote->key(), 0 ), midiTime() );
 		}
 		else
 		{
 			m_pattern->getChannelTrack()->processInEvent(
-				midiEvent( NOTE_OFF, 0, getKey( _me->y() ) ) );
+				midiEvent( NOTE_OFF, 0, getKey( _me->y() ), 0 ),
+								midiTime() );
 		}
 	}
 
@@ -1296,6 +1316,7 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 					PR_BOTTOM_MARGIN - m_notesEditHeight );
 
 		int key_num = getKey( _me->y() );
+		m_keyMouseOver = key_num;
 		int x = _me->x();
 
 		// is the calculated key different from current key?
@@ -1303,10 +1324,17 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 		// but is still on the same key)
 		if( key_num != released_key &&
 			m_action != CHANGE_NOTE_VOLUME &&
-			edit_note == FALSE )
+			edit_note == FALSE &&
+#ifdef QT4
+			_me->buttons() &
+#else
+			_me->state() ==
+#endif
+					Qt::LeftButton )
 		{
 			m_pattern->getChannelTrack()->processInEvent(
-				midiEvent( NOTE_OFF, 0, released_key ) );
+				midiEvent( NOTE_OFF, 0, released_key, 0 ),
+								midiTime() );
 			if(
 #ifdef QT4
 				_me->buttons() &
@@ -1321,7 +1349,8 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 			{
 				m_pattern->getChannelTrack()->processInEvent(
 					midiEvent( NOTE_ON, 0, key_num,
-							DEFAULT_VOLUME ) );
+						DEFAULT_VOLUME * 127 / 100 ),
+								midiTime() );
 			}
 		}
 		if( _me->x() <= WHITE_KEY_WIDTH )
@@ -1342,12 +1371,10 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 								MIN_VOLUME,
 								MAX_VOLUME );
 				m_currentNote->setVolume( vol );
-				if( m_pattern->getChannelTrack()->keyPressed(
-						m_currentNote->key() ) == TRUE )
-				{
-	 m_pattern->getChannelTrack()->noteForKey( m_currentNote->key()
-							)->setVolume( vol );
-				}
+				m_pattern->getChannelTrack()->processInEvent(
+					midiEvent( KEY_PRESSURE, 0, key_num,
+							vol * 127 / 100 ),
+								midiTime() );
 			}
 		}
 		else if( m_currentNote != NULL &&
@@ -1752,23 +1779,6 @@ void pianoRoll::mouseMoveEvent( QMouseEvent * _me )
 
 void pianoRoll::keyPressEvent( QKeyEvent * _ke )
 {
-	if( _ke->key() == Qt::Key_Shift )
-	{
-		m_shiftPressed = TRUE;
-	}
-	else
-	{
-		m_shiftPressed = FALSE;
-	}
-	if( _ke->key() == Qt::Key_Control )
-	{
-		m_controlPressed = TRUE;
-	}
-	else
-	{
-		m_controlPressed = FALSE;
-	}
-
 	switch( _ke->key() )
 	{
 		case Qt::Key_Up:
@@ -1785,8 +1795,9 @@ void pianoRoll::keyPressEvent( QKeyEvent * _ke )
 		{
 			if( ( m_timeLine->pos() -= 16 ) < 0 )
 			{
-				m_timeLine->pos() = 0;
-			}		
+				m_timeLine->pos().setTact( 0 );
+				m_timeLine->pos().setTact64th( 0 );
+			}
 			m_timeLine->updatePosition();
 			break;
 		}
@@ -1875,7 +1886,8 @@ void pianoRoll::keyPressEvent( QKeyEvent * _ke )
 			break;
 
 		case Qt::Key_Home:
-			m_timeLine->pos() = 0;
+			m_timeLine->pos().setTact( 0 );
+			m_timeLine->pos().setTact64th( 0 );
 			m_timeLine->updatePosition();
 			break;
 
@@ -1888,32 +1900,37 @@ void pianoRoll::keyPressEvent( QKeyEvent * _ke )
 
 
 
-void pianoRoll::keyReleaseEvent( QKeyEvent * )
-{
-	m_shiftPressed = FALSE;
-	m_controlPressed = FALSE;
-}
-
-
-
-
 void pianoRoll::wheelEvent( QWheelEvent * _we )
 {
-	if( m_controlPressed )
+	_we->accept();
+	if( lmmsMainWin::isCtrlPressed() == TRUE )
 	{
 		if( _we->delta() > 0 )
 		{
 			m_ppt = tMin( m_ppt * 2, KEY_LINE_HEIGHT *
-						MAX_BEATS_PER_TACT * 8 );
+						DEFAULT_STEPS_PER_TACT * 8 );
 		}
 		else if( m_ppt >= 72 )
 		{
 			m_ppt /= 2;
 		}
+#ifdef QT4
+		// update combobox with zooming-factor
+		m_zoomingComboBox->setCurrentIndex(
+				m_zoomingComboBox->findText( QString::number(
+					static_cast<int>( m_ppt * 100 /
+						DEFAULT_PR_PPT ) ) +"%" ) );
+#else
+		// update combobox with zooming-factor
+		m_zoomingComboBox->setCurrentText( QString::number(
+					static_cast<int>( m_ppt * 100 /
+						DEFAULT_PR_PPT ) ) +"%" );
+#endif
+		// update timeline
 		m_timeLine->setPixelsPerTact( m_ppt );
 		update();
 	}
-	else if( m_shiftPressed )
+	else if( lmmsMainWin::isShiftPressed() )
 	{
 		m_leftRightScroll->setValue( m_leftRightScroll->value() -
 							_we->delta() * 2 / 15 );
@@ -1941,26 +1958,23 @@ void pianoRoll::play( void )
 		{
 			songEditor::inst()->stop();
 			songEditor::inst()->playPattern( m_pattern );
-			m_playButton->setInactiveGraphic(
-					embed::getIconPixmap( "pause" ) );
+			m_playButton->setIcon( embed::getIconPixmap(
+								"pause" ) );
 		}
 		else
 		{
 			songEditor::inst()->pause();
-			m_playButton->setInactiveGraphic(
-					embed::getIconPixmap( "play" ) );
+			m_playButton->setIcon( embed::getIconPixmap( "play" ) );
 		}
 	}
 	else if( songEditor::inst()->paused() )
 	{
 		songEditor::inst()->resumeFromPause();
-		m_playButton->setInactiveGraphic(
-					embed::getIconPixmap( "pause" ) );
+		m_playButton->setIcon( embed::getIconPixmap( "pause" ) );
 	}
 	else
 	{
-		m_playButton->setInactiveGraphic(
-					embed::getIconPixmap( "pause" ) );
+		m_playButton->setIcon( embed::getIconPixmap( "pause" ) );
 		songEditor::inst()->playPattern( m_pattern );
 	}
 }
@@ -1989,7 +2003,7 @@ void pianoRoll::record( void )
 void pianoRoll::stop( void )
 {
 	songEditor::inst()->stop();
-	m_playButton->setInactiveGraphic( embed::getIconPixmap( "play" ) );
+	m_playButton->setIcon( embed::getIconPixmap( "play" ) );
 	m_playButton->update();
 	m_recording = FALSE;
 	m_scrollBack = TRUE;
@@ -2035,56 +2049,47 @@ void pianoRoll::verScrolled( int _new_pos )
 
 
 
-void pianoRoll::drawButtonToggled( bool _on )
+void pianoRoll::drawButtonToggled( void )
 {
-	if( _on )
-	{
-		m_editMode = DRAW;
-		removeSelection();
-		update();
-	}
-}
-
-
-
-void pianoRoll::eraseButtonToggled( bool _on )
-{
-	if( _on )
-	{
-		m_editMode = ERASE;
-		removeSelection();
-		update();
-	}
+	m_editMode = DRAW;
+	removeSelection();
+	update();
 }
 
 
 
 
-void pianoRoll::selectButtonToggled( bool _on )
+void pianoRoll::eraseButtonToggled( void )
 {
-	if( _on )
-	{
-		m_editMode = SELECT;
-		removeSelection();
-		update();
-	}
+	m_editMode = ERASE;
+	removeSelection();
+	update();
 }
 
 
 
-void pianoRoll::moveButtonToggled( bool _on )
+
+void pianoRoll::selectButtonToggled( void )
 {
-	if( _on )
-	{
-		m_editMode = MOVE;
-		m_selNotesForMove.clear();
-		getSelectedNotes( m_selNotesForMove );
-		update();
-	}
+	m_editMode = SELECT;
+	removeSelection();
+	update();
 }
 
 
 
+
+void pianoRoll::moveButtonToggled( void )
+{
+	m_editMode = MOVE;
+	m_selNotesForMove.clear();
+	getSelectedNotes( m_selNotesForMove );
+	update();
+}
+
+
+
+
 void pianoRoll::selectAll( void )
 {
 	if( validPattern() == FALSE )
@@ -2202,15 +2207,15 @@ void pianoRoll::copySelectedNotes( void )
 	noteVector selected_notes;
 	getSelectedNotes( selected_notes );
 
-	if( selected_notes.isEmpty() == FALSE )
+	if( selected_notes.empty() == FALSE )
 	{
-		midiTime start_pos( selected_notes.first()->pos().getTact(),
+		midiTime start_pos( selected_notes.front()->pos().getTact(),
 									0 );
 		for( noteVector::iterator it = selected_notes.begin();
 			it != selected_notes.end(); ++it )
 		{
 			m_notesToCopy.push_back( new note( **it ) );
-			m_notesToCopy.last()->setPos( m_notesToCopy.last()->pos(
+			m_notesToCopy.back()->setPos( m_notesToCopy.back()->pos(
 								start_pos ) );
 		}
 	}
@@ -2237,22 +2242,22 @@ void pianoRoll::cutSelectedNotes( void )
 	noteVector selected_notes;
 	getSelectedNotes( selected_notes );
 
-	if( selected_notes.isEmpty() == FALSE )
+	if( selected_notes.empty() == FALSE )
 	{
 		songEditor::inst()->setModified();
 
-		midiTime start_pos( selected_notes.first()->pos().getTact(),
+		midiTime start_pos( selected_notes.front()->pos().getTact(),
 									0 );
 
-		while( selected_notes.isEmpty() == FALSE )
+		while( selected_notes.empty() == FALSE )
 		{
-			note * new_note = new note( *selected_notes.first() );
+			note * new_note = new note( *selected_notes.front() );
 			new_note->setPos( new_note->pos( start_pos ) );
 			m_notesToCopy.push_back( new_note );
 
 			// note (the memory of it) is also deleted by
 			// pattern::removeNote(...) so we don't have to do that
-			m_pattern->removeNote( selected_notes.first() );
+			m_pattern->removeNote( selected_notes.front() );
 			selected_notes.erase( selected_notes.begin() );
 		}
 	}
@@ -2271,7 +2276,7 @@ void pianoRoll::pasteNotes( void )
 		return;
 	}
 
-	if( m_notesToCopy.isEmpty() == FALSE )
+	if( m_notesToCopy.empty() == FALSE )
 	{
 		for( noteVector::iterator it = m_notesToCopy.begin();
 					it != m_notesToCopy.end(); ++it )
@@ -2302,11 +2307,11 @@ void pianoRoll::deleteSelectedNotes( void )
 	noteVector selected_notes;
 	getSelectedNotes( selected_notes );
 
-	const bool update_after_delete = !selected_notes.isEmpty();
+	const bool update_after_delete = !selected_notes.empty();
 
-	while( selected_notes.isEmpty() == FALSE )
+	while( selected_notes.empty() == FALSE )
 	{
-		m_pattern->removeNote( selected_notes.first() );
+		m_pattern->removeNote( selected_notes.front() );
 		selected_notes.erase( selected_notes.begin() );
 	}
 
@@ -2343,6 +2348,20 @@ void pianoRoll::updatePosition( const midiTime & _t )
 
 
 
+
+void pianoRoll::zoomingChanged( const QString & _zfac )
+{
+	m_ppt = _zfac.left( _zfac.length() - 1 ).toInt() * DEFAULT_PR_PPT / 100;
+#ifdef LMMS_DEBUG
+	assert( m_ppt > 0 );
+#endif
+	m_timeLine->setPixelsPerTact( m_ppt );
+	update();
+
+}
+
+
+
 #undef setChecked
 
 
