@@ -1,8 +1,9 @@
 /*
  * song_editor.cpp - basic window for editing song
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -26,6 +27,8 @@
 #include <config.h>
 #endif
 
+#include <math.h>
+
 
 #include "qt3support.h"
 
@@ -38,13 +41,12 @@
 #include <QFileInfo>
 #include <QFileDialog>
 #include <QSlider>
-#include <QStatusBar>
 #include <QKeyEvent>
 #include <QLabel>
-#include <QToolButton>
 #include <QStatusBar>
 #include <QAction>
-#include <QToolBar>
+#include <QComboBox>
+#include <QLayout>
 
 #else
 
@@ -56,8 +58,9 @@
 #include <qdom.h>
 #include <qslider.h>
 #include <qlabel.h>
-#include <qtoolbutton.h>
 #include <qstatusbar.h>
+#include <qcombobox.h>
+#include <qlayout.h>
 
 #endif
 
@@ -71,7 +74,7 @@
 #include "bb_track.h"
 #include "channel_track.h"
 #include "mmp.h"
-#include "midi_device.h"
+#include "midi_client.h"
 #include "timeline.h"
 #include "pattern.h"
 #include "piano_roll.h"
@@ -82,15 +85,15 @@
 #include "midi_file.h"
 #include "lcd_spinbox.h"
 #include "tooltip.h"
+#include "tool_button.h"
+#include "cpuload_widget.h"
+#include "text_float.h"
 
 #include "debug.h"
 
 
-#include "triple_oscillator.h"
-
 
 extern QString file_to_load;
-extern QString file_to_render;
 
 
 const int SCROLLBAR_SIZE = 16;
@@ -106,24 +109,28 @@ songEditor::songEditor() :
 	m_exporting( FALSE ),
 	m_playing( FALSE ),
 	m_paused( FALSE ),
+	m_loadingProject( FALSE ),
 	m_playMode( PLAY_SONG ),
 	m_trackToPlay( NULL ),
 	m_patternToPlay( NULL ),
 	m_loopPattern( FALSE ),
-	m_scrollBack( FALSE ),
-	m_epd( NULL ),
-	m_shiftPressed( FALSE ),
-	m_controlPressed( FALSE )
+	m_scrollBack( FALSE )
 {
 	// hack, because code called out of this function uses
-	// songEditor::inst(), which assigns s_instanceOfMe after return
-	// of this function...
+	// songEditor::inst(), which assigns s_instanceOfMe this function
+	// returns...
 	s_instanceOfMe = this;
 
 	setWindowTitle( tr( "Song-Editor" ) );
 	setWindowIcon( embed::getIconPixmap( "songeditor" ) );
-	setGeometry( 10, 10, 640, 300 );
-	show();
+	if( lmmsMainWin::inst()->workspace() != NULL )
+	{
+		setGeometry( 10, 10, 680, 300 );
+	}
+	else
+	{
+		setGeometry( 210, 10, 580, 300 );
+	}
 
 #ifdef QT4
 	setFocusPolicy( Qt::StrongFocus );
@@ -132,13 +139,13 @@ songEditor::songEditor() :
 #endif
 	setFocus();
 
-
 	QWidget * cw = new QWidget( this );
 	setCentralWidget( cw );
 
+
 	// create time-line
 	timeLine * tl = new timeLine( TRACK_OP_WIDTH +
-					DEFAULT_SETTINGS_WIDGET_WIDTH, 0,
+					DEFAULT_SETTINGS_WIDGET_WIDTH, 32,
 					pixelsPerTact(), m_playPos[PLAY_SONG],
 					m_currentPosition, cw );
 	connect( this, SIGNAL( positionChanged( const midiTime & ) ),
@@ -147,76 +154,13 @@ songEditor::songEditor() :
 	connect( tl, SIGNAL( positionChanged( const midiTime & ) ),
 			this, SLOT( updatePosition( const midiTime & ) ) );
 
-#ifdef QT4
-	containerWidget()->setParent( cw );
-#else
-	containerWidget()->reparent( cw, 0, QPoint( 0, 0 ) );
-#endif
-	containerWidget()->move( 0, tl->height() );
 
+	// add some essential widgets to global tool-bar 
+	QWidget * tb = lmmsMainWin::inst()->toolBar();
 
-	QToolBar * song_control = new QToolBar( tr( "Song control" ), this );
-#ifdef QT4
-	addToolBar( Qt::TopToolBarArea, song_control );
-#else
-	addDockWindow( song_control, tr( "Song control" ), Qt::DockTop,
-			FALSE );
-#endif
-/*	song_control->setPaletteBackgroundPixmap( embed::getIconPixmap(
-							"toolbar_bg" ) );
-	song_control->setErasePixmap( embed::getIconPixmap( "toolbar_bg" ) );*/
+	lmmsMainWin::inst()->addSpacingToToolBar( 10 );
 
-#ifdef QT4
-	QAction * a;
-
-	a = song_control->addAction( embed::getIconPixmap( "play" ),
-						tr( "Play song (Space)" ),
-						this, SLOT( play() ) );
-	a->setToolTip( tr( "Play/pause song (Space)" ) );
-	a->setWhatsThis( tr( "Click here, if you want to play your whole song. "
-				"Playing will be started at the song-position-"
-				"marker (green). You can also move it while "
-				"playing." ) );
-#else
-	m_playButton = new QToolButton( embed::getIconPixmap( "play" ),
-					tr( "Play song (Space)" ),
-					QString::null, this, SLOT( play() ),
-					song_control );
-#endif
-#ifdef QT4
-	a = song_control->addAction( embed::getIconPixmap( "stop" ),
-						tr( "Stop song (Space)" ),
-						this, SLOT( stop() ) );
-	a->setToolTip( tr( "Stop song (Space)" ) );
-	a->setWhatsThis( tr( "Click here, if you want to stop playing of your "
-				"song. The song-position-marker will be set to "
-				"the start of your song." ) );
-#else
-	m_stopButton = new QToolButton( embed::getIconPixmap( "stop" ),
-					tr( "Stop song (Space)" ),
-					QString::null, this, SLOT( stop() ),
-					song_control );
-#endif
-
-
-	song_control->addSeparator();
-
-	// spacer-item
-	( new QWidget( song_control ) )->setFixedSize( 10, 1 );
-
-
-	QLabel * bpm_label = new QLabel( song_control );
-	bpm_label->setPixmap( embed::getIconPixmap( "clock" ) );
-
-	// spacer-item
-	( new QWidget( song_control ) )->setFixedSize( 8, 1 );
-
-
-	m_bpmSpinBox = new lcdSpinBox( MIN_BPM, MAX_BPM, 3, song_control );
-#ifdef QT4
-	song_control->addWidget( m_bpmSpinBox );
-	song_control->addWidget( bpm_label );
-#endif
+	m_bpmSpinBox = new lcdSpinBox( MIN_BPM, MAX_BPM, 3, tb );
 	m_bpmSpinBox->setLabel( tr( "TEMPO/BPM" ) );
 	connect( m_bpmSpinBox, SIGNAL( valueChanged( int ) ), this,
 							SLOT( setBPM( int ) ) );
@@ -234,32 +178,39 @@ songEditor::songEditor() :
 			"should be played within a minute (or how many tacts "
 			"should be played within four minutes)." ) );
 
-	// spacer-item
-	( new QWidget( song_control ) )->setFixedSize( 10, 1 );
+	int col = lmmsMainWin::inst()->addWidgetToToolBar( m_bpmSpinBox, 0 );
 
 
-	song_control->addSeparator();
+	toolButton * hq_btn = new toolButton( embed::getIconPixmap( "hq_mode" ),
+						tr( "High quality mode" ),
+						NULL, NULL, tb );
+	hq_btn->setCheckable( TRUE );
+	connect( hq_btn, SIGNAL( toggled( bool ) ), mixer::inst(),
+					SLOT( setHighQuality( bool ) ) );
+	hq_btn->setFixedWidth( 42 );
+	lmmsMainWin::inst()->addWidgetToToolBar( hq_btn, 1, col );
 
 
-	QLabel * master_vol_lbl = new QLabel( song_control );
+	lmmsMainWin::inst()->addSpacingToToolBar( 10 );
+
+
+
+	QLabel * master_vol_lbl = new QLabel( tb );
 	master_vol_lbl->setPixmap( embed::getIconPixmap( "master_volume" ) );
 
 #ifdef QT4
-	m_masterVolumeSlider = new QSlider( Qt::Vertical, song_control );
+	m_masterVolumeSlider = new QSlider( Qt::Vertical, tb );
 	m_masterVolumeSlider->setRange( 0, 200 );
-	m_masterVolumeSlider->setPageStep( 10 );
+	m_masterVolumeSlider->setPageStep( 1 );
 	m_masterVolumeSlider->setValue( 100 );
 	m_masterVolumeSlider->setTickPosition( QSlider::TicksLeft );
-	song_control->addWidget( master_vol_lbl );
-	song_control->addWidget( m_masterVolumeSlider );
 #else
-	m_masterVolumeSlider = new QSlider( 0, 200, 10, 100, Qt::Vertical,
-								song_control );
+	m_masterVolumeSlider = new QSlider( 0, 200, 1, 100, Qt::Vertical, tb );
 	m_masterVolumeSlider->setTickPosition( QSlider::Left );
 #endif
-	m_masterVolumeSlider->setFixedSize( 26, 48 );
+	m_masterVolumeSlider->setFixedSize( 26, 60 );
 	m_masterVolumeSlider->setTickInterval( 50 );
-	toolTip::add( m_masterVolumeSlider, tr( "master output volume" ) );
+	toolTip::add( m_masterVolumeSlider, tr( "master volume" ) );
 
 	connect( m_masterVolumeSlider, SIGNAL( valueChanged( int ) ), this,
 			SLOT( masterVolumeChanged( int ) ) );
@@ -270,27 +221,30 @@ songEditor::songEditor() :
 	connect( m_masterVolumeSlider, SIGNAL( sliderReleased() ), this,
 			SLOT( masterVolumeReleased() ) );
 
+	m_mvsStatus = new textFloat( m_masterVolumeSlider );
+	m_mvsStatus->setTitle( tr( "Master volume" ) );
+	m_mvsStatus->setPixmap( embed::getIconPixmap( "master_volume" ) );
 
-	// spacer-item
-	( new QWidget( song_control ) )->setFixedSize( 10, 1 );
+	lmmsMainWin::inst()->addWidgetToToolBar( master_vol_lbl );
+	lmmsMainWin::inst()->addWidgetToToolBar( m_masterVolumeSlider );
 
-	QLabel * master_pitch_lbl = new QLabel( song_control );
+
+	lmmsMainWin::inst()->addSpacingToToolBar( 10 );
+
+	QLabel * master_pitch_lbl = new QLabel( tb );
 	master_pitch_lbl->setPixmap( embed::getIconPixmap( "master_pitch" ) );
 
 #ifdef QT4
-	m_masterPitchSlider = new QSlider( Qt::Vertical, song_control );
+	m_masterPitchSlider = new QSlider( Qt::Vertical, tb );
 	m_masterPitchSlider->setRange( -12, 12 );
 	m_masterPitchSlider->setPageStep( 1 );
 	m_masterPitchSlider->setValue( 0 );
 	m_masterPitchSlider->setTickPosition( QSlider::TicksLeft );
-	song_control->addWidget( master_pitch_lbl );
-	song_control->addWidget( m_masterPitchSlider );
 #else
-	m_masterPitchSlider = new QSlider( -12, 12, 1, 0, Qt::Vertical,
-						song_control );
+	m_masterPitchSlider = new QSlider( -12, 12, 1, 0, Qt::Vertical, tb );
 	m_masterPitchSlider->setTickPosition( QSlider::Left );
 #endif
-	m_masterPitchSlider->setFixedSize( 26, 48 );
+	m_masterPitchSlider->setFixedSize( 26, 60 );
 	m_masterPitchSlider->setTickInterval( 12 );
 	toolTip::add( m_masterPitchSlider, tr( "master pitch" ) );
 	connect( m_masterPitchSlider, SIGNAL( valueChanged( int ) ), this,
@@ -302,29 +256,152 @@ songEditor::songEditor() :
 	connect( m_masterPitchSlider, SIGNAL( sliderReleased() ), this,
 			SLOT( masterPitchReleased() ) );
 
-	// spacer-item
-	( new QWidget( song_control ) )->setFixedSize( 5, 1 );
+	m_mpsStatus = new textFloat( m_masterPitchSlider );
+	m_mpsStatus->setTitle( tr( "Master pitch" ) );
+	m_mpsStatus->setPixmap( embed::getIconPixmap( "master_pitch" ) );
 
-	song_control->addSeparator();
+	lmmsMainWin::inst()->addWidgetToToolBar( master_pitch_lbl );
+	lmmsMainWin::inst()->addWidgetToToolBar( m_masterPitchSlider );
 
-	// spacer-item
-	( new QWidget( song_control ) )->setFixedSize( 5, 1 );
+	lmmsMainWin::inst()->addSpacingToToolBar( 10 );
 
-	m_masterOutputGraph = new visualizationWidget( embed::getIconPixmap(
-					"output_graph" ), song_control ); 
+	// create widget for visualization- and cpu-load-widget
+	QWidget * vc_w = new QWidget( tb );
+	QVBoxLayout * vcw_layout = new QVBoxLayout( vc_w );
+	vcw_layout->setMargin( 0 );
+	vcw_layout->setSpacing( 0 );
+
+	vcw_layout->addStretch();
+	vcw_layout->addWidget( new visualizationWidget(
+			embed::getIconPixmap( "output_graph" ), vc_w ) ); 
+
+	vcw_layout->addWidget( new cpuloadWidget( vc_w ) );
+	vcw_layout->addStretch();
+
+	lmmsMainWin::inst()->addWidgetToToolBar( vc_w );
+
+
+	// create own toolbar
+	m_toolBar = new QWidget( cw );
+	m_toolBar->setFixedHeight( 32 );
+	m_toolBar->move( 0, 0 );
 #ifdef QT4
-	song_control->addWidget( m_masterOutputGraph );
+	QPalette pal;
+	pal.setBrush( m_toolBar->backgroundRole(), QBrush(
+				embed::getIconPixmap( "toolbar_bg" ) ) );
+	m_toolBar->setPalette( pal );
+#else
+	m_toolBar->setPaletteBackgroundPixmap( embed::getIconPixmap(
+							"toolbar_bg" ) );
 #endif
-	// live high-quality mode switching is somewhat experimental so we don't
-	// offer it...
-/*	QToolButton * hq = new QToolButton(
-					embed::getIconPixmap( "presetfile" ),
-					tr( "High quality mode" ),
-					QString::null, NULL, NULL,
-					song_control );
-	hq->setToggleButton( TRUE );
-	connect( hq, SIGNAL( toggled( bool ) ), mixer::inst(),
-					SLOT( setHighQuality( bool ) ) );*/
+
+	QHBoxLayout * tb_layout = new QHBoxLayout( m_toolBar );
+	tb_layout->setMargin( 0 );
+	tb_layout->setSpacing( 0 );
+
+#ifdef QT4
+	containerWidget()->setParent( cw );
+#else
+	containerWidget()->reparent( cw, 0, QPoint( 0, 0 ) );
+#endif
+	containerWidget()->move( 0, m_toolBar->height() + tl->height() );
+
+
+
+	// fill own tool-bar
+	m_playButton = new toolButton( embed::getIconPixmap( "play" ),
+					tr( "Play song (Space)" ),
+					this, SLOT( play() ), m_toolBar );
+
+	m_stopButton = new toolButton( embed::getIconPixmap( "stop" ),
+					tr( "Stop song (Space)" ),
+					this, SLOT( stop() ), m_toolBar );
+
+	m_addBBTrackButton = new toolButton( embed::getIconPixmap(
+						"add_bb_track" ),
+						tr( "Add beat/bassline" ),
+						this, SLOT( addBBTrack() ),
+						m_toolBar );
+
+	m_addSampleTrackButton = new toolButton( embed::getIconPixmap(
+					"add_sample_track" ),
+					tr( "Add sample-track" ),
+					this, SLOT( addSampleTrack() ),
+					m_toolBar );
+
+	m_insertBarButton = new toolButton( embed::getIconPixmap(
+						"insert_bar" ),
+						tr( "Insert bar "
+							"(Shift+Insert)" ),
+						this, SLOT( insertBar() ),
+						m_toolBar );
+
+	m_removeBarButton = new toolButton( embed::getIconPixmap(
+						"remove_bar" ),
+					tr( "Remove bar (Shift+Delete)" ),
+						this, SLOT( removeBar() ),
+						m_toolBar );
+#ifdef QT4
+#else
+	QWhatsThis::add( m_playButton, tr( "Click here, if you want to play "
+						"your whole song. Playing will "
+						"be started at the "
+						"song-position-marker (green). "
+						"You can also move it while "
+						"playing." ) );
+	QWhatsThis::add( m_stopButton, tr ( "Click here, if you want to stop "
+						"playing of your song. The "
+						"song-position-marker will be "
+						"set to the start of your song."
+			) );
+	QWhatsThis::add( m_insertBarButton, tr( "If you click here, a "
+							"bar will "
+							"be inserted at the "
+							"current bar." ) );
+	QWhatsThis::add( m_removeBarButton, tr( "If you click here, the "
+							"current bar will be "
+							"removed." ) );
+#endif
+
+
+	QLabel * zoom_lbl = new QLabel( m_toolBar );
+	zoom_lbl->setPixmap( embed::getIconPixmap( "zoom" ) );
+
+	// setup zooming-stuff
+	m_zoomingComboBox = new QComboBox( m_toolBar );
+	m_zoomingComboBox->setGeometry( 580, 4, 80, 24 );
+	for( int i = 0; i < 7; ++i )
+	{
+		m_zoomingComboBox->addItem( QString::number( 25 *
+					static_cast<int>( powf( 2.0f, i ) ) ) +
+									"%" );
+	}
+#ifdef QT4
+	m_zoomingComboBox->setCurrentIndex( m_zoomingComboBox->findText(
+								"100%" ) );
+#else
+	m_zoomingComboBox->setCurrentText( "100%" );
+#endif
+	connect( m_zoomingComboBox, SIGNAL( activated( const QString & ) ),
+			this, SLOT( zoomingChanged( const QString & ) ) );
+
+
+	tb_layout->addSpacing( 5 );
+	tb_layout->addWidget( m_playButton );
+	tb_layout->addWidget( m_stopButton );
+	tb_layout->addSpacing( 10 );
+	tb_layout->addWidget( m_addBBTrackButton );
+	tb_layout->addWidget( m_addSampleTrackButton );
+	tb_layout->addSpacing( 10 );
+	tb_layout->addWidget( m_insertBarButton );
+	tb_layout->addWidget( m_removeBarButton );
+	tb_layout->addSpacing( 10 );
+	tl->addToolButtons( m_toolBar );
+	tb_layout->addSpacing( 15 );
+	tb_layout->addWidget( zoom_lbl );
+	tb_layout->addSpacing( 5 );
+	tb_layout->addWidget( m_zoomingComboBox );
+	tb_layout->addStretch();
 
 
 	m_leftRightScroll = new QScrollBar( Qt::Horizontal, cw );
@@ -340,111 +417,19 @@ songEditor::songEditor() :
 			SLOT( scrolled( int ) ) );
 
 
-	QToolBar * edit_tb = new QToolBar( tr( "Edit" ), this );
-#ifdef QT4
-	addToolBar( Qt::TopToolBarArea, edit_tb );
-#else
-	addDockWindow( edit_tb, tr( "Edit" ), Qt::DockTop, FALSE );
-#endif
-/*	edit_tb->setPaletteBackgroundPixmap( embed::getIconPixmap(
-							"toolbar_bg" ) );
-	edit_tb->setErasePixmap( embed::getIconPixmap( "toolbar_bg" ) );*/
-#ifdef QT4
-	a = edit_tb->addAction( embed::getIconPixmap( "add_channel_track" ), "",
-					this, SLOT( addChannelTrack() ) );
-	a->setToolTip( tr( "Add channel-track" ) );
-#else
-	m_addChannelTrackButton = new QToolButton( embed::getIconPixmap(
-					"add_channel_track" ), "", "",
-					this, SLOT( addChannelTrack() ),
-					edit_tb );
-#endif
-#ifdef QT4
-	a = edit_tb->addAction( embed::getIconPixmap( "add_bb_track" ), "",
-						this, SLOT( addBBTrack() ) );
-	a->setToolTip( tr( "Add beat/bassline" ) );
-#else
-	m_addBBTrackButton = new QToolButton( embed::getIconPixmap(
-						"add_bb_track" ), "", "",
-						this, SLOT( addBBTrack() ),
-						edit_tb );
-#endif
-#ifdef QT4
-	a = edit_tb->addAction( embed::getIconPixmap( "add_sample_track" ), "",
-				this, SLOT( addSampleTrack() ) );
-	a->setToolTip( tr( "Add sample-track" ) );
-#else
-	m_addSampleTrackButton = new QToolButton( embed::getIconPixmap(
-					"add_sample_track" ), "", "",
-					this, SLOT( addSampleTrack() ),
-					edit_tb );
 
-#endif
-
-	edit_tb->addSeparator();
-
-#ifdef QT4
-	a = edit_tb->addAction( embed::getIconPixmap( "se_insert_tact" ), "",
-						this, SLOT( insertTact() ) );
-	a->setToolTip( tr( "Insert bar at current tact (Shift+Insert)" ) );
-	a->setWhatsThis( tr( "If you click here, a tact will be inserted at "
-				"the current tact." ) );
-#else
-	m_insertTactButton = new QToolButton( embed::getIconPixmap(
-						"se_insert_tact" ), "", "",
-						this, SLOT( insertTact() ),
-						edit_tb );
-#endif
-#ifdef QT4
-	a = edit_tb->addAction( embed::getIconPixmap( "se_remove_tact" ), "",
-						this, SLOT( removeTact() ) );
-	a->setToolTip( tr( "Remove bar at current tact (Shift+Delete)" ) );
-	a->setWhatsThis( tr( "If you click here, the tact at the current tact "
-							"will be removed." ) );
-#else
-	m_removeTactButton = new QToolButton( embed::getIconPixmap(
-						"se_remove_tact" ), "", "",
-						this, SLOT( removeTact() ),
-						edit_tb );
-#endif
-
-	// add tooltips and whats-this-texts to all buttons
-
-	toolTip::add( m_playButton, tr( "Play/pause song (Space)" ) );
-	toolTip::add( m_stopButton, tr( "Stop playing song (Space)" ) );
-	toolTip::add( m_addChannelTrackButton, tr( "Add channel-track" ) );
-	toolTip::add( m_addBBTrackButton, tr( "Add beat/bassline" ) );
-	toolTip::add( m_addSampleTrackButton, tr( "Add sample-track" ) );
-	toolTip::add( m_insertTactButton, tr( "Insert tact at current tact "
-						"(Shift+Insert)" ) );
-	toolTip::add( m_removeTactButton, tr( "Remove tact at current tact "
-						"(Shift+Delete)" ) );
-#ifdef QT4
-#else
-	QWhatsThis::add( m_playButton, tr( "Click here, if you want to play "
-						"your whole song. Playing will "
-						"be started at the "
-						"song-position-marker (green). "
-						"You can also move it while "
-						"playing." ) );
-	QWhatsThis::add( m_stopButton, tr ( "Click here, if you want to stop "
-						"playing of your song. The "
-						"song-position-marker will be "
-						"set to the start of your song."
-			) );
-	QWhatsThis::add( m_insertTactButton, tr( "If you click here, a "
-							"tact will "
-							"be inserted at the "
-							"current tact." ) );
-	QWhatsThis::add( m_removeTactButton, tr( "If you click here, the "
-							"tact at the "
-							"current tact will be "
-							"removed." ) );
-#endif
+	show();
 
 	m_projectNotes = new projectNotes();
 	m_projectNotes->resize( 300, 200 );
-	m_projectNotes->move( 640, 10 );
+	if( lmmsMainWin::inst()->workspace() != NULL )
+	{
+		m_projectNotes->move( 700, 10 );
+	}
+	else
+	{
+		m_projectNotes->move( 800, 10 );
+	}
 	m_projectNotes->show();
 
 
@@ -490,15 +475,19 @@ void songEditor::paintEvent( QPaintEvent * _pe )
 // responsible for moving scrollbars after resizing
 void songEditor::resizeEvent( QResizeEvent * _re )
 {
-	m_leftRightScroll->setGeometry( 0,
+	if( centralWidget() != NULL )
+	{
+		m_leftRightScroll->setGeometry( 0,
 					centralWidget()->height() - 2 -
 								SCROLLBAR_SIZE,
 					centralWidget()->width() -
 								SCROLLBAR_SIZE,
 					SCROLLBAR_SIZE );
 
-	m_playPos[PLAY_SONG].m_timeLine->setFixedWidth(
+		m_playPos[PLAY_SONG].m_timeLine->setFixedWidth(
 						centralWidget()->width() );
+		m_toolBar->setFixedWidth( centralWidget()->width() );
+	}
 	trackContainer::resizeEvent( _re );
 }
 
@@ -507,32 +496,17 @@ void songEditor::resizeEvent( QResizeEvent * _re )
 
 void songEditor::keyPressEvent( QKeyEvent * _ke )
 {
-	if( _ke->key() == Qt::Key_Shift )
+	if( /*_ke->modifiers() & Qt::ShiftModifier*/
+		lmmsMainWin::isShiftPressed() == TRUE &&
+						_ke->key() == Qt::Key_Insert )
 	{
-		m_shiftPressed = TRUE;
+		insertBar();
 	}
-	else
+	else if(/* _ke->modifiers() & Qt::ShiftModifier &&*/
+			lmmsMainWin::isShiftPressed() == TRUE &&
+						_ke->key() == Qt::Key_Delete )
 	{
-		m_shiftPressed = FALSE;
-	}
-	if( _ke->key() == Qt::Key_Control )
-	{
-		m_controlPressed = TRUE;
-	}
-	else
-	{
-		m_controlPressed = FALSE;
-	}
-
-	if( _ke->modifiers() & Qt::ShiftModifier &&
-		_ke->key() == Qt::Key_Insert )
-	{
-		insertTact();
-	}
-	else if( _ke->modifiers() & Qt::ShiftModifier &&
-		_ke->key() == Qt::Key_Delete )
-	{
-		removeTact();
+		removeBar();
 	}
 	else if( _ke->key() == Qt::Key_Left )
 	{
@@ -571,7 +545,7 @@ void songEditor::keyPressEvent( QKeyEvent * _ke )
 	}
 	else
 	{
-		_ke->ignore();
+		QWidget::keyPressEvent( _ke );
 	}
 }
 
@@ -590,7 +564,7 @@ void songEditor::scrolled( int _new_pos )
 
 void songEditor::wheelEvent( QWheelEvent * _we )
 {
-	if( m_controlPressed )
+	if( lmmsMainWin::isCtrlPressed() == TRUE )
 	{
 		if( _we->delta() > 0 )
 		{
@@ -601,15 +575,35 @@ void songEditor::wheelEvent( QWheelEvent * _we )
 		{
 			setPixelsPerTact( (int) pixelsPerTact() / 2 );
 		}
+#ifdef QT4
+		// update combobox with zooming-factor
+		m_zoomingComboBox->setCurrentIndex(
+				m_zoomingComboBox->findText( QString::number(
+					static_cast<int>( pixelsPerTact() *
+				100 / DEFAULT_PIXELS_PER_TACT ) ) + "%" ) );
+#else
+		// update combobox with zooming-factor
+		m_zoomingComboBox->setCurrentText( QString::number(
+					static_cast<int>( pixelsPerTact() *
+				100 / DEFAULT_PIXELS_PER_TACT ) ) + "%" );
+#endif
+		// update timeline
 		m_playPos[PLAY_SONG].m_timeLine->setPixelsPerTact(
 							pixelsPerTact() );
+		// and make sure, all TCO's are resized and relocated
 		realignTracks( TRUE );
 	} 
-	else if( m_shiftPressed )
+	else if( lmmsMainWin::isShiftPressed() == TRUE )
 	{
 		m_leftRightScroll->setValue( m_leftRightScroll->value() -
 							_we->delta() / 30 );
 	}
+	else
+	{
+		_we->ignore();
+		return;
+	}
+	_we->accept();
 }
 
 
@@ -617,7 +611,17 @@ void songEditor::wheelEvent( QWheelEvent * _we )
 
 void songEditor::masterVolumeChanged( int _new_val )
 {
-	mixer::inst()->setMasterOutput( 2.0f - _new_val / 100.0f );
+	masterVolumeMoved( _new_val );
+	if( m_mvsStatus->isVisible() == FALSE && m_loadingProject == FALSE )
+	{
+		m_mvsStatus->reparent( m_masterVolumeSlider );
+		m_mvsStatus->move( m_masterVolumeSlider->mapTo(
+					m_masterVolumeSlider->topLevelWidget(),
+							QPoint( 0, 0 ) ) +
+			QPoint( m_masterVolumeSlider->width() + 2, -2 ) );
+		m_mvsStatus->setVisibilityTimeOut( 1000 );
+	}
+	mixer::inst()->setMasterGain( 2.0f - _new_val / 100.0f );
 	setModified();
 }
 
@@ -626,6 +630,12 @@ void songEditor::masterVolumeChanged( int _new_val )
 
 void songEditor::masterVolumePressed( void )
 {
+	m_mvsStatus->reparent( m_masterVolumeSlider );
+	m_mvsStatus->move( m_masterVolumeSlider->mapTo(
+					m_masterVolumeSlider->topLevelWidget(),
+							QPoint( 0, 0 ) ) +
+			QPoint( m_masterVolumeSlider->width() + 2, -2 ) );
+	m_mvsStatus->show();
 	masterVolumeMoved( m_masterVolumeSlider->value() );
 }
 
@@ -634,9 +644,7 @@ void songEditor::masterVolumePressed( void )
 
 void songEditor::masterVolumeMoved( int _new_val )
 {
-	lmmsMainWin::inst()->statusBar()->showMessage( tr(
-						"Master output volume:" ) +
-				" " + QString::number( 200 - _new_val ) + "%" );
+	m_mvsStatus->setText( tr( "Value: %1%" ).arg( 200 - _new_val ) );
 }
 
 
@@ -644,7 +652,7 @@ void songEditor::masterVolumeMoved( int _new_val )
 
 void songEditor::masterVolumeReleased( void )
 {
-	lmmsMainWin::inst()->statusBar()->clearMessage();
+	m_mvsStatus->hide();
 }
 
 
@@ -652,6 +660,16 @@ void songEditor::masterVolumeReleased( void )
 
 void songEditor::masterPitchChanged( int _new_val )
 {
+	masterPitchMoved( _new_val );
+	if( m_mpsStatus->isVisible() == FALSE && m_loadingProject == FALSE )
+	{
+		m_mpsStatus->reparent( m_masterPitchSlider );
+		m_mpsStatus->move( m_masterPitchSlider->mapTo(
+					m_masterPitchSlider->topLevelWidget(),
+							QPoint( 0, 0 ) ) +
+			QPoint( m_masterPitchSlider->width() + 2, -2 ) );
+		m_mpsStatus->setVisibilityTimeOut( 1000 );
+	}
 	setModified();
 }
 
@@ -660,6 +678,12 @@ void songEditor::masterPitchChanged( int _new_val )
 
 void songEditor::masterPitchPressed( void )
 {
+	m_mpsStatus->reparent( m_masterPitchSlider );
+	m_mpsStatus->move( m_masterPitchSlider->mapTo(
+					m_masterPitchSlider->topLevelWidget(),
+							QPoint( 0, 0 ) ) +
+			QPoint( m_masterPitchSlider->width() + 2, -2 ) );
+	m_mpsStatus->show();
 	masterPitchMoved( m_masterPitchSlider->value() );
 }
 
@@ -668,9 +692,7 @@ void songEditor::masterPitchPressed( void )
 
 void songEditor::masterPitchMoved( int _new_val )
 {
-	lmmsMainWin::inst()->statusBar()->showMessage( tr(
-						"Master output pitch:" ) +
-		" " + QString::number( -_new_val ) + " " + tr( "semitones" ) );
+	m_mpsStatus->setText( tr( "Value: %1 semitones").arg( -_new_val ) );
 
 }
 
@@ -679,15 +701,7 @@ void songEditor::masterPitchMoved( int _new_val )
 
 void songEditor::masterPitchReleased( void )
 {
-	lmmsMainWin::inst()->statusBar()->clearMessage();
-}
-
-
-
-
-void songEditor::toggleHQMode( void )
-{
-	//mixer::inst()->setHighQuality (hq_btn->isChecked());
+	m_mpsStatus->hide();
 }
 
 
@@ -717,10 +731,22 @@ void songEditor::updatePosition( const midiTime & _t )
 
 
 
+void songEditor::zoomingChanged( const QString & _zfac )
+{
+	setPixelsPerTact( _zfac.left( _zfac.length() - 1 ).toInt() *
+					DEFAULT_PIXELS_PER_TACT / 100 );
+	m_playPos[PLAY_SONG].m_timeLine->setPixelsPerTact( pixelsPerTact() );
+	realignTracks( TRUE );
+}
+
+
+
+
 void songEditor::setBPM( int _new_bpm )
 {
 	m_bpmSpinBox->setValue( tLimit( _new_bpm, MIN_BPM, MAX_BPM ) );
 	setModified();
+	emit bpmChanged( _new_bpm );
 }
 
 
@@ -736,13 +762,14 @@ int songEditor::masterPitch( void ) const
 
 void songEditor::doActions( void )
 {
-	while( !m_actions.isEmpty() )
+	while( !m_actions.empty() )
 	{
-		timeLine * tl = m_playPos[m_playMode].m_timeLine;
 		switch( m_actions.front() )
 		{
 			case ACT_STOP_PLAY:
 			{
+				timeLine * tl =
+					m_playPos[m_playMode].m_timeLine;
 				m_playing = FALSE;
 				if( tl != NULL )
 				{
@@ -818,18 +845,23 @@ void songEditor::doActions( void )
 		}
 
 		// a second switch for saving pos when starting to play
-		// anything
+		// anything (need pos for restoring it later in certain
+		// timeline-modes)
 		switch( m_actions.front() )
 		{
 			case ACT_PLAY_SONG:
 			case ACT_PLAY_TRACK:
 			case ACT_PLAY_BB:
 			case ACT_PLAY_PATTERN:
+			{
+				timeLine * tl =
+					m_playPos[m_playMode].m_timeLine;
 				if( tl != NULL )
 				{
 					tl->savePos( m_playPos[m_playMode] );
 				}
 				break;
+			}
 
 			// keep GCC happy...
 			default:
@@ -891,14 +923,16 @@ void songEditor::processNextBuffer( void )
 
 	}
 
-	if( tv.isEmpty() == TRUE )
+	if( tv.empty() == TRUE )
 	{
 		return;
 	}
 
 	// check for looping-mode and act if neccessary
 	timeLine * tl = m_playPos[m_playMode].m_timeLine;
-	if( tl != NULL && m_exporting == FALSE && tl->loopPointsEnabled() )
+	if( tl != NULL && m_exporting == FALSE && tl->loopPointsEnabled() &&
+		!( m_playMode == PLAY_PATTERN &&
+		  		m_patternToPlay->freezing() == TRUE ) )
 	{
 		if( m_playPos[m_playMode] < tl->loopBegin() ||
 					m_playPos[m_playMode] >= tl->loopEnd() )
@@ -925,7 +959,7 @@ void songEditor::processNextBuffer( void )
 	while( total_frames_played < mixer::inst()->framesPerAudioBuffer() )
 	{
 		Uint32 played_frames = mixer::inst()->framesPerAudioBuffer() -
-					total_frames_played;
+							total_frames_played;
 
 		// did we play a whole tact?
 		if( m_playPos[m_playMode].currentFrame() >= frames_per_tact )
@@ -972,6 +1006,7 @@ void songEditor::processNextBuffer( void )
 					m_playPos[m_playMode].currentFrame();
 		}
 
+		// loop through all tracks and play them if they're not muted
 		for( trackVector::iterator it = tv.begin(); it != tv.end();
 									++it )
 		{
@@ -986,9 +1021,8 @@ void songEditor::processNextBuffer( void )
 
 		// update frame-counters
 		total_frames_played += played_frames;
-		m_playPos[m_playMode].setCurrentFrame(
-					m_playPos[m_playMode].currentFrame() +
-					played_frames );
+		m_playPos[m_playMode].setCurrentFrame( played_frames +
+					m_playPos[m_playMode].currentFrame() );
 		m_playPos[m_playMode].setTact64th(
 					( m_playPos[m_playMode].currentFrame() *
 						64 / frames_per_tact) % 64 );
@@ -1000,33 +1034,6 @@ void songEditor::processNextBuffer( void )
 	{
 		m_playPos[m_playMode].m_timeLine->updatePosition(); 
 	}
-
-	if( m_exporting == TRUE )
-	{
-		tact tacts = lengthInTacts() + 1;
-		if( m_playPos[PLAY_SONG].getTact() >= tacts )
-		{
-			// now pause the mixer - method
-			// exportProjectDialog::redrawProgressBar() which is
-			// called every 100 ms will find out that export
-			// is done and will act according to this
-			mixer::inst()->pause();
-		}
-		else
-		{
-			m_epd->updateProgressBar(
-					( m_playPos[PLAY_SONG].getTact() * 64 +
-					m_playPos[PLAY_SONG].getTact64th() ) * 
-						100 / ( tacts * 64 ) );
-		}
-	}
-
-	if( m_playMode == PLAY_PATTERN && m_loopPattern == FALSE &&
-		m_patternToPlay->isFreezing() == TRUE &&
-		m_playPos[PLAY_PATTERN] > m_patternToPlay->length() )
-	{
-		m_patternToPlay->finishFreeze();
-	}
 }
 
 
@@ -1207,12 +1214,6 @@ void songEditor::stopExport( void )
 {
 	stop();
 	m_exporting = FALSE;
-
-	// if we rendered file from cmd-line quit after export
-	if( file_to_render != "" )
-	{
-		qApp->quit();
-	}
 }
 
 
@@ -1220,7 +1221,7 @@ void songEditor::stopExport( void )
 
 
 
-void songEditor::insertTact( void )
+void songEditor::insertBar( void )
 {
 	trackVector tv = tracks();
 	for( trackVector::iterator it = tv.begin(); it != tv.end(); ++it )
@@ -1233,7 +1234,7 @@ void songEditor::insertTact( void )
 
 
 
-void songEditor::removeTact( void )
+void songEditor::removeBar( void )
 {
 	trackVector tv = tracks();
 	for( trackVector::iterator it = tv.begin(); it != tv.end(); ++it )
@@ -1246,24 +1247,9 @@ void songEditor::removeTact( void )
 
 
 
-void songEditor::addChannelTrack( void )
-{
-	channelTrack * t = dynamic_cast< channelTrack * >(
-			track::createTrack( track::CHANNEL_TRACK, this ) );
-#ifdef LMMS_DEBUG
-	assert( t != NULL );
-#endif
-	t->loadPluginSettings( tripleOscillator::defaultSettings() );
-	t->toggledChannelButton( TRUE );
-	t->show();
-}
-
-
-
-
 void songEditor::addBBTrack( void )
 {
-	track * t = track::createTrack( track::BB_TRACK, this );
+	track * t = track::create( track::BB_TRACK, this );
 	if( dynamic_cast<bbTrack *>( t ) != NULL )
 	{
 		dynamic_cast<bbTrack *>( t )->clickedTrackLabel();
@@ -1275,7 +1261,7 @@ void songEditor::addBBTrack( void )
 
 void songEditor::addSampleTrack( void )
 {
-	(void) track::createTrack( track::SAMPLE_TRACK, this );
+	(void) track::create( track::SAMPLE_TRACK, this );
 }
 
 
@@ -1283,13 +1269,20 @@ void songEditor::addSampleTrack( void )
 
 float songEditor::framesPerTact( void ) const
 {
-	return( mixer::inst()->sampleRate() * 60.0f * MAIN_BEATS_PER_TACT /
+	return( mixer::inst()->sampleRate() * 60.0f * BEATS_PER_TACT /
 			m_bpmSpinBox->value() );
 }
 
 
 
 
+int songEditor::getBPM( void )
+{
+	return( m_bpmSpinBox->value() );
+}
+
+
+
 
 bool songEditor::mayChangeProject( void )
 {
@@ -1351,7 +1344,6 @@ void songEditor::clearProject( void )
 		// access non existing data (as you can see in the next lines,
 		// all data is cleared!)
 		stop();
-		doActions();
 	}
 
 	// make sure all running notes are cleared, otherwise the whole
@@ -1359,6 +1351,11 @@ void songEditor::clearProject( void )
 	//mixer::inst()->clear();
 	while( mixer::inst()->haveNoRunningNotes() == FALSE )
 	{
+#ifdef QT4
+		QApplication::processEvents( QEventLoop::AllEvents );
+#else
+		qApp->processEvents();
+#endif
 	}
 
 	trackVector tv = tracks();
@@ -1384,18 +1381,20 @@ void songEditor::createNewProject( void )
 	clearProject();
 
 	track * t;
-	t = track::createTrack( track::CHANNEL_TRACK, this );
-	dynamic_cast< channelTrack * >( t )->loadPluginSettings(
-					tripleOscillator::defaultSettings() );
-	track::createTrack( track::SAMPLE_TRACK, this );
-	t = track::createTrack( track::CHANNEL_TRACK, bbEditor::inst() );
-	dynamic_cast< channelTrack * >( t )->loadPluginSettings(
-					tripleOscillator::defaultSettings() );
-	track::createTrack( track::BB_TRACK, this );
+	t = track::create( track::CHANNEL_TRACK, this );
+	dynamic_cast< channelTrack * >( t )->loadInstrument(
+					"tripleoscillator" );
+	track::create( track::SAMPLE_TRACK, this );
+	t = track::create( track::CHANNEL_TRACK, bbEditor::inst() );
+	dynamic_cast< channelTrack * >( t )->loadInstrument(
+						"tripleoscillator" );
+	track::create( track::BB_TRACK, this );
 
+	m_loadingProject = TRUE;
 	setBPM( DEFAULT_BPM );
 	m_masterVolumeSlider->setValue( 100 );
 	m_masterPitchSlider->setValue( 0 );
+	m_loadingProject = FALSE;
 
 	m_fileName = m_oldFileName = "";
 
@@ -1427,6 +1426,8 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 	m_fileName = _file_name;
 	m_oldFileName = _file_name;
 
+	m_loadingProject = TRUE;
+
 	multimediaProject mmp( m_fileName );
 	// if file could not be opened, head-node is null and we create
 	// new project
@@ -1459,7 +1460,7 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 				else
 				{
 					m_masterVolumeSlider->setValue(
-						DEFAULT_VOLUME );
+							DEFAULT_VOLUME );
 				}
 			}
 			else if( node.nodeName() == "masterpitch" )
@@ -1493,6 +1494,8 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 	m_modified = FALSE;
 	m_leftRightScroll->setValue( 0 );
 
+	m_loadingProject = FALSE;
+
 	lmmsMainWin::inst()->resetWindowTitle( "" );
 }
 
@@ -1527,15 +1530,19 @@ bool songEditor::saveProject( void )
 	{
 		m_modified = FALSE;
 
-		lmmsMainWin::inst()->statusBar()->showMessage(
-					tr( "%1 saved." ).arg( m_fileName ),
-									3000 );
+		textFloat::displayMessage( tr( "Project saved" ),
+					tr( "The project %1 is now saved."
+							).arg( m_fileName ),
+				embed::getIconPixmap( "project_save", 24, 24 ),
+									2000 );
 		lmmsMainWin::inst()->resetWindowTitle( "" );
 	}
 	else
 	{
-		lmmsMainWin::inst()->statusBar()->showMessage(
-					tr( "Project NOT saved." ), 3000 );
+		textFloat::displayMessage( tr( "Project NOT saved." ),
+				tr( "The project %1 was not saved!" ).arg(
+							m_fileName ),
+				embed::getIconPixmap( "error" ), 4000 );
 		return( FALSE );
 	}
 	return( TRUE );
@@ -1562,25 +1569,24 @@ bool FASTCALL songEditor::saveProjectAs( const QString & _file_name )
 
 
 
+
 void songEditor::importProject( void )
 {
 #ifdef QT4
-		QFileDialog ofd( this, tr( "Import file" ), "",
+	QFileDialog ofd( this, tr( "Import file" ), "",
 					tr( "MIDI-files (*.mid)" ) );
 #else
-		QFileDialog ofd( QString::null,
-					tr( "MIDI-files (*.mid)" ),
+	QFileDialog ofd( QString::null, tr( "MIDI-files (*.mid)" ),
 							this, "", TRUE );
-		ofd.setWindowTitle( tr( "Import file" ) );
+	ofd.setWindowTitle( tr( "Import file" ) );
 #endif
-		ofd.setDirectory( configManager::inst()->projectsDir() );
-		ofd.setFileMode( QFileDialog::ExistingFiles );
-		if( ofd.exec () == QDialog::Accepted &&
-						!ofd.selectedFiles().isEmpty() )
-		{
-			midiFile mf( ofd.selectedFiles()[0] );
-			mf.importToTrackContainer( this );
-		}
+	ofd.setDirectory( configManager::inst()->projectsDir() );
+	ofd.setFileMode( QFileDialog::ExistingFiles );
+	if( ofd.exec () == QDialog::Accepted && !ofd.selectedFiles().isEmpty() )
+	{
+		midiFile mf( ofd.selectedFiles()[0] );
+		mf.importToTrackContainer( this );
+	}
 }
 
 
@@ -1592,15 +1598,7 @@ void songEditor::exportProject( void )
 
 	if( m_fileName != "" )
 	{
-#ifdef QT4
-		base_filename = QFileInfo( m_fileName ).absolutePath() + "/" +
-					QFileInfo( m_fileName
-							).completeBaseName();
-#else
-		base_filename = QFileInfo( m_fileName ).dirPath() + "/" +
-					QFileInfo( m_fileName ).baseName(
-									TRUE );
-#endif
+		base_filename = baseName( m_fileName );
 	}
 	else
 	{
@@ -1661,21 +1659,15 @@ void songEditor::exportProject( void )
 		{
 			return;
 		}
-
-		m_epd = new exportProjectDialog( export_file_name,
+		exportProjectDialog epd( export_file_name,
 							lmmsMainWin::inst() );
-		m_epd->exec();
-		delete m_epd;
-		m_epd = NULL;
+		epd.exec();
 	}
 }
 
 
 
 
-
-
-
 void songEditor::saveSettings( QDomDocument & _doc, QDomElement & _parent )
 {
 	trackContainer::saveSettings( _doc, _parent );
