@@ -2,8 +2,9 @@
  * track.cpp - implementation of classes concerning tracks -> neccessary for
  *             all track-like objects (beat/bassline, sample-track...)
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -39,6 +40,8 @@
 #include <qdom.h>
 #include <qpopupmenu.h>
 #include <qlayout.h>
+#include <qcursor.h>
+#include <qwhatsthis.h>
 
 #endif
 
@@ -55,6 +58,10 @@
 #include "pixmap_button.h"
 #include "debug.h"
 #include "tooltip.h"
+#include "string_pair_drag.h"
+#include "mmp.h"
+#include "lmms_main_win.h"
+#include "text_float.h"
 
 
 
@@ -65,6 +72,8 @@ const Uint16 TRACK_OP_BTN_HEIGHT = 14;
 
 
 
+textFloat * trackContentObject::s_textFloat = NULL;
+
 
 // ===========================================================================
 // trackContentObject
@@ -83,6 +92,13 @@ trackContentObject::trackContentObject( track * _track ) :
 	m_autoResize( FALSE ),
 	m_initialMouseX( 0 )
 {
+	if( s_textFloat == NULL )
+	{
+		s_textFloat = new textFloat( this );
+		s_textFloat->setPixmap( embed::getIconPixmap(
+						"xclock", 24, 24 ) );
+	}
+
 #ifdef QT4
 	setAttribute( Qt::WA_DeleteOnClose );
 	setFocusPolicy( Qt::StrongFocus );
@@ -92,9 +108,10 @@ trackContentObject::trackContentObject( track * _track ) :
 	show();
 	movePosition( 0 );
 	changeLength( 0 );
-	setFixedHeight( parentWidget()->height()-2 );
-//	if( useFixedWidth() )
-//		setFixedWidth( _channel_track->getTrackContentWidget()->width() );
+
+	setFixedHeight( parentWidget()->height() - 2 );
+	setAcceptDrops( TRUE );
+	setMouseTracking( TRUE );
 }
 
 
@@ -126,7 +143,6 @@ void trackContentObject::movePosition( const midiTime & _pos )
 	// moving of TCO can result in change of song-length etc.,
 	// therefore we update the trackcontainer
 	m_track->getTrackContainer()->update();
-	setMouseTracking( TRUE );
 }
 
 
@@ -163,25 +179,104 @@ float trackContentObject::pixelsPerTact( void )
 
 
 
+void trackContentObject::dragEnterEvent( QDragEnterEvent * _dee )
+{
+	stringPairDrag::processDragEnterEvent( _dee, "tco_" +
+					QString::number( m_track->type() ) );
+}
+
+
+
+
+void trackContentObject::dropEvent( QDropEvent * _de )
+{
+	QString type = stringPairDrag::decodeKey( _de );
+	QString value = stringPairDrag::decodeValue( _de );
+	if( type == ( "tco_" + QString::number( m_track->type() ) ) )
+	{
+		// value contains our XML-data so simply create a
+		// multimediaProject which does the rest for us...
+		multimediaProject mmp( value, FALSE );
+		// at least save position before getting to moved to somewhere
+		// the user doesn't expect...
+		midiTime pos = startPosition();
+		loadSettings( mmp.content().firstChild().toElement() );
+		movePosition( pos );
+		_de->accept();
+	}
+}
+
+
+
+
+void trackContentObject::leaveEvent( QEvent * _e )
+{
+	while( QApplication::overrideCursor() != NULL )
+	{
+		QApplication::restoreOverrideCursor();
+	}
+	if( _e != NULL )
+	{
+		QWidget::leaveEvent( _e );
+	}
+}
+
+
+
 
 void trackContentObject::mousePressEvent( QMouseEvent * _me )
 {
-	if( _me->button() == Qt::LeftButton && fixedTCOs() == FALSE )
+	if( _me->button() == Qt::LeftButton &&
+					lmmsMainWin::isCtrlPressed() == TRUE )
 	{
-		m_initialMouseX = _me->x() - TCO_BORDER_WIDTH;
+		multimediaProject mmp( multimediaProject::DRAG_N_DROP_DATA );
+		saveSettings( mmp, mmp.content() );
+#ifdef QT4
+		QPixmap thumbnail = QPixmap::grabWidget( this ).scaled(
+						128, 128,
+						Qt::KeepAspectRatio,
+						Qt::SmoothTransformation );
+#else
+		QSize s( size() );
+		s.scale( 128, 128, QSize::ScaleMin );
+		QPixmap thumbnail = QPixmap::grabWidget( this ).
+					convertToImage().smoothScale( s );
+#endif
+		new stringPairDrag( "tco_" +
+					QString::number( m_track->type() ),
+					mmp.toString(), thumbnail, this );
+	}
+	else if( _me->button() == Qt::LeftButton &&
+			/*	lmmsMainWin::isShiftPressed() == FALSE &&*/
+							fixedTCOs() == FALSE )
+	{
+		m_initialMouseX = _me->x();
 		
-		if( _me->x() < width()-RESIZE_GRIP_WIDTH )
+		if( _me->x() < width() - RESIZE_GRIP_WIDTH )
 		{
 			m_moving = TRUE;
 			QCursor c( Qt::SizeAllCursor );
 			QApplication::setOverrideCursor( c );
+			s_textFloat->setTitle( tr( "Current position" ) );
 		}
 		else if( m_autoResize == FALSE )
 		{
 			m_resizing = TRUE;
 			QCursor c( Qt::SizeHorCursor );
 			QApplication::setOverrideCursor( c );
+			s_textFloat->setTitle( tr( "Current length" ) );
 		}
+		s_textFloat->reparent( this );
+		// setup text-float as if TCO was already moved/resized
+		mouseMoveEvent( _me );
+		s_textFloat->show();
+	}
+	else if( ( _me->button() == Qt::MidButton/* ||
+			( _me->button() == Qt::LeftButton &&
+		  		lmmsMainWin::isShiftPressed() == TRUE )*/ ) &&
+							fixedTCOs() == FALSE )
+	{
+		close();
 	}
 }
 
@@ -198,15 +293,25 @@ void trackContentObject::mouseMoveEvent( QMouseEvent * _me )
 							currentPosition() +
 					static_cast<int>( x * 64 / ppt ) ) );
 		m_track->getTrackWidget()->changePosition();
+		s_textFloat->setText( QString( "%1:%2" ).
+					arg( m_startPosition.getTact() + 1 ).
+					arg( m_startPosition.getTact64th() ) );
+		s_textFloat->move( mapTo( topLevelWidget(), QPoint( 0, 0 ) ) +
+				QPoint( -2 - s_textFloat->width(), 8 ) );
 	}
 	else if( m_resizing )
 	{
 		changeLength( tMax( 64,
 				static_cast<int>( _me->x() * 64 / ppt ) ) );
+		s_textFloat->setText( QString( "%1:%2" ).
+					arg( length().getTact() + 1 ).
+					arg( length().getTact64th() ) );
+		s_textFloat->move( mapTo( topLevelWidget(), QPoint( 0, 0 ) ) +
+						QPoint( width() + 2, 8 ) );
 	}
 	else
 	{
-		if( _me->x() >= width()-RESIZE_GRIP_WIDTH )
+		if( _me->x() > width() - RESIZE_GRIP_WIDTH )
 		{
 			if( QApplication::overrideCursor() != NULL &&
 				QApplication::overrideCursor()->shape() !=
@@ -222,10 +327,7 @@ void trackContentObject::mouseMoveEvent( QMouseEvent * _me )
 		}
 		else
 		{
-			while( QApplication::overrideCursor() != NULL )
-			{
-	 			QApplication::restoreOverrideCursor();
-			}
+			leaveEvent( NULL );
 		}
 	}
 }
@@ -235,10 +337,8 @@ void trackContentObject::mouseMoveEvent( QMouseEvent * _me )
 
 void trackContentObject::mouseReleaseEvent( QMouseEvent * _me )
 {
-	while( QApplication::overrideCursor() != NULL )
-	{
-		QApplication::restoreOverrideCursor();
-	}
+	s_textFloat->hide();
+	leaveEvent( NULL );
 	m_moving = FALSE;
 	m_resizing = FALSE;
 }
@@ -252,7 +352,8 @@ void trackContentObject::contextMenuEvent( QContextMenuEvent * _cme )
 	if( fixedTCOs() == FALSE )
 	{
 		contextMenu.addAction( embed::getIconPixmap( "cancel" ),
-					tr( "Delete" ), this, SLOT( close() ) );
+					tr( "Delete (middle mousebutton)" ),
+							this, SLOT( close() ) );
 #ifdef QT4
 		contextMenu.addSeparator();
 #else
@@ -341,6 +442,7 @@ trackContentWidget::trackContentWidget( trackWidget * _parent ) :
 	setPaletteBackgroundColor( QColor( 96, 96, 96 ) );
 #endif
 	setMouseTracking( TRUE );
+	setAcceptDrops( TRUE );
 }
 
 
@@ -361,9 +463,8 @@ trackContentObject * FASTCALL trackContentWidget::getTCO( csize _tco_num )
 	}
 	printf( "called trackContentWidget::getTCO( %d, TRUE ), "
 			" but TCO %d doesn't exist\n", _tco_num, _tco_num );
-	//m_trackWidget->getTrack().addTCO(
-	//				m_trackWidget->getTrack().createTCO() );
-	return( NULL );
+	return( getTrack()->addTCO( getTrack()->createTCO( _tco_num * 64 ) ) );
+//	return( NULL );
 }
 
 
@@ -381,7 +482,7 @@ trackContentObject * FASTCALL trackContentWidget::addTCO(
 						trackContentObject * _tco )
 {
 	m_trackContentObjects.push_back( _tco );
-	_tco->move( 0, 1 );//(m_trackWidget->height()-_tco->height())/2 );
+	_tco->move( 0, 1 );
 	m_trackWidget->changePosition();
 	songEditor::inst()->setModified();
 	return( _tco );		// just for convenience
@@ -420,9 +521,9 @@ void trackContentWidget::removeTCO( trackContentObject * _tco,
 
 void trackContentWidget::removeAllTCOs( void )
 {
-	while( !m_trackContentObjects.isEmpty() )
+	while( !m_trackContentObjects.empty() )
 	{
-		delete m_trackContentObjects.first();
+		delete m_trackContentObjects.front();
 		m_trackContentObjects.erase( m_trackContentObjects.begin() );
 	}
 }
@@ -508,24 +609,49 @@ void trackContentWidget::updateTCOs( void )
 
 
 
+void trackContentWidget::dragEnterEvent( QDragEnterEvent * _dee )
+{
+	stringPairDrag::processDragEnterEvent( _dee, "tco_" +
+					QString::number( getTrack()->type() ) );
+}
+
+
+
+
+void trackContentWidget::dropEvent( QDropEvent * _de )
+{
+	QString type = stringPairDrag::decodeKey( _de );
+	QString value = stringPairDrag::decodeValue( _de );
+	if( type == ( "tco_" + QString::number( getTrack()->type() ) ) &&
+			getTrack()->getTrackContainer()->fixedTCOs() == FALSE )
+	{
+		const midiTime position = getPosition( _de->pos().x() );
+		trackContentObject * tco = addTCO( getTrack()->createTCO(
+								position ) );
+		// value contains our XML-data so simply create a
+		// multimediaProject which does the rest for us...
+		multimediaProject mmp( value, FALSE );
+		// at least save position before getting to moved to somewhere
+		// the user doesn't expect...
+		tco->loadSettings( mmp.content().firstChild().toElement() );
+		tco->movePosition( position );
+		_de->accept();
+	}
+}
+
+
+
+
+
+
 void trackContentWidget::mousePressEvent( QMouseEvent * _me )
 {
 	if( _me->button() == Qt::LeftButton &&
-		m_trackWidget->getTrack()->getTrackContainer()->fixedTCOs() ==
-									FALSE )
+			getTrack()->getTrackContainer()->fixedTCOs() == FALSE )
 	{
-		const midiTime position = midiTime( m_trackWidget->getTrack()->
-						getTrackContainer()->
-						currentPosition() + _me->x() *
-						64 /
-						static_cast<int>(
-							m_trackWidget->
-							getTrack()->
-							getTrackContainer()->
-							pixelsPerTact()
-						) );
-		trackContentObject * tco = addTCO( m_trackWidget->getTrack()->
-							createTCO( position ) );
+		const midiTime position = getPosition( _me->x() );
+		trackContentObject * tco = addTCO( getTrack()->createTCO(
+								position ) );
 		tco->movePosition( position );
 	}
 }
@@ -567,6 +693,61 @@ void trackContentWidget::resizeEvent( QResizeEvent * _re )
 
 
 
+track * trackContentWidget::getTrack( void )
+{
+	return( m_trackWidget->getTrack() );
+}
+
+
+
+
+midiTime trackContentWidget::getPosition( int _mouse_x )
+{
+	const trackContainer * tc = getTrack()->getTrackContainer();
+	return( midiTime( tc->currentPosition() + _mouse_x * 64 /
+				static_cast<int>( tc->pixelsPerTact() ) ) );
+}
+
+
+
+
+// ===========================================================================
+// trackSettingsWidget
+// ===========================================================================
+
+trackSettingsWidget::trackSettingsWidget( trackWidget * _parent ) :
+	QWidget( _parent ),
+	m_trackWidget( _parent )
+{
+}
+
+
+
+
+trackSettingsWidget::~trackSettingsWidget()
+{
+}
+
+
+
+
+void trackSettingsWidget::mousePressEvent( QMouseEvent * _me )
+{
+	if( _me->button() == Qt::LeftButton &&
+			m_trackWidget->getTrack()->type() != track::BB_TRACK )
+	{
+		multimediaProject mmp( multimediaProject::DRAG_N_DROP_DATA );
+		m_trackWidget->getTrack()->saveSettings( mmp, mmp.content() );
+		new stringPairDrag( "track_" +
+			QString::number( m_trackWidget->getTrack()->type() ),
+			mmp.toString(), QPixmap::grabWidget( this ), this );
+	}
+}
+
+
+
+
+
 
 // ===========================================================================
 // trackWidget
@@ -593,7 +774,7 @@ trackWidget::trackWidget( track * _track, QWidget * _parent ) :
 
 
 	QPushButton * clntr_btn = new QPushButton( embed::getIconPixmap(
-						"pr_edit_copy", 12, 12 ),
+						"edit_copy", 12, 12 ),
 									"",
 						&m_trackOperationsWidget );
 	clntr_btn->setGeometry( 1, 1, TRACK_OP_BTN_WIDTH, TRACK_OP_BTN_HEIGHT );
@@ -607,10 +788,10 @@ trackWidget::trackWidget( track * _track, QWidget * _parent ) :
 						"", &m_trackOperationsWidget );
 	deltr_btn->setGeometry( 1, 1+TRACK_OP_BTN_HEIGHT, TRACK_OP_BTN_WIDTH,
 							TRACK_OP_BTN_HEIGHT );
-	connect( deltr_btn, SIGNAL( clicked() ), this, SLOT( deleteTrack() ) );
+	connect( deltr_btn, SIGNAL( clicked() ), this, SLOT( removeTrack() ) );
 	connect( deltr_btn, SIGNAL( clicked() ), deltr_btn,
 							SLOT( clearFocus() ) );
-	toolTip::add( deltr_btn, tr( "Delete this track" ) );
+	toolTip::add( deltr_btn, tr( "Remove this track" ) );
 
 	QPushButton * muptr_btn = new QPushButton( embed::getIconPixmap(
 							"arp_up_on", 12, 12 ),
@@ -674,6 +855,8 @@ trackWidget::trackWidget( track * _track, QWidget * _parent ) :
 #ifndef QT4
 	setBackgroundMode( Qt::NoBackground );
 #endif
+	setAcceptDrops( TRUE );
+	setMouseTracking( TRUE );
 }
 
 
@@ -686,62 +869,6 @@ trackWidget::~trackWidget()
 
 
 
-void trackWidget::paintEvent( QPaintEvent * _pe )
-{
-#ifdef QT4
-	QPainter p( this );
-#else
-	// create pixmap for whole widget
-	QPixmap pm( rect().size() );
-	pm.fill( QColor( 128, 128, 128 ) );
-
-	// and a painter for it
-	QPainter p( &pm );
-#endif
-	p.setPen( QColor( 0, 0, 0 ) );
-	p.drawLine( 0, height()-1, width()-1, height()-1 );
-
-#ifndef QT4
-	// blit drawn pixmap to actual widget
-	bitBlt( this, rect().topLeft(), &pm );
-#endif
-}
-
-
-
-
-void trackWidget::cloneTrack( void )
-{
-	m_track->getTrackContainer()->cloneTrack( m_track );
-}
-
-
-
-
-void trackWidget::deleteTrack( void )
-{
-	m_track->getTrackContainer()->removeTrack( m_track );
-}
-
-
-
-
-void trackWidget::moveTrackUp( void )
-{
-	m_track->getTrackContainer()->moveTrackUp( m_track );
-}
-
-
-
-
-void trackWidget::moveTrackDown( void )
-{
-	m_track->getTrackContainer()->moveTrackDown( m_track );
-}
-
-
-
-
 bool trackWidget::muted( void ) const
 {
 #ifdef QT4
@@ -754,39 +881,6 @@ bool trackWidget::muted( void ) const
 
 
 
-void trackWidget::setMuted( bool _muted )
-{
-#ifdef QT4
-	m_muteBtn->setChecked( _muted );
-#else
-	m_muteBtn->setOn( _muted );
-#endif
-	m_trackContentWidget.updateTCOs();
-}
-
-
-
-
-void trackWidget::muteBtnRightClicked( void )
-{
-	bool m = muted();
-	m_track->getTrackContainer()->setMutedOfAllTracks( m );
-	setMuted( !m );
-}
-
-
-
-
-midiTime trackWidget::endPosition( const midiTime & _pos_start )
-{
-	const float ppt = m_track->getTrackContainer()->pixelsPerTact();
-	const int cww = m_trackContentWidget.width();
-	return( _pos_start + static_cast<int>( cww * 64 / ppt ) );
-}
-
-
-
-
 // resposible for moving track-content-widgets to appropriate position after
 // change of visible viewport
 void trackWidget::changePosition( const midiTime & _new_pos )
@@ -827,6 +921,111 @@ void trackWidget::changePosition( const midiTime & _new_pos )
 
 
 
+void trackWidget::cloneTrack( void )
+{
+	m_track->getTrackContainer()->cloneTrack( m_track );
+}
+
+
+
+
+void trackWidget::removeTrack( void )
+{
+	m_track->getTrackContainer()->removeTrack( m_track );
+}
+
+
+
+
+void trackWidget::moveTrackUp( void )
+{
+	m_track->getTrackContainer()->moveTrackUp( m_track );
+}
+
+
+
+
+void trackWidget::moveTrackDown( void )
+{
+	m_track->getTrackContainer()->moveTrackDown( m_track );
+}
+
+
+
+
+void trackWidget::setMuted( bool _muted )
+{
+#ifdef QT4
+	m_muteBtn->setChecked( _muted );
+#else
+	m_muteBtn->setOn( _muted );
+#endif
+	m_trackContentWidget.updateTCOs();
+}
+
+
+
+
+void trackWidget::muteBtnRightClicked( void )
+{
+	bool m = muted();
+	m_track->getTrackContainer()->setMutedOfAllTracks( m );
+	setMuted( !m );
+}
+
+
+
+
+void trackWidget::dragEnterEvent( QDragEnterEvent * _dee )
+{
+	stringPairDrag::processDragEnterEvent( _dee, "track_" +
+					QString::number( m_track->type() ) );
+}
+
+
+
+
+void trackWidget::dropEvent( QDropEvent * _de )
+{
+	QString type = stringPairDrag::decodeKey( _de );
+	QString value = stringPairDrag::decodeValue( _de );
+	if( type == ( "track_" + QString::number( m_track->type() ) ) )
+	{
+		// value contains our XML-data so simply create a
+		// multimediaProject which does the rest for us...
+		multimediaProject mmp( value, FALSE );
+		m_track->loadSettings( mmp.content().firstChild().toElement() );
+		_de->accept();
+	}
+}
+
+
+
+
+void trackWidget::paintEvent( QPaintEvent * _pe )
+{
+#ifdef QT4
+	QPainter p( this );
+#else
+	// create pixmap for whole widget
+	QPixmap pm( rect().size() );
+	pm.fill( QColor( 128, 128, 128 ) );
+
+	// and a painter for it
+	QPainter p( &pm );
+#endif
+	p.setPen( QColor( 0, 0, 0 ) );
+	p.drawLine( 0, height() - 1, width() - 1, height() - 1 );
+
+#ifndef QT4
+	// blit drawn pixmap to actual widget
+	bitBlt( this, rect().topLeft(), &pm );
+#endif
+}
+
+
+
+
 void trackWidget::resizeEvent( QResizeEvent * _re )
 {
 	m_trackOperationsWidget.setFixedSize( TRACK_OP_WIDTH, height() - 1 );
@@ -845,6 +1044,18 @@ void trackWidget::resizeEvent( QResizeEvent * _re )
 
 
 
+midiTime trackWidget::endPosition( const midiTime & _pos_start )
+{
+	const float ppt = m_track->getTrackContainer()->pixelsPerTact();
+	const int cww = m_trackContentWidget.width();
+	return( _pos_start + static_cast<int>( cww * 64 / ppt ) );
+}
+
+
+
+
+
+
 // ===========================================================================
 // track
 // ===========================================================================
@@ -873,7 +1084,7 @@ track::~track()
 
 
 
-track * FASTCALL track::createTrack( trackTypes _tt, trackContainer * _tc )
+track * FASTCALL track::create( trackTypes _tt, trackContainer * _tc )
 {
 	switch( _tt )
 	{
@@ -891,10 +1102,11 @@ track * FASTCALL track::createTrack( trackTypes _tt, trackContainer * _tc )
 
 
 
-track * FASTCALL track::createTrack( const QDomElement & _this,
+
+track * FASTCALL track::create( const QDomElement & _this,
 							trackContainer * _tc )
 {
-	track * t = createTrack( static_cast<trackTypes>( _this.attribute(
+	track * t = create( static_cast<trackTypes>( _this.attribute(
 						"type" ).toInt() ), _tc );
 #ifdef LMMS_DEBUG
 	assert( t != NULL );
@@ -906,13 +1118,13 @@ track * FASTCALL track::createTrack( const QDomElement & _this,
 
 
 
-track * FASTCALL track::cloneTrack( track * _track )
+track * FASTCALL track::clone( track * _track )
 {
 	QDomDocument doc;
 	QDomElement parent = doc.createElement( "clone" );
 	_track->saveSettings( doc, parent );
 	QDomElement e = parent.firstChild().toElement();
-	return( createTrack( e, _track->getTrackContainer() ) );
+	return( create( e, _track->getTrackContainer() ) );
 }
 
 
@@ -931,7 +1143,7 @@ void FASTCALL track::saveSettings( QDomDocument & _doc, QDomElement & _parent )
 	csize num_of_tcos = getTrackContentWidget()->numOfTCOs();
 
 	QDomElement track_de = _doc.createElement( "track" );
-	track_de.setAttribute( "type", QString::number( trackType() ) );
+	track_de.setAttribute( "type", QString::number( type() ) );
 	track_de.setAttribute( "muted", muted() );
 	_parent.appendChild( track_de );
 
@@ -952,7 +1164,7 @@ void FASTCALL track::saveSettings( QDomDocument & _doc, QDomElement & _parent )
 
 void FASTCALL track::loadSettings( const QDomElement & _this )
 {
-	if( _this.attribute( "type" ).toInt() != trackType() )
+	if( _this.attribute( "type" ).toInt() != type() )
 	{
 		qWarning( "Current track-type does not match track-type of "
 							"settings-node!\n" );
