@@ -1,8 +1,9 @@
 /*
  * sample_buffer.cpp - container-class sampleBuffer
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -36,6 +37,7 @@
 #include <QFileDialog>
 #include <QFileInfo>
 #include <QFile>
+#include <QBuffer>
 
 #else
 
@@ -45,6 +47,7 @@
 #include <qfiledialog.h>
 #include <qfileinfo.h>
 #include <qfile.h>
+#include <qbuffer.h>
 
 #if QT_VERSION < 0x030100
 #include <qregexp.h>
@@ -67,6 +70,14 @@
 #include <vorbis/vorbisfile.h>
 #endif
 
+#ifdef HAVE_FLAC_STREAM_ENCODER_H
+#include <FLAC/stream_encoder.h>
+#endif
+
+#ifdef HAVE_FLAC_STREAM_DECODER_H
+#include <FLAC/stream_decoder.h>
+#endif
+
 
 #include "sample_buffer.h"
 #include "interpolation.h"
@@ -77,10 +88,21 @@
 #include "debug.h"
 
 
+#ifndef QT4
 
-sampleBuffer::sampleBuffer( const QString & _audio_file ) :
+#define write writeBlock
+#define read readBlock
+#define seek at 
+#define pos at
+
+#endif
+
+
+
+sampleBuffer::sampleBuffer( const QString & _audio_file,
+							bool _is_base64_data ) :
 	QObject(),
-	m_audioFile( _audio_file ),
+	m_audioFile( ( _is_base64_data == TRUE ) ? "" : _audio_file ),
 	m_origData( NULL ),
 	m_origFrames( 0 ),
 	m_data( NULL ),
@@ -98,6 +120,10 @@ sampleBuffer::sampleBuffer( const QString & _audio_file ) :
 #ifdef HAVE_SAMPLERATE_H
 	initResampling();
 #endif
+	if( _is_base64_data == TRUE )
+	{
+		loadFromBase64( _audio_file );
+	}
 	update();
 }
 
@@ -118,7 +144,7 @@ sampleBuffer::sampleBuffer( const sampleFrame * _data, Uint32 _frames ) :
 	m_dataMutex()
 {
 	m_origData = new sampleFrame[_frames];
-	memcpy( m_origData, _data, _frames*BYTES_PER_FRAME );
+	memcpy( m_origData, _data, _frames * BYTES_PER_FRAME );
 	m_origFrames = _frames;
 #ifdef SDL_SDL_SOUND_H
 	// init sound-file-system of SDL
@@ -170,7 +196,14 @@ void sampleBuffer::update( bool _keep_settings )
 		{
 			m_frames = m_origFrames;
 			m_startFrame = 0;
-			m_endFrame = m_frames-1;
+			if( m_frames > 0 )
+			{
+				m_endFrame = m_frames - 1;
+			}
+			else
+			{
+				m_endFrame = 0;
+			}
 		}
 	}
 	else if( m_audioFile != "" )
@@ -192,18 +225,18 @@ void sampleBuffer::update( bool _keep_settings )
 		Sint16 * buf = NULL;
 		Uint8 channels;
 
-#ifdef SDL_SDL_SOUND_H
-		if( m_frames == 0 )
-		{
-			m_frames = decodeSampleSDL( f, buf, channels );
-		}
-#endif
 #ifdef HAVE_SNDFILE_H
 		if( m_frames == 0 )
 		{
 			m_frames = decodeSampleSF( f, buf, channels );
 		}
 #endif
+#ifdef SDL_SDL_SOUND_H
+		if( m_frames == 0 )
+		{
+			m_frames = decodeSampleSDL( f, buf, channels );
+		}
+#endif
 #ifdef HAVE_VORBIS_VORBISFILE_H
 		if( m_frames == 0 )
 		{
@@ -216,7 +249,14 @@ void sampleBuffer::update( bool _keep_settings )
 			{
 				// update frame-variables
 				m_startFrame = 0;
-				m_endFrame = m_frames - 1;
+				if( m_frames > 0 )
+				{
+					m_endFrame = m_frames - 1;
+				}
+				else
+				{
+					m_endFrame = 0;
+				}
 			}
 
 			// following code transforms int-samples into
@@ -261,21 +301,19 @@ m_data[frame][chnl] = buf[idx] * fac;
 		else
 		{
 			m_data = new sampleFrame[1];
-			for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
-			{
-				m_data[0][chnl] = 0.0f;
-			}
+			memset( m_data, 0, sizeof( *m_data ) );
 			m_frames = 1;
+			m_startFrame = 0;
+			m_endFrame = 1;
 		}
 	}
 	else
 	{
 		m_data = new sampleFrame[1];
-		for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
-		{
-			m_data[0][chnl] = 0.0f;
-		}
+		memset( m_data, 0, sizeof( *m_data ) * 1 );
 		m_frames = 1;
+		m_startFrame = 0;
+		m_endFrame = 1;
 	}
 
 	m_dataMutex.unlock();
@@ -300,7 +338,7 @@ Uint32 sampleBuffer::decodeSampleSDL( const char * _f, Sint16 * & _buf,
 
 	Sound_Sample * snd_sample = Sound_NewSampleFromFile( _f,
 						&STD_AUDIO_INFO, 16384 );
-	// file not found?
+	// file found?
 	if( snd_sample != NULL )
 	{
 		// let SDL_sound decode our file to requested format
@@ -361,9 +399,6 @@ Uint32 sampleBuffer::decodeSampleSF( const char * _f, Sint16 * & _buf,
 // callback-functions for reading ogg-file
 
 #ifndef QT4
-#define read readBlock
-#define seek at 
-#define pos at
 #endif
 
 size_t qfileReadCallback( void * _ptr, size_t _size, size_t _n, void * _udata )
@@ -411,9 +446,6 @@ long qfileTellCallback( void * _udata )
 	return( static_cast<QFile *>( _udata )->pos() );
 }
 
-#undef read
-#undef seek
-#undef pos
 
 
 
@@ -529,12 +561,10 @@ SRC_STATE * sampleBuffer::createResamplingContext( void )
 {
 	int error;
 	SRC_STATE * state;
-	if( ( state = src_new(
-#ifdef HQ_SINC
-					SRC_SINC_MEDIUM_QUALITY,
-#else
-					SRC_ZERO_ORDER_HOLD,
-#endif
+	if( ( state = src_new(/*
+		( mixer::inst()->highQuality() == TRUE ) ?
+					SRC_SINC_FASTEST :*/
+					SRC_LINEAR,
 					DEFAULT_CHANNELS, &error ) ) == NULL )
 	{
 		printf( "Error: src_new() failed in sample_buffer.cpp!\n" );
@@ -566,7 +596,7 @@ bool FASTCALL sampleBuffer::play( sampleFrame * _ab, Uint32 _start_frame,
 		return( FALSE );
 	}
 
-	const float freq_factor = 1.0f / (BASE_FREQ / _freq);
+	const double freq_factor = (double) _freq / (double) BASE_FREQ;
 	const Sint16 freq_diff = static_cast<Sint16>( BASE_FREQ - _freq );
 
 	Uint32 frames_to_process = _frames;
@@ -579,13 +609,6 @@ bool FASTCALL sampleBuffer::play( sampleFrame * _ab, Uint32 _start_frame,
 	{
 		return( FALSE );
 	}
-	// this holds the number of the first frame to play
-	const Uint32 play_frame = m_startFrame + ( _start_frame %
-					total_frames_for_current_pitch );
-
-	// this holds the number of remaining frames in current loop
-	Uint32 frames_for_loop = total_frames_for_current_pitch -
-						( play_frame - m_startFrame );
 
 	// do we have frames left?? this is only important when not in
 	// looping-mode because in looping-mode we loop to start-frame...
@@ -594,6 +617,14 @@ bool FASTCALL sampleBuffer::play( sampleFrame * _ab, Uint32 _start_frame,
 		return( FALSE );
 	}
 
+	// this holds the number of the first frame to play
+	const Uint32 play_frame = m_startFrame + ( _start_frame %
+					total_frames_for_current_pitch );
+
+	// this holds the number of remaining frames in current loop
+	Uint32 frames_for_loop = total_frames_for_current_pitch -
+						( play_frame - m_startFrame );
+
 	// make sure, data isn't accessed in any other way (e.g. deleting
 	// of this buffer...)
 	m_dataMutex.lock();
@@ -602,11 +633,21 @@ bool FASTCALL sampleBuffer::play( sampleFrame * _ab, Uint32 _start_frame,
 	{
 		frames_to_process = frames_for_loop;
 	}
-
+	const Uint32 f1 = static_cast<Uint32>( play_frame * freq_factor );
+/*	Uint32 f2 = 0;
+	while( f2 < f1 )
+	{
+		f2 += frames_to_process * freq_factor;
+	}
+	if( f2 > f1 && f2 >= frames_to_process )
+	{
+		f2 -= frames_to_process * freq_factor;
+	}*/
+//	static int foo = 0;
 	// calc pointer of first frame
-	sampleFrame * start_frame = (sampleFrame *) m_data +
-					static_cast<Uint32>( play_frame *
-								freq_factor );
+	sampleFrame * start_frame = (sampleFrame *) m_data + f1;
+	//printf("diff:%d %f  %d f2: %d  input: %d\n", f2 -foo, play_frame * freq_factor, static_cast<Uint32>( play_frame * freq_factor ), f2, (Uint32)( frames_for_loop * freq_factor ) );
+//	foo = f2;
 	sampleFrame * loop_start = start_frame;
 
 	// check whether we have to change pitch...
@@ -637,7 +678,7 @@ bool FASTCALL sampleBuffer::play( sampleFrame * _ab, Uint32 _start_frame,
 #else
 		Uint32 src_frame_base = 0;
 		// check whether we're in high-quality-mode
-		if( mixer::inst()->highQuality() )
+		if( mixer::inst()->highQuality() == TRUE )
 		{
 			// we are, so let's use cubic interpolation...
 			for( Uint32 frame = 0; frame < frames_to_process;
@@ -755,8 +796,9 @@ bool FASTCALL sampleBuffer::play( sampleFrame * _ab, Uint32 _start_frame,
 
 void sampleBuffer::drawWaves( QPainter & _p, QRect _dr, drawMethods _dm )
 {
-	_p.setClipRect( _dr );
-	_p.setPen (QColor(0x22, 0xFF, 0x44));
+//	_p.setClipRect( _dr );
+//	_p.setPen( QColor( 0x22, 0xFF, 0x44 ) );
+	//_p.setPen( QColor( 64, 224, 160 ) );
 #ifdef QT4
 	// TODO: save and restore aa-settings
 	_p.setRenderHint( QPainter::Antialiasing );
@@ -764,12 +806,12 @@ void sampleBuffer::drawWaves( QPainter & _p, QRect _dr, drawMethods _dm )
 	const int w = _dr.width();
 	const int h = _dr.height();
 
-	const Uint16 y_base = h/2+_dr.y();
-	const float y_space = h/2;
+	const Uint16 y_base = h / 2 + _dr.y();
+	const float y_space = h / 2;
 
 	if( m_data == NULL || m_frames == 0 )
 	{
-		_p.drawLine( _dr.x(), y_base, _dr.x()+w, y_base );
+		_p.drawLine( _dr.x(), y_base, _dr.x() + w, y_base );
 		return;
 	}
 	else if( _dm == LINE_CONNECT )
@@ -805,11 +847,11 @@ void sampleBuffer::drawWaves( QPainter & _p, QRect _dr, drawMethods _dm )
 		{
 			const int x = static_cast<int>( frame /
 							(float) m_frames * w ) +
-					_dr.x();
+								_dr.x();
 			for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 			{
 				const Uint16 y = y_base +
-			static_cast<Uint16>( m_data[frame][chnl]*y_space );
+			static_cast<Uint16>( m_data[frame][chnl] * y_space );
 				_p.drawLine( old_x, old_y[chnl], x, y );
 				old_y[chnl] = y;
 			}
@@ -826,12 +868,11 @@ void sampleBuffer::drawWaves( QPainter & _p, QRect _dr, drawMethods _dm )
 			for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 			{
 				_p.drawPoint( x, y_base +
-					static_cast<Uint16>(
-						m_data[frame][chnl]*y_space ) );
+			static_cast<Uint16>( m_data[frame][chnl] * y_space ) );
 			}
 		}
 	}
-	_p.setClipping( FALSE );
+//	_p.setClipping( FALSE );
 }
 
 
@@ -932,6 +973,186 @@ QString sampleBuffer::openAudioFile( void ) const
 }
 
 
+#undef HAVE_FLAC_STREAM_ENCODER_H	/* not yet... */
+#undef HAVE_FLAC_STREAM_DECODER_H
+
+#ifdef HAVE_FLAC_STREAM_ENCODER_H
+FLAC__StreamEncoderWriteStatus flacStreamEncoderWriteCallback(
+					const FLAC__StreamEncoder *
+								/*_encoder*/,
+					const FLAC__byte _buffer[], 
+					unsigned int/* _samples*/,
+					unsigned int _bytes,
+					unsigned int/* _current_frame*/,
+					void * _client_data )
+{
+/*	if( _bytes == 0 )
+	{
+		return( FLAC__STREAM_ENCODER_WRITE_STATUS_OK );
+	}*/
+	return( ( static_cast<QBuffer *>( _client_data )->write(
+				(const char *) _buffer, _bytes ) ==
+								(int) _bytes ) ?
+				FLAC__STREAM_ENCODER_WRITE_STATUS_OK :
+				FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR );
+}
+
+
+void flacStreamEncoderMetadataCallback( const FLAC__StreamEncoder *,
+					const FLAC__StreamMetadata * _metadata,
+					void * _client_data )
+{
+	QBuffer * b = static_cast<QBuffer *>( _client_data );
+	b->seek( 0 );
+	b->write( (const char *) _metadata, sizeof( *_metadata ) );
+}
+
+#endif
+
+
+QString sampleBuffer::toBase64( void ) const
+{
+	if( m_data == NULL || m_frames == 0 )
+	{
+		return( "" );
+	}
+#ifdef HAVE_FLAC_STREAM_ENCODER_H
+	const Uint32 FRAMES_PER_BUF = 1152;
+
+	FLAC__StreamEncoder * flac_enc = FLAC__stream_encoder_new();
+	FLAC__stream_encoder_set_channels( flac_enc, DEFAULT_CHANNELS );
+	FLAC__stream_encoder_set_blocksize( flac_enc, FRAMES_PER_BUF );
+/*	FLAC__stream_encoder_set_do_exhaustive_model_search( flac_enc, TRUE );
+	FLAC__stream_encoder_set_do_mid_side_stereo( flac_enc, TRUE );*/
+	FLAC__stream_encoder_set_sample_rate( flac_enc,
+						mixer::inst()->sampleRate() );
+	QBuffer ba_writer;
+#ifdef QT4
+	ba_writer.open( QBuffer::WriteOnly );
+#else
+	ba_writer.open( IO_WriteOnly );
+#endif
+
+	FLAC__stream_encoder_set_write_callback( flac_enc,
+					flacStreamEncoderWriteCallback );
+	FLAC__stream_encoder_set_metadata_callback( flac_enc,
+					flacStreamEncoderMetadataCallback );
+	FLAC__stream_encoder_set_client_data( flac_enc, &ba_writer );
+	if( FLAC__stream_encoder_init( flac_enc ) != FLAC__STREAM_ENCODER_OK )
+	{
+		printf( "error within FLAC__stream_encoder_init()!\n" );
+	}
+	Uint32 frame_cnt = 0;
+	while( frame_cnt < m_frames )
+	{
+		Uint32 remaining = tMin<Uint32>( FRAMES_PER_BUF,
+							m_frames - frame_cnt );
+		FLAC__int32 buf[FRAMES_PER_BUF * DEFAULT_CHANNELS];
+		for( Uint32 f = 0; f < remaining; ++f )
+		{
+			for( Uint8 ch = 0; ch < DEFAULT_CHANNELS; ++ch )
+			{
+				buf[f*DEFAULT_CHANNELS+ch] = (FLAC__int32)(
+					mixer::clip( m_data[f+frame_cnt][ch] ) *
+						OUTPUT_SAMPLE_MULTIPLIER );
+			}
+		}
+		FLAC__stream_encoder_process_interleaved( flac_enc, buf,
+								remaining );
+		frame_cnt += remaining;
+	}
+	FLAC__stream_encoder_finish( flac_enc );
+	FLAC__stream_encoder_delete( flac_enc );
+	printf("%d %d\n", frame_cnt, (int)ba_writer.size() );
+	ba_writer.close();
+#ifdef QT4
+	return( ba_writer.data().toBase64() );
+#else
+	QByteArray ba = ba_writer.buffer();
+	const Uint32 ssize = ba.size();
+	const Uint8 * src = (const Uint8 *) ba.data();
+#endif
+
+#else	/* HAVE_FLAC_STREAM_ENCODER_H */
+
+#ifdef QT4
+	return( QByteArray( (const char *) m_data, m_frames *
+					sizeof( sampleFrame ) ).toBase64() );
+#else
+	const Uint32 ssize = m_frames * sizeof( sampleFrame );
+	const Uint8 * src = (const Uint8 *) m_data;
+#endif
+
+#endif	/* HAVE_FLAC_STREAM_ENCODER_H */
+
+
+#ifndef QT4
+	// code mostly taken from
+	// qt-x11-opensource-src-4.0.1/src/corelib/tools/qbytearray.cpp
+
+	const char alphabet[] = "ABCDEFGH" "IJKLMNOP" "QRSTUVWX" "YZabcdef"
+				"ghijklmn" "opqrstuv" "wxyz0123" "456789+/";
+	const char padchar = '=';
+	int padlen = 0;
+
+	const Uint32 dsize = ( ( ssize * 4 ) / 3 ) + 3;
+	char * ptr = new char[dsize + 1];
+	char * out = ptr;
+
+	Uint32 i = 0;
+	while( i < ssize )
+	{
+		Uint32 chunk = 0;
+		chunk |= Uint32( src[i++] ) << 16;
+		if( i == dsize )
+		{
+			padlen = 2;
+		}
+		else
+		{
+			chunk |= Uint32( src[i++] ) << 8;
+			if( i == ssize )
+			{
+				padlen = 1;
+			}
+			else
+			{
+				chunk |= Uint32( src[i++] );
+			}
+		}
+
+		Uint32 j = ( chunk & 0x00fc0000 ) >> 18;
+		Uint32 k = ( chunk & 0x0003f000 ) >> 12;
+		Uint32 l = ( chunk & 0x00000fc0 ) >> 6;
+		Uint32 m = ( chunk & 0x0000003f );
+		*out++ = alphabet[j];
+		*out++ = alphabet[k];
+		if( padlen > 1 )
+		{
+			*out++ = padchar;
+		}
+		else
+		{
+			*out++ = alphabet[l];
+		}
+		if( padlen > 0 )
+		{
+			*out++ = padchar;
+		}
+		else
+		{
+			*out++ = alphabet[m];
+		}
+	}
+	// terminate string
+	ptr[out - ptr] = 0;
+	QString s( ptr );
+	delete[] ptr;
+	return( s );
+#endif
+}
+
+
 
 
 void sampleBuffer::setAudioFile( const QString & _audio_file )
@@ -955,6 +1176,213 @@ void sampleBuffer::setAudioFile( const QString & _audio_file )
 
 
 
+#ifdef HAVE_FLAC_STREAM_DECODER_H
+
+struct flacStreamDecoderClientData
+{
+	QBuffer * read_buffer;
+	QBuffer * write_buffer;
+} ;
+
+
+
+FLAC__StreamDecoderReadStatus flacStreamDecoderReadCallback(
+					const FLAC__StreamDecoder *
+								/*_decoder*/,
+					FLAC__byte * _buffer,
+					unsigned int * _bytes,
+					void * _client_data )
+{
+	int res = static_cast<flacStreamDecoderClientData *>(
+					_client_data )->read_buffer->read(
+						(char *) _buffer, *_bytes );
+
+	if( res > 0 )
+	{
+		*_bytes = res;
+		return( FLAC__STREAM_DECODER_READ_STATUS_CONTINUE );
+
+	}
+	*_bytes = 0;
+	return( FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM );
+}
+
+
+
+
+FLAC__StreamDecoderWriteStatus flacStreamDecoderWriteCallback(
+					const FLAC__StreamDecoder *
+								/*_decoder*/,
+					const FLAC__Frame * _frame,
+					const FLAC__int32 * const _buffer[],
+					void * _client_data )
+{
+	if( _frame->header.channels != 2 )
+	{
+		printf( "channels != 2 in "
+					"flacStreamDecoderWriteCallback()\n" );
+		return( FLAC__STREAM_DECODER_WRITE_STATUS_ABORT );
+	}
+
+	if( _frame->header.bits_per_sample != 16 )
+	{
+		printf( "bits_per_sample != 16 in "
+					"flacStreamDecoderWriteCallback()\n" );
+		return( FLAC__STREAM_DECODER_WRITE_STATUS_ABORT );
+	}
+
+	const Uint32 frames = _frame->header.blocksize;
+	for( Uint32 frame = 0; frame < frames; ++frame )
+	{
+		sampleFrame sframe = { _buffer[0][frame] /
+						OUTPUT_SAMPLE_MULTIPLIER,
+					_buffer[1][frame] /
+						OUTPUT_SAMPLE_MULTIPLIER
+					} ;
+		static_cast<flacStreamDecoderClientData *>(
+					_client_data )->write_buffer->write(
+				(const char *) sframe, sizeof( sframe ) );
+	} 
+	return( FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE );
+}
+
+
+void flacStreamDecoderMetadataCallback( const FLAC__StreamDecoder *,
+					const FLAC__StreamMetadata *,
+					void * /*_client_data*/ )
+{
+	printf("stream decoder metadata callback\n");
+/*	QBuffer * b = static_cast<QBuffer *>( _client_data );
+	b->seek( 0 );
+	b->write( (const char *) _metadata, sizeof( *_metadata ) );*/
+}
+
+
+void flacStreamDecoderErrorCallback( const FLAC__StreamDecoder *,
+					FLAC__StreamDecoderErrorStatus _status,
+					void * /*_client_data*/ )
+{
+	printf("error callback! %d\n", _status);
+	// what to do now??
+}
+
+#endif
+
+
+void sampleBuffer::loadFromBase64( const QString & _data )
+{
+#ifdef QT4
+	QByteArray orig_data = QByteArray::fromBase64( _data.toAscii() );
+#else
+	const char * src = _data.ascii();
+	csize ssize = _data.length();
+	csize dsize = ( _data.length() * 3 ) / 4;
+	char * dst = new char[dsize]; 
+
+	// code mostly taken from
+	// qt-x11-opensource-src-4.0.1/src/corelib/tools/qbytearray.cpp
+	unsigned int buf = 0;
+	int nbits = 0;
+	int offset = 0;
+
+	for( csize i = 0; i < ssize; ++i )
+	{
+		int ch = src[i];
+		int d;
+
+		if( ch >= 'A' && ch <= 'Z' )
+		{
+			d = ch - 'A';
+		}
+		else if( ch >= 'a' && ch <= 'z' )
+		{
+			d = ch - 'a' + 26;
+		}
+		else if( ch >= '0' && ch <= '9' )
+		{
+			d = ch - '0' + 52;
+		}
+		else if( ch == '+' )
+		{
+			d = 62;
+		}
+		else if( ch == '/' )
+		{
+			d = 63;
+		}
+		else
+		{
+			d = -1;
+		}
+
+		if( d != -1 )
+		{
+			buf = ( buf << 6 ) | (Uint32)d;
+			nbits += 6;
+			if( nbits >= 8 )
+			{
+				nbits -= 8;
+				dst[offset++] = buf >> nbits;
+				buf &= ( 1 << nbits ) - 1;
+			}
+		}
+	}
+	QByteArray orig_data;
+	orig_data.setRawData( dst, dsize );
+#endif
+#ifdef HAVE_FLAC_STREAM_DECODER_H
+
+#ifdef QT4
+	QBuffer ba_reader( &orig_data );
+	ba_reader.open( QBuffer::ReadOnly );
+#else
+	QBuffer ba_reader( orig_data );
+	ba_reader.open( IO_ReadOnly );
+#endif
+
+	QBuffer ba_writer;
+#ifdef QT4
+	ba_writer.open( QBuffer::WriteOnly );
+#else
+	ba_writer.open( IO_WriteOnly );
+#endif
+
+	flacStreamDecoderClientData cdata = { &ba_reader, &ba_writer } ;
+
+	FLAC__StreamDecoder * flac_dec = FLAC__stream_decoder_new();
+
+	FLAC__stream_decoder_set_read_callback( flac_dec,
+					flacStreamDecoderReadCallback );
+	FLAC__stream_decoder_set_write_callback( flac_dec,
+					flacStreamDecoderWriteCallback );
+	FLAC__stream_decoder_set_error_callback( flac_dec,
+					flacStreamDecoderErrorCallback );
+	FLAC__stream_decoder_set_metadata_callback( flac_dec,
+					flacStreamDecoderMetadataCallback );
+	FLAC__stream_decoder_set_client_data( flac_dec, &cdata );
+
+	FLAC__stream_decoder_init( flac_dec );
+
+	FLAC__stream_decoder_process_until_end_of_stream( flac_dec );
+
+	FLAC__stream_decoder_finish( flac_dec );
+	FLAC__stream_decoder_delete( flac_dec );
+
+	orig_data = ba_writer.buffer();
+	printf("%d\n", (int) orig_data.size() );
+#endif
+	m_origFrames = orig_data.size() / sizeof( sampleFrame ); 
+	delete[] m_origData;
+	m_origData = new sampleFrame[m_origFrames];
+	memcpy( m_origData, orig_data.data(), orig_data.size() );
+	update();
+#ifndef QT4
+//	delete[] dst;
+#endif
+}
+
+
+
 
 void sampleBuffer::setStartFrame( Uint32 _s )
 {
@@ -1009,6 +1437,11 @@ void sampleBuffer::deleteResamplingData( void * * _ptr )
 }
 
 
+#undef write
+#undef read
+#undef seek
+#undef pos
+
 
 #include "sample_buffer.moc"
 
