@@ -2,8 +2,9 @@
  * channel_track.cpp - implementation of channel-track-class
  *                     (window + data-structures)
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -56,11 +57,12 @@
 
 
 #include "channel_track.h"
-#include "sound_generator.h"
+#include "instrument.h"
 #include "song_editor.h"
 #include "effect_board.h"
 #include "envelope_tab_widget.h"
 #include "arp_and_chords_tab_widget.h"
+#include "midi_tab_widget.h"
 #include "embed.h"
 #include "note_play_handle.h"
 #include "paths.h"
@@ -69,15 +71,17 @@
 #include "pixmap_button.h"
 #include "knob.h"
 #include "mmp.h"
-#include "midi_device.h"
+#include "midi_client.h"
+#include "midi_port.h"
 #include "pattern.h"
 #include "config_mgr.h"
-#include "tab_bar.h"
 #include "debug.h"
 #include "lcd_spinbox.h"
 #include "buffer_allocator.h"
 #include "tooltip.h"
-
+#include "tab_widget.h"
+#include "string_pair_drag.h"
+#include "audio_port.h"
 
 
 const char * volume_help = QT_TRANSLATE_NOOP( "channelTrack",
@@ -94,26 +98,37 @@ const char * surroundarea_help = QT_TRANSLATE_NOOP( "channelTrack",
 						"feature." );
 
 
-const int CHANNEL_WIDTH			= 250;
-const int SOUND_GEN_PLUGIN_WIDTH	= CHANNEL_WIDTH;
-const int SOUND_GEN_PLUGIN_HEIGHT	= SOUND_GEN_PLUGIN_WIDTH;
-const int PIANO_HEIGHT			= 84;
+const int CHANNEL_WIDTH		= 250;
+const int INSTRUMENT_WIDTH	= CHANNEL_WIDTH;
+const int INSTRUMENT_HEIGHT	= INSTRUMENT_WIDTH;
+const int PIANO_HEIGHT		= 84;
 
 
 
 channelTrack::channelTrack( trackContainer * _tc ) :
 	QWidget( lmmsMainWin::inst()->workspace() ),
  	track( _tc ),
-	midiDevice( this ),
+	midiEventProcessor(),
 	m_trackType( CHANNEL_TRACK ),
+	m_midiPort( mixer::inst()->getMIDIClient()->addPort( this,
+						tr( "unnamed_channel" ) ) ),
+	m_audioPort( new audioPort( tr( "unnamed_channel" ) ) ),
 	m_baseTone( A ),
 	m_baseOctave( OCTAVE_4 ),
-	m_soundGenPluginWidget( NULL )
+	m_instrument( NULL )
 {
+	for( int i = 0; i < NOTES_PER_OCTAVE * OCTAVES; ++i )
+	{
+		m_notes[i] = NULL;
+	}
+
+
 #ifdef QT4
 	lmmsMainWin::inst()->workspace()->addWindow( this );
 #endif
 
+	setAcceptDrops( TRUE );
+
 	hide();
 
 	getTrackWidget()->setFixedHeight( 32 );
@@ -167,23 +182,25 @@ channelTrack::channelTrack( trackContainer * _tc ) :
 	setFocusPolicy( StrongFocus );
 #endif
 	QVBoxLayout * vlayout = new QVBoxLayout( this );
+	vlayout->setMargin( 0 );
+	vlayout->setSpacing( 0 );
+
+	m_generalSettingsWidget = new tabWidget( tr( "GENERAL SETTINGS" ),
+									this );
+	m_generalSettingsWidget->setFixedHeight( 90 );
 
 	// setup line-edit for changing channel-name
-	m_channelNameLE = new QLineEdit( this );
+	m_channelNameLE = new QLineEdit( m_generalSettingsWidget );
 	m_channelNameLE->setFont( pointSize<8>( m_channelNameLE->font() ) );
+	m_channelNameLE->setGeometry( 10, 16, 230, 18 );
 	connect( m_channelNameLE, SIGNAL( textChanged( const QString & ) ),
 				this, SLOT( textChanged( const QString & ) ) );
 
 
-	m_generalSettingsWidget = new QWidget( this );
-	QHBoxLayout * gsw_layout = new QHBoxLayout( m_generalSettingsWidget );
-	gsw_layout->setSpacing( 15 );
-
-
-
 	// setup volume-knob
 	m_volumeKnob = new knob( knobBright_26, m_generalSettingsWidget,
 						tr( "Channel volume" ) );
+	m_volumeKnob->move( 10, 44 );
 	m_volumeKnob->setRange( MIN_VOLUME, MAX_VOLUME, 1.0f );
 	m_volumeKnob->setValue( DEFAULT_VOLUME, TRUE );
 	m_volumeKnob->setHintText( tr( "Channel volume:" ) + " ", "%" );
@@ -201,6 +218,7 @@ channelTrack::channelTrack( trackContainer * _tc ) :
 
 	// setup surround-area
 	m_surroundArea = new surroundArea( m_generalSettingsWidget );
+	m_surroundArea->move( 20 + m_volumeKnob->width(), 38 );
 	m_surroundArea->show();
 #ifdef QT4
 	m_surroundArea->setWhatsThis(
@@ -221,12 +239,16 @@ channelTrack::channelTrack( trackContainer * _tc ) :
 						m_generalSettingsWidget );
 	m_effectChannelNumber->setValue( DEFAULT_EFFECT_CHANNEL );
 	m_effectChannelNumber->setLabel( tr( "FX CHNL" ) );
-
+	m_effectChannelNumber->move( m_surroundArea->x() +
+					m_surroundArea->width() + 16, 40 );
 
 
 	m_saveSettingsBtn = new QPushButton( embed::getIconPixmap(
-							"project_save" ),
-						"", m_generalSettingsWidget );
+							"project_save" ), "",
+						m_generalSettingsWidget );
+	m_saveSettingsBtn->setGeometry( m_effectChannelNumber->x() +
+					m_effectChannelNumber->width() + 20, 40,
+					32, 32 );
 	connect( m_saveSettingsBtn, SIGNAL( clicked() ), this,
 					SLOT( saveSettingsBtnClicked() ) );
 	toolTip::add( m_saveSettingsBtn, 
@@ -241,59 +263,37 @@ channelTrack::channelTrack( trackContainer * _tc ) :
 			"double-clicking it in the preset-browser." ) );
 
 
-	gsw_layout->addSpacing( 5 );
-	gsw_layout->addWidget( m_volumeKnob );
-	gsw_layout->addWidget( m_surroundArea );
-	gsw_layout->addWidget( m_effectChannelNumber );
-	gsw_layout->addWidget( m_saveSettingsBtn );
-	gsw_layout->addSpacing( 5 );
-
-
-
 	setVolume( DEFAULT_VOLUME );
 	setSurroundAreaPos( QPoint() );
-	setName( "Default" );
+	setName( tr( "Default" ) );
 
 
-	m_tabBar = new tabBar( this );
-	m_tabBar->setFixedSize( CHANNEL_WIDTH, 24 );
+	m_tabWidget = new tabWidget( "", this );
+	m_tabWidget->setFixedHeight( INSTRUMENT_HEIGHT + 12 );
 
-	m_widgetStack = new QWidget( this );
-	m_widgetStack->setFixedSize( SOUND_GEN_PLUGIN_WIDTH,
-						SOUND_GEN_PLUGIN_HEIGHT );
-	connect( m_tabBar, SIGNAL( allWidgetsHidden() ), m_widgetStack,
-							SLOT( hide() ) );
-	connect( m_tabBar, SIGNAL( widgetShown() ), m_widgetStack,
-							SLOT( show() ) );
-	connect( m_tabBar, SIGNAL( allWidgetsHidden() ), this,
-							SLOT( adjustSize() ) );
-	connect( m_tabBar, SIGNAL( widgetShown() ), this,
-							SLOT( adjustSize() ) );
 
 	// create other tab-widgets
-	m_envWidget = new envelopeTabWidget( this, m_widgetStack );
-	m_arpWidget = new arpAndChordsTabWidget( this, m_widgetStack );
-	m_tabBar->addTab( m_envWidget, tr( "ENV/LFO/FILTER" ), 1 );
-	m_tabBar->addTab( m_arpWidget, tr( "ARP/CHORD" ), 2 );
+	m_envWidget = new envelopeTabWidget( this );
+	m_arpWidget = new arpAndChordsTabWidget( this );
+	m_midiWidget = new midiTabWidget( this, m_midiPort );
+	m_tabWidget->addTab( m_envWidget, tr( "ENV/LFO/FILTER" ), 1 );
+	m_tabWidget->addTab( m_arpWidget, tr( "ARP/CHORD" ), 2 );
+	m_tabWidget->addTab( m_midiWidget, tr( "MIDI" ), 3 );
 
-	// set window-icon
-	setWindowIcon( embed::getIconPixmap( "channel_track" ) );
-
-	
-	// setup m_piano-widget
+	// setup piano-widget
 	m_pianoWidget = new pianoWidget( this );
 	m_pianoWidget->setFixedSize( CHANNEL_WIDTH, PIANO_HEIGHT );
 
-	vlayout->addSpacing( 5 );
-	vlayout->addWidget( m_channelNameLE );
-	vlayout->addSpacing( 5 );
+
 	vlayout->addWidget( m_generalSettingsWidget );
-	vlayout->addSpacing( 5 );
-	vlayout->addWidget( m_tabBar );
-	vlayout->addWidget( m_widgetStack );
+	vlayout->addWidget( m_tabWidget );
 	vlayout->addWidget( m_pianoWidget );
 
 
+	// set window-icon
+	setWindowIcon( embed::getIconPixmap( "channel_track" ) );
+
+
 	_tc->updateAfterTrackAdd();
 }
 
@@ -305,13 +305,8 @@ channelTrack::channelTrack( trackContainer * _tc ) :
 channelTrack::~channelTrack()
 {
 	invalidateAllMyNPH();
-
-	// make sure active midi-device isn't using pointer to this
-	// channel-track-instance anymore
-	if( mixer::inst()->getMIDIDevice()->getChannelTrack() == this )
-	{
-		mixer::inst()->getMIDIDevice()->setChannelTrack( NULL );
-	}
+	delete m_audioPort;
+	mixer::inst()->getMIDIClient()->removePort( m_midiPort );
 }
 
 
@@ -336,7 +331,7 @@ void channelTrack::saveSettingsBtnClicked( void )
 	{
 		QDir().mkdir( preset_root );
 	}
-	if( !QDir( preset_root+pluginName() ).exists() )
+	if( !QDir( preset_root + pluginName() ).exists() )
 	{
 		QDir( preset_root ).mkdir( pluginName() );
 	}
@@ -377,9 +372,9 @@ float channelTrack::frequency( notePlayHandle * _n ) const
 Uint32 channelTrack::beatLen( notePlayHandle * _n ) const
 {
 #ifdef LMMS_DEBUG
-	assert( m_soundGenPluginWidget != NULL );
+	assert( m_instrument != NULL );
 #endif
-	Uint32 len = m_soundGenPluginWidget->beatLen( _n );
+	Uint32 len = m_instrument->beatLen( _n );
 	if( len > 0 )
 	{
 		return( len );
@@ -392,7 +387,7 @@ Uint32 channelTrack::beatLen( notePlayHandle * _n ) const
 
 
 
-void channelTrack::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
+void channelTrack::processAudioBuffer( sampleFrame * _buf, Uint32 _frames,
 							notePlayHandle * _n )
 {
 	// we may not play the sound if this channelTrack is muted...
@@ -400,13 +395,109 @@ void channelTrack::processAudioBuffer( sampleFrame * _ab, Uint32 _frames,
 	{
 		return;
 	}
-	m_envWidget->processAudioBuffer( _ab, _frames, _n );
+	float v_scale = (float) getVolume() / DEFAULT_VOLUME;
+
+	// instruments using instrument-play-handles will call this method
+	// without any knowledge about notes, so they pass NULL for _n, whicj
+	// is no problem for us since we just bypass the envelopes+LFO's
+	if( _n != NULL )
+	{
+		m_envWidget->processAudioBuffer( _buf, _frames, _n );
+		v_scale *= ( (float) _n->getVolume() / DEFAULT_VOLUME );
+	}
 
-	float v_scale = ( (float) getVolume() / DEFAULT_VOLUME ) *
-				( (float) _n->getVolume() / DEFAULT_VOLUME );
 	volumeVector v = m_surroundArea->getVolumeVector( v_scale );
 
-	mixer::inst()->addBuffer( _ab, _frames, _n->framesAhead(), v );
+	mixer::inst()->bufferToPort( _buf, _frames,
+				( _n != NULL ) ? _n->framesAhead() : 0, v,
+								m_audioPort );
+}
+
+
+
+
+void channelTrack::processInEvent( const midiEvent & _me,
+							const midiTime & _time )
+{
+	switch( _me.m_type )
+	{
+		case NOTE_ON: 
+			if( _me.velocity() > 0 )
+			{
+				if( m_notes[_me.key()] == NULL )
+				{
+					// create temporary note
+					note n;
+					n.setKey( _me.key() );
+					n.setVolume( _me.velocity() * 100 /
+									127 );
+					// create (timed) note-play-handle
+					notePlayHandle * nph = new
+						notePlayHandle( this,
+			_time.frames( songEditor::inst()->framesPerTact() ),
+								~0, &n );
+					mixer::inst()->addPlayHandle( nph );
+					m_notes[_me.key()] = nph;
+				}
+				break;
+			}
+
+		case NOTE_OFF:
+			if( m_notes[_me.key()] != NULL )
+			{
+				notePlayHandle * n = m_notes[_me.key()];
+				// create dummy-note which has the same length
+				// as the played note for sending it later
+				// to all slots connected to signal noteDone()
+				// this is for example needed by piano-roll for
+				// recording notes into a pattern
+				note done_note( midiTime( static_cast<Uint32>(
+					n->totalFramesPlayed() * 64 /
+					songEditor::inst()->framesPerTact() ) ),
+					0, n->tone(), n->octave(),
+					n->getVolume(), n->getPanning() );
+				n->noteOff();
+				m_notes[_me.key()] = NULL;
+				emit noteDone( done_note );
+			}
+			break;
+
+		case KEY_PRESSURE:
+			if( m_notes[_me.key()] != NULL )
+			{
+				m_notes[_me.key()]->setVolume( _me.velocity() *
+								100 / 128 );
+			}
+			break;
+
+/*		case PITCH_BEND:
+			if( m_pitchBendKnob != NULL )
+			{
+				float range = tAbs(
+						m_pitchBendKnob->maxValue() -
+						m_pitchBendKnob->minValue() );
+				m_pitchBendKnob->setValue(
+						m_pitchBendKnob->minValue() +
+						_me.m_data.m_param[0] *
+								range / 16384 );
+			}
+			break;*/
+
+		default:
+			printf( "channel-track: unhandled MIDI-event %d\n",
+								_me.m_type );
+			break;
+	}
+}
+
+
+
+
+void channelTrack::processOutEvent( const midiEvent & _me,
+							const midiTime & _time )
+{
+	// if appropriate, midi-port does futher routing
+	m_midiPort->processOutEvent( _me, _time );
 }
 
 
@@ -421,10 +512,10 @@ void channelTrack::playNote( notePlayHandle * _n )
 	if( _n->arpBaseNote() == FALSE )
 	{
 #ifdef LMMS_DEBUG
-		assert( m_soundGenPluginWidget != NULL );
+		assert( m_instrument != NULL );
 #endif
 		// all is done, so now lets play the note!
-		m_soundGenPluginWidget->playNote( _n );
+		m_instrument->playNote( _n );
 	}
 }
 
@@ -433,10 +524,12 @@ void channelTrack::playNote( notePlayHandle * _n )
 
 const QString & channelTrack::pluginName( void ) const
 {
-#ifdef LMMS_DEBUG
-	assert( m_soundGenPluginWidget != NULL );
-#endif
-	return( m_soundGenPluginWidget->pluginName() );
+	if( m_instrument != NULL )
+	{
+		return( m_instrument->publicName() );
+	}
+	static const QString empty = "";
+	return( empty );
 }
 
 
@@ -445,9 +538,9 @@ const QString & channelTrack::pluginName( void ) const
 void channelTrack::deleteNotePluginData( notePlayHandle * _n )
 {
 #ifdef LMMS_DEBUG
-	assert( m_soundGenPluginWidget != NULL );
+	assert( m_instrument != NULL );
 #endif
-	m_soundGenPluginWidget->deleteNotePluginData( _n );
+	m_instrument->deleteNotePluginData( _n );
 }
 
 
@@ -477,6 +570,8 @@ void channelTrack::setName( const QString & _new_name )
 	assert( m_tswChannelButton != NULL );
 #endif
 	m_tswChannelButton->setText( m_name );
+	m_midiPort->setName( m_name );
+	m_audioPort->setName( m_name );
 }
 
 
@@ -496,7 +591,7 @@ void channelTrack::setVolume( volume _new_volume )
 
 volume channelTrack::getVolume( void ) const
 {
-	return( (volume) m_volumeKnob->value() );
+	return( static_cast<volume>( m_volumeKnob->value() ) );
 }
 
 
@@ -525,7 +620,6 @@ const QPoint & channelTrack::surroundAreaPos( void ) const
 
 
 
-
 void channelTrack::setBaseTone( tones _new_tone )
 {
 	if( _new_tone >= C && _new_tone <= H )
@@ -619,11 +713,12 @@ bool FASTCALL channelTrack::play( const midiTime & _start, Uint32 _start_frame,
 							cur_start.getTact() *
 							frames_per_tact ),
 						_frames );
-			mixer::inst()->addBuffer( buf, _frames, _frame_base +
+			mixer::inst()->bufferToPort( buf, _frames, _frame_base +
 				static_cast<Uint32>(
 					p->startPosition().getTact64th() *
 							frames_per_tact /
-								64.0f ), v );
+								64.0f ),
+							v, m_audioPort );
 			bufferAllocator::free( buf );
 			continue;
 		}
@@ -668,18 +763,37 @@ bool FASTCALL channelTrack::play( const midiTime & _start, Uint32 _start_frame,
 				const Uint32 note_frames =
 					cur_note->length().frames(
 							frames_per_tact );
+
+/*				// generate according MIDI-events
+				processOutEvent( midiEvent( NOTE_ON,
+						m_midiPort->outputChannel(),
+							cur_note->key(),
+							cur_note->getVolume() *
+								128 / 100 ),
+						midiTime::fromFrames(
+							frames_ahead,
+							frames_per_tact ) );
+
+				processOutEvent( midiEvent( NOTE_OFF,
+						m_midiPort->outputChannel(),
+							cur_note->key(), 0 ),
+						midiTime::fromFrames(
+							frames_ahead +
+								note_frames,
+							frames_per_tact ) );*/
+
 				notePlayHandle * note_play_handle =
 					new notePlayHandle( this,
 								frames_ahead,
 								note_frames,
 								cur_note );
 				note_play_handle->play();
-				// could we play all within
-				// FRAMES_PER_AUDIO_BUFFER??
+				// could we play all within current number of
+				// frames per audio-buffer?
 				if( note_play_handle->done() == FALSE )
 				{
 					// no, then insert it into
-					// note-play-handle-vector of mixer
+					// play-handle-vector of mixer
 					mixer::inst()->addPlayHandle(
 							note_play_handle );
 				}
@@ -725,14 +839,16 @@ void channelTrack::saveTrackSpecificSettings( QDomDocument & _doc,
 					m_effectChannelNumber->value() ) );
 	ct_de.setAttribute( "basetone", QString::number( m_baseTone ) );
 	ct_de.setAttribute( "baseoct", QString::number( m_baseOctave ) );
-	ct_de.setAttribute( "tab", QString::number( m_tabBar->activeTab() ) );
+	ct_de.setAttribute( "tab", QString::number(
+						m_tabWidget->activeTab() ) );
 
 #ifdef LMMS_DEBUG
-	assert( m_soundGenPluginWidget != NULL );
+	assert( m_instrument != NULL );
 #endif
-	m_soundGenPluginWidget->saveSettings( _doc, ct_de );
+	m_instrument->saveSettings( _doc, ct_de );
 	m_envWidget->saveSettings( _doc, ct_de );
 	m_arpWidget->saveSettings( _doc, ct_de );
+	m_midiWidget->saveSettings( _doc, ct_de );
 
 	_parent.appendChild( ct_de );
 
@@ -775,26 +891,32 @@ void channelTrack::loadTrackSpecificSettings( const QDomElement & _this )
 			{
 				m_arpWidget->loadSettings( node.toElement() );
 			}
+			else if( m_midiWidget->nodeName() == node.nodeName() )
+			{
+				m_midiWidget->loadSettings( node.toElement() );
+			}
 			else
 			{
-				delete m_soundGenPluginWidget;
-				m_soundGenPluginWidget =
-					soundGenerator::pluginInstance(
+				// if node-name doesn't match any known one,
+				// we assume that it is an instrument-plugin
+				// which we'll try to load
+				delete m_instrument;
+				m_instrument = instrument::instantiate(
 							node.nodeName(), this );
-				if( m_soundGenPluginWidget->nodeName() ==
+				if( m_instrument->nodeName() ==
 							node.nodeName() )
 				{
-					m_soundGenPluginWidget->loadSettings(
+					m_instrument->loadSettings(
 							node.toElement() );
 				}
-				m_tabBar->addTab( m_soundGenPluginWidget,
+				m_tabWidget->addTab( m_instrument,
 							tr( "PLUGIN" ), 0 );
 			}
 		}
 		node = node.nextSibling();
         }
 
-	m_tabBar->setActiveTab( tab );
+	m_tabWidget->setActiveTab( tab );
 
 	//mixer::inst()->play();
 
@@ -805,23 +927,19 @@ void channelTrack::loadTrackSpecificSettings( const QDomElement & _this )
 
 
 
-void channelTrack::loadPluginSettings( const QDomElement & _plugin_settings )
+instrument * channelTrack::loadInstrument( const QString & _plugin_name )
 {
-	//mixer::inst()->pause();
-
 	invalidateAllMyNPH();
 
-	delete m_soundGenPluginWidget;
-	m_soundGenPluginWidget = soundGenerator::pluginInstance(
-					_plugin_settings.nodeName(), this );
-	if( m_soundGenPluginWidget->nodeName() == _plugin_settings.nodeName() )
-	{
-		m_soundGenPluginWidget->loadSettings( _plugin_settings );
-	}
-	m_tabBar->addTab( m_soundGenPluginWidget, tr( "PLUGIN" ), 0 );
-	m_tabBar->setActiveTab( 0 );
+	delete m_instrument;
+	m_instrument = instrument::instantiate( _plugin_name, this );
 
-	//mixer::inst()->play();
+	m_tabWidget->addTab( m_instrument, tr( "PLUGIN" ), 0 );
+	m_tabWidget->setActiveTab( 0 );
+
+	m_tswChannelButton->update();
+
+	return( m_instrument );
 }
 
 
@@ -873,18 +991,10 @@ void channelTrack::toggledChannelButton( bool _on )
 
 
 
-void channelTrack::noteDone( const note & _n )
-{
-	emit noteFromMidiDeviceDone( _n );
-}
-
-
-
-
 void channelTrack::closeEvent( QCloseEvent * _ce )
 {
-	_ce->ignore ();
-	hide ();
+	_ce->ignore();
+	hide();
 	m_tswChannelButton->setChecked( FALSE );
 }
 
@@ -899,6 +1009,37 @@ void channelTrack::focusInEvent( QFocusEvent * )
 
 
 
+void channelTrack::dragEnterEvent( QDragEnterEvent * _dee )
+{
+	stringPairDrag::processDragEnterEvent( _dee, "instrument,presetfile" );
+}
+
+
+
+
+void channelTrack::dropEvent( QDropEvent * _de )
+{
+	QString type = stringPairDrag::decodeKey( _de );
+	QString value = stringPairDrag::decodeValue( _de );
+	if( type == "instrument" )
+	{
+		loadInstrument( value );
+		songEditor::inst()->setModified();
+		_de->accept();
+	}
+	else if( type == "presetfile" )
+	{
+		multimediaProject mmp( value );
+		loadTrackSpecificSettings( mmp.content().firstChild().
+								toElement() );
+		songEditor::inst()->setModified();
+		_de->accept();
+	}
+}
+
+
+
+
 void channelTrack::invalidateAllMyNPH( void )
 {
 	// note-play-handles check track-type to determine whether their
@@ -914,6 +1055,68 @@ void channelTrack::invalidateAllMyNPH( void )
 
 
 
+
+
+
+
+
+
+channelButton::channelButton( channelTrack * _channel_track ) :
+	QPushButton( _channel_track->getTrackSettingsWidget() ),
+	m_channelTrack( _channel_track )
+{
+	setAcceptDrops( TRUE );
+}
+
+
+
+
+channelButton::~channelButton()
+{
+}
+
+
+
+
+void channelButton::drawButtonLabel( QPainter * _p )
+{
+	QString pn = m_channelTrack->pluginName() + ":";
+	int extra = isChecked() ? -1 : -3;
+	_p->setFont( pointSize<7>( _p->font() ) );
+#ifdef QT4
+	_p->setPen( QApplication::palette().buttonText().color() );
+#else
+	_p->setPen( QApplication::palette().color( QPalette::Normal,
+						QColorGroup::ButtonText ) );
+#endif
+	_p->drawText( ( width() - QFontMetrics( _p->font() ).width( pn ) ) / 2 +
+					extra, height() / 2 + extra, pn );
+	_p->setPen( QColor( 0, 0, 0 ) );
+	_p->drawText( ( width() - QFontMetrics( _p->font() ).width( text() ) ) /
+				2 + extra, height() / 2 +
+				QFontMetrics( _p->font() ).height() + extra,
+								text() );
+}
+
+
+
+
+void channelButton::dragEnterEvent( QDragEnterEvent * _dee )
+{
+	m_channelTrack->dragEnterEvent( _dee );
+}
+
+
+
+
+void channelButton::dropEvent( QDropEvent * _de )
+{
+	m_channelTrack->dropEvent( _de );
+	setChecked( TRUE );
+}
+
+
+
 #undef setChecked
 #undef isChecked
 
