@@ -1,8 +1,9 @@
 /*
  * tooltip.cpp - namespace toolTip, a tooltip-wrapper for LMMS
  *
- * Linux MultiMedia Studio
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
+ * Copyright (c) 2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * 
+ * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -50,10 +51,3 @@ void toolTip::add( QWidget * _w, const QString & _txt )
 	}
 }
 
-
-/*
-void toolTip::remove( QWidget * _w )
-{
-	QToolTip::remove( _w );
-}
-*/
